package org.gridschool.unicore.plugins.hello;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 *  Main class for plug-in example
 */
public class HelloPlugin extends TaskPlugable {

	/**
	 *  Return a new container for this plugin
	 *
	 *@param  parentContainer  job that this task belongs to
	 *@return                  new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {
		// build new plugin container
		HelloContainer container = new HelloContainer(parentContainer);
		container.setName("New_" + getName() + counter);

		// count instance
		counter++;

		return container;
	}

	/**
	 * Gets the path to the task icon  
	 * The super classes will do the icon handling.
	 *
	 *@return    path to the plugin icon
	 */
	public String getIconPath() {
		return "org/gridschool/unicore/plugins/hello/hello.gif";
	}

	/**
	 *  Get name for plugin task application
	 *
	 *@return    The name value
	 */
	public String getName() {
		return "HelloWorld";
	}

	/**
	 *  Build a new panel that manages the parameters of a plugin container
	 *
	 *@param  container  ActionContainer that the panel belongs to
	 *@return            panel that represents parameters in job preparation area
	 */
	public JPAPanel getPanelInstance(ActionContainer container) {
		// return new instance of plugin JPAPanel
		return new HelloJPAPanel(getClient(), (HelloContainer) container);
	}

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return "Grid School Example: Hello World Plugin\n";
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		// nothing to do here for our example
	}

	/**
	 *  Called when terminating
	 */
	public void stopPlugin() {
		// nothing to do here for our example
	}

}
