/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.ogsa.services.caservice;

import java.rmi.RemoteException;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.description.OperationDesc;
import org.apache.axis.description.ParameterDesc;

import com.pallas.ogsa.util.OGSIStub;
import com.pallas.ogsa.util.ObjectWrapperBean;

/**
 *
 *@author     Ralf Ratering
 *@version    $Id: CAServiceStub.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class CAServiceStub extends OGSIStub implements ICAService {
	public CAServiceStub() throws AxisFault {
		this(null);
	}

	public CAServiceStub(java.net.URL endpointURL, Service service)
		throws AxisFault {
		this(service);
		super.cachedEndpoint = endpointURL;
	}

	public CAServiceStub(Service service) throws AxisFault {
		if (service == null) {
			super.service = new org.apache.axis.client.Service();
		} else {
			super.service = service;
		}
		init();
		setPortName("ICAService");
	}

	public ObjectWrapperBean getCACertificate() throws RemoteException {
		if (super.cachedEndpoint == null) {
			throw new org.apache.axis.NoEndPointException();
		}
		Call call = createCall();
		call.setOperation((OperationDesc) operations.get("getCACertificate"));
		call.setOperationName(
			new QName(
				"http://caservice.services.ogsa.pallas.com",
				"getCACertificate"));

		Object resp = call.invoke(new Object[0]);
		return getResponse(call, resp);
	}


	protected void init() {

		operations.clear();

		Class wrapperClass = ObjectWrapperBean.class;

		// consign
		OperationDesc desc = new OperationDesc();
		desc.setName("getCACertificate");
		desc.setReturnType(
			new QName("http://util.ogsa.pallas.com", "ObjectWrapperBean"));
		desc.setReturnClass(wrapperClass);
		desc.setReturnQName(new QName("", "getCACertificateReturn"));
		desc.setStyle(org.apache.axis.enum.Style.RPC);
		desc.setUse(org.apache.axis.enum.Use.LITERAL);

		operations.put(desc.getName(), desc);

		desc = new OperationDesc();
		desc.setName("signRequest");

		ParameterDesc param =
			new ParameterDesc(
				new QName("", "csr"),
				ParameterDesc.IN,
				new QName("http://www.w3.org/2001/XMLSchema", "base64Binary"));
		desc.addParameter(param);

		desc.setReturnType(
			new QName("http://util.ogsa.pallas.com", "ObjectWrapperBean"));
		desc.setReturnClass(wrapperClass);
		desc.setReturnQName(new QName("", "signRequestReturn"));
		desc.setStyle(org.apache.axis.enum.Style.RPC);
		desc.setUse(org.apache.axis.enum.Use.LITERAL);

		operations.put(desc.getName(), desc);

	}

	public ObjectWrapperBean signRequest(byte[] csr) throws RemoteException {
		if (super.cachedEndpoint == null) {
			throw new org.apache.axis.NoEndPointException();
		}
		Call call = createCall();
		call.setOperation((OperationDesc) operations.get("signRequest"));
		call.setOperationName(
			new QName(
				"http://caservice.services.ogsa.pallas.com",
				"signRequest"));

		Object resp = call.invoke(new Object[] { csr });
		return getResponse(call, resp);
	}

}
