/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.ogsa.util;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Stub;

/**
 * @author Ralf Ratering
 * @version $Id: OGSIStub.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public abstract class OGSIStub extends Stub {

	protected Hashtable operations = new Hashtable();

	protected Call createCall() throws RemoteException {
		try {
			Call call = (Call) super.service.createCall();
			if (super.maintainSessionSet) {
				call.setMaintainSession(super.maintainSession);
			}
			if (super.cachedUsername != null) {
				call.setUsername(super.cachedUsername);
			}
			if (super.cachedPassword != null) {
				call.setPassword(super.cachedPassword);
			}
			if (super.cachedEndpoint != null) {
				call.setTargetEndpointAddress(super.cachedEndpoint);
			}
			if (super.cachedTimeout != null) {
				call.setTimeout(super.cachedTimeout);
			}
			if (super.cachedPortName != null) {
				call.setPortName(super.cachedPortName);
			}
			Enumeration keys = super.cachedProperties.keys();
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				call.setProperty(key, super.cachedProperties.get(key));
			}
			// All the type mapping information is registered
			// when the first call is made.
			// The type mapping information is actually registered in
			// the TypeMappingRegistry of the service, which
			// is the reason why registration is only needed for the first call.
			synchronized (this) {
				if (firstCall()) {
					// must set encoding style before registering serializers
					call.setEncodingStyle(null);
					OGSITypeRegistration.registerAll(call);
				}
			}
			// init stuff
			call.setUseSOAPAction(true);
			call.setSOAPActionURI("");
			call.setEncodingStyle(null);
			call.setProperty(Call.SEND_TYPE_ATTR, Boolean.FALSE);
			call.setProperty(org.apache.axis.AxisEngine.PROP_DOMULTIREFS,
					Boolean.FALSE);
			call
					.setSOAPVersion(org.apache.axis.soap.SOAPConstants.SOAP11_CONSTANTS);
			setRequestHeaders(call);
			setAttachments(call);
			return call;
		}
		catch (Throwable t) {
			throw new AxisFault("Failure trying to get the Call object", t);
		}
	}

	protected ObjectWrapperBean getResponse(Call call, Object resp)
			throws RemoteException {
		try {
			System.err.println("--------- Server Response -------");
			call.getMessageContext().getResponseMessage().writeTo(System.err);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		if (resp instanceof RemoteException) {
			throw (RemoteException) resp;
		}
		else {
			getResponseHeaders(call);
			extractAttachments(call);
			try {
				return (ObjectWrapperBean) resp;
			}
			catch (Exception exception) {
				return (ObjectWrapperBean) org.apache.axis.utils.JavaUtils
						.convert(resp, ObjectWrapperBean.class);
			}
		}
	}
}