/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.ogsa.util;

import java.util.HashMap;
import java.util.Iterator;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.rpc.encoding.TypeMappingRegistry;
import org.apache.axis.client.Call;
import org.apache.axis.encoding.DeserializerFactory;
import org.apache.axis.encoding.SerializerFactory;
import org.apache.axis.encoding.ser.ArrayDeserializerFactory;
import org.apache.axis.encoding.ser.ArraySerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.gridforum.www.ogsi.HandleType;
import org.gridforum.www.ogsi.LocatorType;
import org.gridforum.www.ogsi.ReferenceType;
import com.pallas.ogsa.extensions.ServiceReferenceType;

/**
 * @author Ralf Ratering
 * @version $Id: OGSITypeRegistration.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class OGSITypeRegistration {

	private static HashMap mappings = new HashMap();
	public static String OGSI_EXTENSIONS_NAMESPACE = "http://extensions.ogsa.pallas.com";
	public static String OGSI_NAMESPACE = "http://ogsi.www.gridforum.org";
	public static String UTIL_NAMESPACE = "http://util.ogsa.pallas.com";
	static {
		init();
	}

	public static OGSIMapping get(Class cls) {
		return (OGSIMapping) mappings.get(cls);
	}

	private static void init() {
		// register OGSI classes
		Class cls = LocatorType.class;
		QName qname = new QName(OGSI_NAMESPACE, "LocatorType");
		register(cls, qname, new BeanSerializerFactory(cls, qname),
				new BeanDeserializerFactory(cls, qname));
		qname = new QName(OGSI_NAMESPACE, "ArrayOf_LocatorType");
		register(LocatorType[].class, qname, new ArraySerializerFactory(),
				new ArrayDeserializerFactory());
		cls = HandleType.class;
		qname = new QName(OGSI_NAMESPACE, "HandleType");
		register(cls, qname, new BeanSerializerFactory(cls, qname),
				new BeanDeserializerFactory(cls, qname));
		cls = ReferenceType.class;
		qname = new QName(OGSI_NAMESPACE, "ReferenceType");
		register(cls, qname, new BeanSerializerFactory(cls, qname),
				new BeanDeserializerFactory(cls, qname));
		cls = ServiceReferenceType.class;
		qname = new QName(OGSI_EXTENSIONS_NAMESPACE, "ServiceReferenceType");
		register(cls, qname, new BeanSerializerFactory(cls, qname),
				new BeanDeserializerFactory(cls, qname));
		// register wrapper bean
		cls = ObjectWrapperBean.class;
		qname = new QName(UTIL_NAMESPACE, "ObjectWrapperBean");
		register(cls, qname, new BeanSerializerFactory(cls, qname),
				new BeanDeserializerFactory(cls, qname));
	}

	/**
	 * Description of the Method
	 * 
	 * @param cls
	 *            Description of the Parameter
	 * @param qname
	 *            Description of the Parameter
	 * @param serializerFactory
	 *            Description of the Parameter
	 * @param deserializerFactory
	 *            Description of the Parameter
	 */
	public static void register(Class cls, QName qname,
			SerializerFactory serializerFactory,
			DeserializerFactory deserializerFactory) {
		mappings.put(cls, new OGSIMapping(cls, qname, serializerFactory,
				deserializerFactory));
	}

	public static void registerAll(Call call) {
		Iterator iterator = mappings.keySet().iterator();
		while (iterator.hasNext()) {
			OGSIMapping mapping = (OGSIMapping) mappings.get(iterator.next());
			call.registerTypeMapping(mapping.getMappedClass(), mapping
					.getQName(), mapping.getSerializerFactory(), mapping
					.getDeserializerFactory());
		}
	}

	public static void registerAll(TypeMappingRegistry registry) {
		Iterator iterator = mappings.keySet().iterator();
		while (iterator.hasNext()) {
			TypeMapping typeMapping = registry.getDefaultTypeMapping();
			OGSIMapping mapping = (OGSIMapping) mappings.get(iterator.next());
			typeMapping.register(mapping.getMappedClass(), mapping.getQName(),
					mapping.getSerializerFactory(), mapping
							.getDeserializerFactory());
		}
	}

	private OGSITypeRegistration() {
	}
}