/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.ogsa.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Ralf Ratering
 * @version $Id: ObjectWrapperBean.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class ObjectWrapperBean implements Serializable {

	private static Logger logger = Logger.getLogger("com.pallas.ogsa.util");
	private byte[] bytes;
	private Object equalsCalc;
	private boolean hashCodeCalc = false;

	public ObjectWrapperBean() {
	}

	public ObjectWrapperBean(Object object) {
		serialize(object);
	}

	public Object deserialize() {
		try {
			ObjectInputStream stream = new ObjectInputStream(
					new ByteArrayInputStream(bytes));
			Object object = stream.readObject();
			return object;
		}
		catch (Exception e) {
			logger.log(Level.WARNING, "Could not deserialize object.", e);
		}
		return null;
	}

	public synchronized boolean equals(Object obj) {
		if (!(obj instanceof ObjectWrapperBean)) {
			return false;
		}
		ObjectWrapperBean other = (ObjectWrapperBean) obj;
		if (obj == null) {
			return false;
		}
		if (this == obj) {
			return true;
		}
		if (equalsCalc != null) {
			return (equalsCalc == obj);
		}
		equalsCalc = obj;
		boolean equals = true && ((bytes == null && other.getBytes() == null) || (bytes != null && Arrays
				.equals(bytes, other.getBytes())));
		equalsCalc = null;
		return equals;
	}

	public byte[] getBytes() {
		return bytes;
	}

	public synchronized int hashCode() {
		if (hashCodeCalc) {
			return 0;
		}
		hashCodeCalc = true;
		int hashCode = 1;
		if (getBytes() != null) {
			for (int i = 0; i < Array.getLength(getBytes()); i++) {
				Object obj = Array.get(getBytes(), i);
				if (obj != null && !obj.getClass().isArray()) {
					hashCode += obj.hashCode();
				}
			}
		}
		hashCodeCalc = false;
		return hashCode;
	}

	public void serialize(Object object) {
		try {
			ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
			ObjectOutputStream objectOut = new ObjectOutputStream(byteOut);
			objectOut.writeObject(object);
			objectOut.flush();
			byteOut.flush();
			objectOut.close();
			bytes = byteOut.toByteArray();
		}
		catch (Exception e) {
			// delete previous contents
			bytes = null;
			logger.log(Level.WARNING, "Could not serialize object: " + object,
					e);
		}
	}

	public void setBytes(byte[] bytes) {
		this.bytes = bytes;
	}
}