/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.Home;
import org.unicore.resources.Resource;
import org.unicore.resources.Root;
import org.unicore.resources.Spool;
import org.unicore.resources.StorageServer;
import org.unicore.resources.Temp;
import org.unicore.resources.USpace;

/**
 * @author Ralf Ratering
 * @version $Id: ResourceEntry.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class ResourceEntry {

	public static String getName(Resource resource) {
		if (resource instanceof Home) {
			return "Home";
		}
		else if (resource instanceof USpace) {
			return "Job Directory";
		}
		else if (resource instanceof StorageServer) {
			StorageServer server = (StorageServer) resource;
			return server.getName();
		}
		else if (resource instanceof Spool) {
			return "Spool";
		}
		else if (resource instanceof AlternativeUspace) {
			return "Alternative Job Directory";
		}
		else if (resource instanceof Root) {
			return "Root";
		}
		else if (resource instanceof Temp) {
			return "Temporary";
		}
		else {
			return resource.toString();
		}
	}
	private String entry = "Unknown";
	private Resource resource;

	public ResourceEntry(Resource resource) {
		this.resource = resource;
		entry = getName(resource);
	}

	public boolean equals(Object obj) {
		if (obj instanceof ResourceEntry) {
			return getResource().equals(((ResourceEntry) obj).getResource());
		}
		else {
			return false;
		}
	}

	public Resource getResource() {
		return resource;
	}

	public String toString() {
		return entry;
	}
}