/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import com.pallas.unicore.utility.UserMessages;

/**
 * @author Ralf Ratering
 * @version $Id: AddToKeystoreDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class AddToKeystoreDialog extends JDialog {

	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == detailsButton) {
				showCertificate();
				return;
			}
			else if (e.getSource() == onlyForSessionButton) {
				result = ONLY_FOR_SESSION;
			}
			else if (e.getSource() == acceptButton) {
				result = ACCEPT;
			}
			else if (e.getSource() == rejectButton) {
				result = REJECT;
			}
			dispose();
		}
	}
	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.dialogs");
	public final static int ONLY_FOR_SESSION = 0, ACCEPT = 1, REJECT = 2;
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	private X509Certificate certificate;
	private JButton detailsButton, acceptButton, rejectButton,
			onlyForSessionButton;
	private JFrame detailsFrame;
	private String issuer;
	private URL pluginURL;
	private int result;

	public AddToKeystoreDialog(Frame parent, URL pluginURL,
			X509Certificate certificate) {
		super(parent, res.getString("UNKNOWN_TITLE"), true);
		this.pluginURL = pluginURL;
		this.certificate = certificate;
		String all = certificate.getSubjectDN().toString();
		int in1 = all.indexOf("CN=") + 3;
		issuer = all.substring(in1, all.indexOf(',', in1));
		buildComponents();
		pack();
		setLocationRelativeTo(parent);
	}

	private void buildComponents() {
		JTextArea textArea = new JTextArea();
		String message = res.getString("LOADING_PLUGIN") + pluginURL + ">\n";
		message += res.getString("SIGNED_BY") + issuer + "\n";
		message += res.getString("ADDING_SIGNER");
		textArea.setText(message);
		ButtonListener buttonListener = new ButtonListener();
		detailsButton = new JButton(res.getString("SHOW_DETAILS"));
		detailsButton.setToolTipText(res.getString("SHOW_DETAILS_EXPLICIT"));
		detailsButton.addActionListener(buttonListener);
		onlyForSessionButton = new JButton(res.getString("ACCEPT_FOR_SESSION"));
		onlyForSessionButton.setToolTipText(res.getString("ACCEPT_TIP"));
		onlyForSessionButton.addActionListener(buttonListener);
		acceptButton = new JButton(res.getString("ADD"));
		acceptButton.setToolTipText(res.getString("ADD_TIP"));
		acceptButton.addActionListener(buttonListener);
		rejectButton = new JButton(res.getString("NOT_ACCEPT"));
		rejectButton.setToolTipText(res.getString("NOT_ACCEPT_TIP"));
		rejectButton.addActionListener(buttonListener);
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(detailsButton);
		buttonPanel.add(onlyForSessionButton);
		buttonPanel.add(acceptButton);
		buttonPanel.add(rejectButton);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(textArea, BorderLayout.CENTER);
		// window closing = pressing cancel
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent we) {
				result = REJECT;
				dispose();
			}
		});
	}

	public int getResult() {
		return result;
	}

	private void showCertificate() {
		String subject = "";
		try {
			subject = subject + certificate.getSubjectDN().toString();
			if (subject == null) {
				return;
			}
			int indexFrom = subject.indexOf("CN=");
			if (indexFrom != -1) {
				int indexTo = subject.indexOf(",", indexFrom);
				subject = subject.substring(indexFrom + 3, indexTo);
				System.err.println("*******subject: " + subject);
			}
			if (certificate != null) {
				KeyStoreDetailsDialog dialog = new KeyStoreDetailsDialog(
						certificate, (JFrame) getParent(), subject, true);
				dialog.show();
			}
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "", ex);
			UserMessages.error("Could not read certificate " + subject
					+ " from keystore.", ex.getMessage());
		}
	}
}