/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import org.unicore.Vsite;
import com.pallas.unicore.client.tables.SiteListTable;
import com.pallas.unicore.client.tables.UsiteListTable;
import com.pallas.unicore.client.tables.VsiteListTable;
import com.pallas.unicore.client.util.SiteDefaults;
import com.pallas.unicore.extensions.SiteEntry;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.extensions.UsiteEntry;
import com.pallas.unicore.extensions.VsiteEntry;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Thomas Kentemich
 * @version $Id: ConfigureSitesDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ConfigureSitesDialog extends GenericDialog {

	private JScrollPane urlPanel;
	private JScrollPane usitePanel;
	private JScrollPane vsitePanel;
	private UsiteListTable usiteTable;
	private VsiteListTable vsiteTable;
	private SiteListTable urlTable;
	private SiteDefaults defaults = ResourceManager.getSiteDefaults();

	
	public ConfigureSitesDialog(JFrame parent) {
		super(parent, "Configure Sites", true, GenericDialog.OK_CANCEL);
		buildComponents();
		pack();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.pallas.unicore.client.dialogs.GenericDialog#applyValues()
	 */
	protected boolean applyValues() {
		// write out Usite settingd
		for (int i = 0; i < usiteTable.getRowCount(); i++) {
			defaults.setUsite((Usite) usiteTable.getModel().getValueAt(i, 0),
					((Boolean) usiteTable.getModel().getValueAt(i, 1))
							.booleanValue());
		}
		// .. and the Vsites
		for (int i = 0; i < vsiteTable.getRowCount(); i++) {
			defaults.setVsite((Vsite) vsiteTable.getModel().getValueAt(i, 1),
					((Boolean) vsiteTable.getModel().getValueAt(i, 2))
							.booleanValue());
		}
		// ... and the URLs
		for (int i = 0; i < urlTable.getRowCount(); i++) {
			defaults.setUrl((String) urlTable.getModel().getValueAt(i, 0),
					((Boolean) urlTable.getModel().getValueAt(i, 1))
							.booleanValue());
		}
		defaults.removeDeletedUrls(ResourceManager.getUsiteURLs());
		Vector allUsites = ResourceManager.getAllUsites();
		Vector allVsites = new Vector();
		defaults.removeDeletedUsites(allUsites);
		for (int i = 0; i < allUsites.size(); i++) {
			allVsites.addAll(ResourceManager.getAllVsites((Usite) allUsites
					.elementAt(i)));
		}
		defaults.removeDeletedVsites(allVsites);
		defaults.writeToFile();
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.pallas.unicore.client.dialogs.GenericDialog#cancelValues()
	 */
	protected boolean cancelValues() {
		return true;
	}

	private void buildComponents() {
		buildUrlPanel();
		buildUsitePanel();
		buildVsitePanel();
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(usitePanel, "UNICORE sites");
		tabbedPane.add(vsitePanel, "Virtual sites");
		tabbedPane.add(urlPanel, "Site definitions");
		getContentPane().add(tabbedPane, BorderLayout.CENTER);
	}

	private void buildUrlPanel() {
		urlTable = new SiteListTable();
		Vector urls = ResourceManager.getUsiteURLs();
		Vector urlEntries = new Vector();
		for (int i = 0; i < urls.size(); i++) {
			String next = (String) urls.elementAt(i);
			SiteEntry entry = null;
			if (defaults.isSiteContained(next)) {
				// if the usite is contained in the defaults add it with
				// the value listed in the defaults
				entry = new SiteEntry(next, defaults.getSiteValue(next));
			}
			else {
				entry = new SiteEntry(next, true);
			}
			urlEntries.addElement(entry);
		}
		urlTable.setDataVector(urlEntries);
		urlTable.adjustWidths();
		urlPanel = new JScrollPane(urlTable);
	}

	private void buildUsitePanel() {
		usiteTable = new UsiteListTable();
		Vector usites = ResourceManager.getAllUsites();
		Vector usiteEntries = new Vector();
		for (int i = 0; i < usites.size(); i++) {
			Usite next = (Usite) usites.elementAt(i);
			UsiteEntry entry = null;
			if (defaults.isUsiteContained(next)) {
				// if the usite is contained in the defaults add it with
				// the value listed in the defaults
				entry = new UsiteEntry(next, defaults.getUsiteValue(next));
			}
			else {
				entry = new UsiteEntry(next, true);
			}
			usiteEntries.addElement(entry);
		}
		usiteTable.setDataVector(usiteEntries);
		usiteTable.adjustWidths();
		usitePanel = new JScrollPane(usiteTable);
	}

	private void buildVsitePanel() {
		vsiteTable = new VsiteListTable();
		// get the enabled Usite
		Vector usites = ResourceManager.getAllUsites();
		Vector vsiteEntries = new Vector();
		for (int j = 0; j < usites.size(); j++) {
			Vector vsites = ResourceManager.getAllVsites((Usite) usites
					.elementAt(j));
			for (int i = 0; i < vsites.size(); i++) {
				Vsite next = (Vsite) vsites.elementAt(i);
				VsiteEntry entry = null;
				if (defaults.isVsiteContained(next)) {
					// if the usite is contained in the defaults add it with
					// the value listed in the defaults
					entry = new VsiteEntry((Usite) usites.elementAt(j), next,
							defaults.getVsiteValue(next));
				}
				else {
					entry = new VsiteEntry((Usite) usites.elementAt(j), next,
							true);
				}
				vsiteEntries.addElement(entry);
			}
		}
		vsiteTable.setDataVector(vsiteEntries);
		vsiteTable.adjustWidths();
		vsitePanel = new JScrollPane(vsiteTable);
	}
}