/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

/**
 *  Dialog to login with username and password.
 *
 *@author     Ralf Ratering
 *@version    $Id: LoginDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class LoginDialog extends GenericDialog {
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		LoginDialog dialog = new LoginDialog(frame, "TEST: PasswordDialog");
		dialog.show();
	}
	private char[] password;
	private JPasswordField passwordField = new JPasswordField(15);
	private String username;
	private JTextField usernameField = new JTextField(15);

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title   dialog title
	 *@param  parent  dialog parent - important for modal dialogs
	 */
	public LoginDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);
		buildComponents();
		pack();
	}

	/**
	 *  Takes current textfield entry as password
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		username = usernameField.getText();
		password = passwordField.getPassword();
		return true;
	}

	/**
	 *  Add a password field to the parent dialog
	 */
	private void buildComponents() {
		JLabel usernameLabel = new JLabel(res.getString("USERNAME"));
		JLabel passwordLabel = new JLabel(res.getString("PASSWORD"));

		JPanel usernamePanel = new JPanel();
		usernamePanel.add(usernameLabel);
		usernamePanel.add(usernameField);

		JPanel passwordPanel = new JPanel();
		passwordPanel.add(passwordLabel);
		passwordPanel.add(passwordField);

		JPanel subPanel = new JPanel(new GridLayout(2, 1, 10, 10));
		subPanel.add(usernamePanel);
		subPanel.add(passwordPanel);
		// add some spacing around
		subPanel.setBorder(new EmptyBorder(5, 20, 5, 20));

		getContentPane().add(subPanel, BorderLayout.CENTER);
	}

	/**
	 *  Sets password to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		password = null;
		return true;
	}

	/**
	 *  Get the result from the dialog
	 *
	 *@return    null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getPassword() {
		return password;
	}

	/**
	 *  Get the entered user name
	 *
	 *@return    name as string
	 */
	public String getUsername() {
		return username;
	}

	/**
	 *  Put focus into password field and make okButton default
	 */
	public void show() {
		usernameField.requestFocus();
		super.show();
	}
}
