/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

/**
 * @author Thomas Kentemich
 * @version $Id: ServiceURLDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ServiceURLDialog extends GenericDialog {

	private String text;
	private JTextField textField = new JTextField(40);
	private JLabel textLabel;

	/**
	 * Constructor builds gui and sets a title
	 * 
	 * @param title
	 *            dialog title
	 * @param parent
	 *            dialog parent - important for modal dialogs
	 * @param modal
	 *            true if dialog is modal
	 * @param labelText
	 *            Label
	 */
	public ServiceURLDialog(JFrame parent, String labelText, String title,
			boolean modal) {
		super(parent, title, modal, OK_CANCEL);
		textLabel = new JLabel(labelText);
		buildComponents();
		pack();
	}

	/**
	 * Takes current textfield entry as text
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean applyValues() {
		text = textField.getText();
		return true;
	}

	private void buildComponents() {
		okButton.setEnabled(true);
		JPanel textPanel = new JPanel();
		textPanel.add(textLabel);
		textPanel.add(textField);
		textPanel.setBorder(new EmptyBorder(20, 5, 10, 5));
		getContentPane().add(textPanel, BorderLayout.CENTER);
	}

	/**
	 * Sets text to null
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		text = null;
		return true;
	}

	/**
	 * Enable okButton only when textfield is not empty. Overwrite this method
	 * if you want to allow empty inputs.
	 */
	protected void caretMoved() {
		okButton.setEnabled(true);
	}

	/**
	 * Get the result from the dialog
	 * 
	 * @return null if cancel was pressed and a String if ok was pressed
	 */
	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.textField.setText(text);
	}

	/**
	 * Set label for textfield from outside
	 * 
	 * @param label
	 *            label for textfield
	 */
	public void setLabel(String label) {
		textLabel.setText(label);
	}

	public void show() {
		getRootPane().setDefaultButton(okButton);
		super.show();
	}
}