/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.explorer;

import java.io.File;
import java.util.Date;
import java.util.Vector;

/**
 * @author Ralf Ratering
 * @version $Id: RowData.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class RowData extends Vector implements Comparable {

	private static int compareIndex = 0;

	/**
	 * Sets the sortingIndex attribute of the RowData object
	 * 
	 * @param _compareIndex
	 *            The new compareIndex value
	 */
	public static void setCompareIndex(int _compareIndex) {
		compareIndex = _compareIndex;
	}

	public RowData(File file, String systemName, String description) {
		add(new FileEntry(file, systemName));
		add(new SizeEntry(file.length()));
		// description may be null on UNIX systems
		if (description == null) {
			description = "";
		}
		add(description);
		add(new Date(file.lastModified()));
	}

	public Comparable comparableAt(int i) {
		return (Comparable) elementAt(i);
	}

	public int compareTo(Object object) {
		int value = comparableAt(compareIndex).compareTo(
				((RowData) object).comparableAt(compareIndex));
		// if equal use filenames for second comparison
		if (value == 0 && compareIndex != 0) {
			value = comparableAt(0).compareTo(
					((RowData) object).comparableAt(0));
		}
		return value;
	}

	public File getFile() {
		return ((FileEntry) firstElement()).getFile();
	}
}