/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.util.Vector;

/**
 * Sorts a vector of comparable vertexs belonging to a graph in ascending order.
 * The ordering depends on the deph and the affiliations of their belonging
 * vertexs.
 * 
 * @author Kirsten Foerster
 * @version $Id: TreeSorter.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class TreeSorter {

	private ITreeComparable treeComparable;

	public TreeSorter(ITreeComparable treeComparable) {
		this.treeComparable = treeComparable;
	}

	/**
	 * Sorts a vector of comparable vertexs belonging to a graph in ascending
	 * order.
	 * 
	 * @param unsortVector -
	 *            a unsorted vector
	 * @return - the sorted vector
	 */
	public Vector sort(Vector unsortVector) {
		if (unsortVector.isEmpty()) {
			return null;
		}
		Vector sortVector = new Vector();
		Object element = unsortVector.firstElement();
		sortVector.addElement(element);
		boolean inserted = false;
		int comparison;
		int maxPrioritaet;
		int pos = 0;
		boolean flag = false;
		for (int i = 1; i < unsortVector.size(); i++) {
			element = unsortVector.elementAt(i);
			if (this.treeComparable.noChildrens(element)
					&& this.treeComparable.noParents(element)) {
				sortVector.addElement(element);
				continue;
			}
			maxPrioritaet = 0;
			for (int j = 0; j < sortVector.size(); j++) {
				comparison = this.treeComparable.compareTo(element, sortVector
						.elementAt(j));
				if (comparison == VertexsTreeComparable.IMMEDIATE_INSERTING) {
					sortVector.insertElementAt(element, j);
					inserted = true;
					break;
				}
				if (comparison == VertexsTreeComparable.POSITION_MARKER) {
					if (!flag) {
						pos = j;
						flag = true;
					}
					continue;
				}
				//the computed value of comparison is between
				//NEXT_POSITION_MARKER_LOWEST_PRIORITY and
				//NEXT_POSITION_MAKRER_TOP_PRIORITY
				if (comparison > 0) {
					if (comparison >= maxPrioritaet) {
						pos = j + 1;
						maxPrioritaet = comparison;
						flag = false;
					}
				}
			}
			if (!inserted) {
				sortVector.insertElementAt(element, pos);
			}
			inserted = false;
			flag = false;
			maxPrioritaet = 0;
		}
		return sortVector;
	}
}