/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.container.DoNContainer;

/**
 * @author Ralf Ratering
 * @author Sven Haubold
 * @version $Id: DoNPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class DoNPanel extends GroupDependencyPanel {

	private IntTextField nTextField;

	public DoNPanel(JFrame parentFrame, DoNContainer container) {
		super(parentFrame, container);
	}

	/**
	 * Apply values from gui to JobContainer
	 */
	public void applyValues() {
		DoNContainer doNContainer = (DoNContainer) container;
		doNContainer.setIterations(nTextField.getValue());
		super.applyValues();
	}

	protected JPanel buildControlPanel() {
		nTextField = new IntTextField(1, 0, 10000);
		nTextField.setToolTipText("Repeat the belonging job task n times.");
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		JLabel label = new JLabel("Number of iterations:");
		label.setToolTipText("Repeat the belonging job task n times.");
		panel.add(label);
		panel.add(nTextField);
		JPanel controlPanel = new JPanel();
		controlPanel.add(panel);
		return controlPanel;
	}

	/**
	 * Fill values from JobContainer into gui
	 */
	public void resetValues() {
		DoNContainer doNContainer = (DoNContainer) container;
		nTextField.setValue(doNContainer.getIterations());
		super.resetValues();
	}
}