/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.Date;
import java.util.Vector;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystem;
import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Panel to define file exports from USpace to Nspace or Xspace.
 * 
 * @author Ralf Ratering
 * @version $Id: FileExportPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class FileExportPanel extends FileStoragePanel {

	final static int COL_BINARY = 4;
	final static int COL_OVERWRITE = 3;
	final static int COL_SOURCE = 0;
	final static int COL_STORAGE = 1;
	final static int COL_TARGET = 2;
	private static int count = 0;
	private JComboBox storagesEditorComboBox;
	private ComboBoxCellRenderer storagesRendererComboBox;

	/**
	 * Constructor builds GUI
	 * 
	 * @param container
	 *            container to be modified
	 */
	public FileExportPanel(TaskContainer container) {
		this(container, null);
	}

	public FileExportPanel(TaskContainer container, String initialDirectory) {
		super(container, initialDirectory);
		buildComponents();
	}

	/**
	 * Add and initialize a new empty row to the table
	 */
	protected void addEmptyRow() {
		// get last row
		Vector lastRowData = null;
		int selectedRow = table.getSelectedRow();
		if (selectedRow >= 0) {
			lastRowData = (Vector) tableModel.getDataVector().elementAt(
					selectedRow);
		}
		Vector rowData = new Vector();
		if (lastRowData == null) {
			rowData.add("");
			rowData.add(FileStorage.NSPACE_STRING);
			rowData.add("");
			rowData.add(Boolean.FALSE);
			rowData.add(Boolean.FALSE);
		}
		else {
			rowData.add("");
			rowData.add(lastRowData.elementAt(COL_STORAGE));
			rowData.add("");
			rowData.add(lastRowData.elementAt(COL_OVERWRITE));
			rowData.add(lastRowData.elementAt(COL_BINARY));
		}
		tableModel.addRow(rowData);
		selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 * Apply current GUI entries to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		int nrOfRows = table.getRowCount();
		FileExport[] fileExports = new FileExport[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			fileExports[i] = (FileExport) getFileOperationAt(i);
		}
		updateModifiedTime(fileExports);
		// apply stuff to container
		container.setFileExports(fileExports);
		container.checkContents();
	}

	private void buildComponents() {
		// minimize checkbox and combobox columns
		int storageChoice = 80;
		table.getColumnModel().getColumn(COL_STORAGE).setPreferredWidth(
				storageChoice);
		// table.getColumnModel().getColumn(COL_STORAGE).setMinWidth(storageChoice);
		// table.getColumnModel().getColumn(COL_STORAGE).setMaxWidth(storageChoice);
		int overwriteChoice = 115;
		table.getColumnModel().getColumn(COL_OVERWRITE).setPreferredWidth(
				overwriteChoice);
		// table.getColumnModel().getColumn(COL_OVERWRITE).setMinWidth(overwriteChoice);
		// table.getColumnModel().getColumn(COL_OVERWRITE).setMaxWidth(overwriteChoice);
		int binaryChoice = 55;
		table.getColumnModel().getColumn(COL_BINARY).setPreferredWidth(
				binaryChoice);
		// table.getColumnModel().getColumn(COL_BINARY).setMinWidth(binaryChoice);
		// table.getColumnModel().getColumn(COL_BINARY).setMaxWidth(binaryChoice);
		// build the combobox editors and renderers in tables
		storages = new Vector();
		storages.add(FileStorage.NSPACE_STRING);
		storagesEditorComboBox = new JComboBox(storages);
		storagesRendererComboBox = new ComboBoxCellRenderer(storages);
		table.getColumnModel().getColumn(COL_STORAGE).setCellEditor(
				new DefaultCellEditor(storagesEditorComboBox));
		table.getColumnModel().getColumn(COL_STORAGE).setCellRenderer(
				storagesRendererComboBox);
	}

	protected boolean getBinary(int row) {
		return ((Boolean) tableModel.getValueAt(row, COL_BINARY))
				.booleanValue();
	}

	/**
	 * Gets the columnNames
	 */
	protected String[] getColumnNames() {
		String[] columnNames = {res.getString("USPACE_NAME"),
				res.getString("DESTINATION"), res.getString("NAME_AT_DEST"),
				res.getString("OVERWRITE"), res.getString("BINARY")};
		return columnNames;
	}

	/**
	 * Gets the dataFlavor
	 */
	protected DataFlavor getDataFlavor() {
		return FileExport.getFlavor();
	}

	/**
	 * Build a FileStorage object out of a certain table row
	 * 
	 * @param row
	 *            nr of row
	 * @return new FileStorage object
	 */
	protected FileOperation getFileOperationAt(int row) {
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		Object storageItem = getStorageItem(row);
		String destinationName = (String) tableModel
				.getValueAt(row, COL_TARGET);
		boolean overwrite = getOverwrite(row);
		boolean isAscii = !getBinary(row);
		return new FileExport(container, storageItem, sourceName,
				destinationName, overwrite, isAscii);
	}

	/**
	 * Gets the overwrite attribute for a table row
	 */
	protected boolean getOverwrite(int row) {
		return ((Boolean) tableModel.getValueAt(row, COL_OVERWRITE))
				.booleanValue();
	}

	/**
	 * Gets the storageItem for a table row
	 */
	protected Object getStorageItem(int row) {
		return tableModel.getValueAt(row, COL_STORAGE);
	}

	/**
	 * Reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		clearTable();
		FileExport[] fileExports = container.getFileExports();
		for (int i = 0; i < fileExports.length; i++) {
			addFileOperation(fileExports[i]);
		}
	}

	/**
	 * Fill an existing table row with a certain FileExport object. If row index
	 * is too high add a new row and use that one.
	 * 
	 * @param index
	 *            nr of table row
	 * @param fileOp
	 *            The new fileOperation
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileExport fileExport = (FileExport) fileOp;
		String source = fileExport.getSourceName();
		Boolean overwrite = new Boolean(fileExport.getOverwritePolicy());
		Boolean binary = new Boolean(!fileExport.getFileMode());
		// the table is edited
		if (index < table.getRowCount()) {
			String oldSource = (String) tableModel
					.getValueAt(index, COL_SOURCE);
			String oldTarget = (String) tableModel
					.getValueAt(index, COL_TARGET);
			// if table entry already contains a source name, keep it
			if (!oldSource.equals("")) {
				source = oldSource;
			}
			// if table entry already contains a source or target name, keep the
			// flags
			if (!oldSource.equals("") || !oldTarget.equals("")) {
				overwrite = (Boolean) tableModel.getValueAt(index,
						COL_OVERWRITE);
				binary = (Boolean) tableModel.getValueAt(index, COL_BINARY);
			}
		}
		if (index > table.getRowCount() - 1) {
			tableModel.addRow(new Vector());
		}
		tableModel.setValueAt(source, index, COL_SOURCE);
		tableModel.setValueAt(fileExport.getStorageItem(), index, COL_STORAGE);
		tableModel.setValueAt(fileExport.getDestinationName(), index,
				COL_TARGET);
		tableModel.setValueAt(overwrite, index, COL_OVERWRITE);
		tableModel.setValueAt(binary, index, COL_BINARY);
	}

	/**
	 * Popup the remote file chooser
	 */
	protected void showBrowser() {
		File selectedFile = null;
		if (table.getSelectedRow() >= 0) {
			FileExport fileExport = (FileExport) getFileOperationAt(table
					.getSelectedRow());
			logger.fine("Selected file export: " + fileExport);
			if (fileExport != null) {
				Object storageItem = fileExport.getStorageItem();
				String filename = fileExport.getSourceName();
				if (storageItem.equals(FileStorage.NSPACE_STRING)) {
					selectedFile = new File(filename);
				}
				else {
					selectedFile = new RemoteFile(filename,
							(Storage) storageItem, new RemoteFileSystem(
									container.getVsite()));
				}
			}
		}
		logger.fine("Selected file: " + selectedFile);
		SelectorDialog chooser = ResourceManager.getSelectorDialog(
				selectedFile, container.getVsite());
		chooser.setTitle("UNICORE: Choose destination for export");
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			selectedFile = chooser.getSelectedFile();
			Object storageObject = FileStorage.NSPACE_STRING;
			String path = selectedFile.getPath();
			String name = selectedFile.getName();
			if (selectedFile instanceof RemoteFile) {
				RemoteFile file = (RemoteFile) selectedFile;
				storageObject = file.getStorage();
				path = file.getSubspacePath();
				// always add a slash to path for remote directories
				if (selectedFile.isDirectory()) {
					path += "/";
					name += "/";
				}
			}
			else {
				// for local directories add platform specific separator
				if (selectedFile.isDirectory()) {
					path += File.separator;
					name += File.separator;
				}
			}
			FileExport fileExport = new FileExport(container, storageObject,
					name, path);
			// if we have selected a single row, assume we wanted to edit it
			if (table.getSelectedRowCount() == 1) {
				setFileOperationAt(fileExport, table.getSelectedRow());
			}
			else {
				addFileOperation(fileExport);
			}
		}
	}

	private void updateModifiedTime(FileExport[] newExports) {
		FileExport[] oldExports = container.getFileExports();
		// first a simple check...
		if (oldExports.length != newExports.length) {
			container.setModifiedTime(new Date());
			return;
		}
		// .. now try to find matching exports
		// loop over old exports
		for (int i = 0; i < oldExports.length; i++) {
			boolean found = false;
			// loop over new imports
			for (int j = 0; j < newExports.length; j++) {
				// try to find matching new export
				if (newExports[j].equals(oldExports[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one import was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 * Update storage resource entries
	 */
	public void updateValues() {
		// get storage resources from vsite
		updateStorageResources();
		// replace existing storage resources
		replaceStorageResources(COL_STORAGE);
		for (int i = 0; i < storages.size(); i++) {
			if (storages.elementAt(i) instanceof USpace) {
				storages.remove(storages.elementAt(i));
				break;
			}
		}
		// update combo boxes
		storagesEditorComboBox.setModel(new DefaultComboBoxModel(storages));
		storagesRendererComboBox.setModel(new DefaultComboBoxModel(storages));
	}
}