/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import org.unicore.Vsite;
import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.Spool;
import org.unicore.resources.Storage;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.controls.ResourceEntry;
import com.pallas.unicore.client.controls.StorageComboBox;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystem;
import com.pallas.unicore.container.ControlContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.extensions.FileTest;
import com.pallas.unicore.extensions.TestType;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @author Kirsten Foerster
 * @version $Id: FileTestPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class FileTestPanel extends AbstractTestPanel {

	private JButton browseButton;
	private ControlContainer controlContainer;
	private JTextField filenameField;
	private boolean repeatMode;
	private StorageComboBox storageBox;
	private JLabel testLabel, locationLabel, fileLabel;
	private Vsite vsite;

	/**
	 * Constructor for the FileTestPanel object
	 */
	public FileTestPanel(ControlContainer controlContainer, boolean repeatMode) {
		super(controlContainer);
		this.controlContainer = controlContainer;
		this.repeatMode = repeatMode;
		buildComponents();
		setEnabled(false);
	}

	/**
	 * Apply values from this gui to FileTest.
	 */
	public void applyValues() {
		FileTest test = controlContainer.getFileTest();
		test.setType((TestType) testBox.getSelectedItem());
		test.setSubspacePath(filenameField.getText());
		if (storageBox.getSelectedStorage() instanceof Storage) {
			test.setStorage((Storage) storageBox.getSelectedStorage());
		}
	}

	/**
	 * Build all components of this panel.
	 */
	private void buildComponents() {
		GridBagLayout g = new GridBagLayout();
		setLayout(g);
		GridBagConstraints c = new GridBagConstraints();
		if (repeatMode) {
			testLabel = new JLabel("Repeat until: ");
			testLabel
					.setToolTipText("Repeat this queue until the status of the file is achieved.");
		}
		else {
			testLabel = new JLabel("If: ");
			testLabel
					.setToolTipText("Interrogate the status of the choosen file.");
		}
		c = LayoutTools.makegbc(0, 0, 1, 0, false);
		add(testLabel, c);
		testBox = new JComboBox(FileTest.getTypes());
		if (repeatMode) {
			testBox
					.setToolTipText("Repeat this queue until the status of the file is achieved.");
		}
		else {
			testBox
					.setToolTipText("Interrogate the status of the choosen file.");
		}
		c = LayoutTools.makegbc(1, 0, 2, 100, true);
		add(testBox, c);
		locationLabel = new JLabel("Location: ");
		locationLabel.setToolTipText("Choose a directory");
		c = LayoutTools.makegbc(0, 1, 1, 0, false);
		add(locationLabel, c);
		storageBox = new StorageComboBox();
		storageBox.addItemListener(new StorageBoxListener());
		storageBox.setToolTipText("Choose a directory");
		c = LayoutTools.makegbc(1, 1, 2, 100, true);
		add(storageBox, c);
		fileLabel = new JLabel("Filename: ");
		fileLabel.setToolTipText("the file to be tested");
		c = LayoutTools.makegbc(0, 2, 1, 0, false);
		add(fileLabel, c);
		c = LayoutTools.makegbc(1, 2, 1, 100, true);
		filenameField = new JTextField(13);
		filenameField.setToolTipText("the file to be tested");
		add(filenameField, c);
		browseButton = new JButton(new BrowseAction());
		browseButton.setToolTipText("Choose local file");
		c = LayoutTools.makegbc(2, 2, 1, 0, false);
		c.insets = new Insets(5, 0, 5, 5);
		add(browseButton, c);
		setBorder(new TitledBorder("File Test"));
	}

	/**
	 * Popup file chooser and return the selected directory
	 *  
	 */
	private RemoteFile chooseFile() {
		RemoteFile selectedFile = new RemoteFile(filenameField.getText(),
				(Storage) storageBox.getSelectedStorage(),
				new RemoteFileSystem(vsite));
		SelectorDialog chooser = ResourceManager.getSelectorDialog(
				selectedFile, controlContainer.getVsite());
		chooser.setTitle("UNICORE: Choose test file");
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		//chooser.setLocalEnabled(false);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			return (RemoteFile) chooser.getSelectedFile();
		}
		return null;
	}

	/**
	 * Fill values from FileTest into this gui.
	 */
	public void resetValues() {
		logger.fine("Resetting file test panel " + container);
		FileTest test = controlContainer.getFileTest();
		testBox.setSelectedItem(test.getType());
		updateStorageBox();
		storageBox.setSelectedStorage(test.getStorage());
		filenameField.setText(test.getSubspacePath());
		logger.fine("Selected storage after reset: "
				+ storageBox.getSelectedItem());
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		updateControls();
	}

	private void updateControls() {
		TestType test = (TestType) testBox.getSelectedItem();
		boolean testEnabled = test.isEnabled();
		locationLabel.setEnabled(testEnabled);
		storageBox.setEnabled(testEnabled);
		fileLabel.setEnabled(testEnabled);
		filenameField.setEnabled(testEnabled);
		browseButton.setEnabled(testEnabled);
		if (testEnabled) {
			Object storage = storageBox.getSelectedStorage();
			Object item = storageBox.getSelectedItem();
			if ((null == storage && !(item instanceof ResourceEntry))
					|| item.toString().equals(StorageComboBox.NOT_AVAILABLE)) {
				locationLabel.setEnabled(false);
				storageBox.setEnabled(false);
			}
			if (!storageBox.browsableItem()) {
				browseButton.setEnabled(false);
			}
		}
	}

	private void updateStorageBox() {
		storageBox.update(controlContainer.getVsite());
		// remove unwanted storages
		Vector toRemove = new Vector();
		for (int i = 0; i < storageBox.getItemCount(); i++) {
			if (storageBox.getStorageAt(i) instanceof AlternativeUspace
					|| storageBox.getStorageAt(i) instanceof Spool) {
				toRemove.add(storageBox.getStorageAt(i));
			}
		}
		for (int i = 0; i < toRemove.size(); i++) {
			storageBox.removeStorage((Storage) toRemove.elementAt(i));
		}
		updateControls();
	}

	/**
	 * Update values from this panel.
	 */
	public void updateValues(boolean vsiteChanged) {
		logger.fine("Updating file test panel for " + container);
		if (vsiteChanged) {
			updateStorageBox();
			vsite = controlContainer.getVsite();
			applyValues();
		}
	}

	private class BrowseAction extends AbstractAction {

		public BrowseAction() {
			super("Browse");
		}

		public void actionPerformed(ActionEvent ev) {
			RemoteFile selectedFile = chooseFile();
			if (selectedFile == null) {
				return;
			}
			storageBox.setSelectedStorage(selectedFile.getStorage());
			filenameField.setText(selectedFile.getSubspacePath());
		}
	}

	private class StorageBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			updateControls();
		}
	}

	public static void main(String args[]) {
		JFrame frame = new JFrame("File Test");
		frame.getContentPane().add(
				new FileTestPanel(new DoRepeatContainer(), true),
				BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);
	}
}