/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ResourceBundle;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import com.pallas.unicore.client.controls.FontSelectionComboBox;

/**
 * @author Ralf Ratering
 * @version $Id: FontSelectionPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class FontSelectionPanel extends JPanel {

	private class SelectionListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			firePropertyChange("FONT_CHANGED", null, getSelectedFont());
		}
	}
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.panels.ResourceStrings");

	public static void main(String args[]) {
		JFrame frame = new JFrame("Test font selector");
		frame.getContentPane().add(new FontSelectionPanel());
		frame.pack();
		frame.show();
	}
	private FontSelectionComboBox fontComboBox;
	private SelectionListener listener;
	private JComboBox sizeComboBox;
	private JLabel sizeLabel;

	public FontSelectionPanel() {
		this(false);
	}

	public FontSelectionPanel(boolean preview) {
		fontComboBox = new FontSelectionComboBox(preview);
		sizeComboBox = new JComboBox();
		listener = new SelectionListener();
		buildComponents();
	}

	private void buildComponents() {
		for (int i = 8; i < 33; i++) {
			sizeComboBox.addItem(new Integer(i));
		}
		fontComboBox.addItemListener(listener);
		sizeComboBox.addItemListener(listener);
		JPanel subPanel = new JPanel();
		subPanel.add(fontComboBox);
		sizeLabel = new JLabel(res.getString("SIZE"));
		subPanel.add(sizeLabel);
		subPanel.add(sizeComboBox);
		add(subPanel);
	}

	public FontSelectionComboBox getFontComboBox() {
		return this.fontComboBox;
	}

	public Font getSelectedFont() {
		String fontName = (String) fontComboBox.getSelectedItem();
		Integer size = (Integer) sizeComboBox.getSelectedItem();
		return new Font(fontName, Font.PLAIN, size.intValue());
	}

	public JComboBox getSizeComboBox() {
		return this.sizeComboBox;
	}

	public JLabel getSizeLabel() {
		return this.sizeLabel;
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		fontComboBox.setEnabled(enabled);
		sizeComboBox.setEnabled(enabled);
	}

	public void setSelectedFont(Font font) {
		fontComboBox.setSelectedItem(font.getFontName());
		sizeComboBox.setSelectedItem(new Integer(font.getSize()));
		repaint();
	}
}