/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import com.pallas.unicore.client.graphs.DependencyEditor;
import com.pallas.unicore.client.trees.JPATree;
import com.pallas.unicore.container.ControlContainer;
import com.pallas.unicore.container.ElseContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.ThenContainer;
import com.pallas.unicore.extensions.TestType;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Ralf Ratering
 * @author Sven Haubold
 * @version $Id: IfPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class IfPanel extends GroupPanel {

	class ArrangeAction extends AbstractAction {

		private DependencyEditor editor;

		public ArrangeAction(DependencyEditor editor) {
			super(res.getString("ARRANGE_GRAPH"));
			this.editor = editor;
		}

		public void actionPerformed(ActionEvent event) {
			editor.arrangeGraph(80, 80);
			// update graph
			editor.updateValues();
			// update panel size to make scrollbars appear
			editor.revalidate();
		}
	}

	private class SelectionListener implements ItemListener {

		public void itemStateChanged(ItemEvent ev) {
			if (ev.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			JComboBox testBox = (JComboBox) ev.getSource();
			TestType test = (TestType) testBox.getSelectedItem();
			if (ev.getSource() == returnCodeTestPanel.getTestBox()) {
				returnCodeTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					fileTestPanel.setEnabled(false);
					timeTestPanel.setEnabled(false);
				}
			}
			else if (ev.getSource() == fileTestPanel.getTestBox()) {
				fileTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					returnCodeTestPanel.setEnabled(false);
					timeTestPanel.setEnabled(false);
				}
			}
			else if (ev.getSource() == timeTestPanel.getTestBox()) {
				timeTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					returnCodeTestPanel.setEnabled(false);
					fileTestPanel.setEnabled(false);
				}
			}
		}
	}
	private Color dependencyBackground = new Color(0, 100, 200);
	private FileTestPanel fileTestPanel;
	private ReturnCodeTestPanel returnCodeTestPanel;
	private JSplitPane rightSplitPane;
	private DependencyEditor thenDeps, elseDeps;
	private TimeTestPanel timeTestPanel;

	public IfPanel(JFrame parentFrame, IfContainer container) {
		super(parentFrame, container);
	}

	/**
	 * Apply values from gui to JobContainer
	 */
	public void applyValues() {
		fileTestPanel.applyValues();
		timeTestPanel.applyValues();
		returnCodeTestPanel.applyValues();
		thenDeps.applyValues();
		elseDeps.applyValues();
		super.applyValues();
	}

	protected void buildComponents() {
		ThenContainer thenContainer = null;
		ElseContainer elseContainer = null;
		Vector tasks = container.getTasks();
		for (int i = 0; i < tasks.size(); i++) {
			if (tasks.elementAt(i) instanceof ThenContainer) {
				thenContainer = (ThenContainer) tasks.elementAt(i);
			}
			else if (tasks.elementAt(i) instanceof ElseContainer) {
				elseContainer = (ElseContainer) tasks.elementAt(i);
			}
		}
		thenDeps = new DependencyEditor(thenContainer);
		elseDeps = new DependencyEditor(elseContainer);
		JPanel thenPanel = buildDependencyPanel(thenDeps,
				"Then Group Dependencies");
		JPanel elsePanel = buildDependencyPanel(elseDeps,
				"Else Group Dependencies");
		rightSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, thenPanel,
				elsePanel);
		// use absolute value here, cause nothing else seemed to work
		rightSplitPane.setDividerLocation(300);
		JSplitPane mainSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				new JScrollPane(buildControlPanel()), rightSplitPane);
		add(buildNamePanel(), BorderLayout.NORTH);
		add(mainSplitPane, BorderLayout.CENTER);
	}

	protected JPanel buildControlPanel() {
		JPanel panel = new JPanel(new GridLayout(3, 1, 10, 10));
		ControlContainer controlContainer = (ControlContainer) container;
		returnCodeTestPanel = new ReturnCodeTestPanel(controlContainer, false);
		fileTestPanel = new FileTestPanel(controlContainer, false);
		timeTestPanel = new TimeTestPanel(controlContainer, false);
		SelectionListener selectionListener = new SelectionListener();
		returnCodeTestPanel.getTestBox().addItemListener(selectionListener);
		fileTestPanel.getTestBox().addItemListener(selectionListener);
		timeTestPanel.getTestBox().addItemListener(selectionListener);
		setTitledBorder("If Then Else");
		panel.add(returnCodeTestPanel);
		panel.add(fileTestPanel);
		panel.add(timeTestPanel);
		return panel;
	}

	protected JPanel buildDependencyPanel(DependencyEditor editor, String title) {
		editor.setColor(Color.white);
		editor.setSelectionColor(Color.green);
		JScrollPane dependencyScrollPane = new JScrollPane(editor);
		dependencyScrollPane.setViewportView(editor);
		JButton arrangeButton = new JButton(new ArrangeAction(editor));
		JPanel dependencyControlPanel = new JPanel();
		dependencyControlPanel.add(arrangeButton);
		JPanel dependencyPanel = new JPanel(new BorderLayout());
		dependencyPanel.setBorder(new TitledBorder(title));
		dependencyPanel.add(dependencyScrollPane, BorderLayout.CENTER);
		dependencyPanel.add(dependencyControlPanel, BorderLayout.SOUTH);
		return dependencyPanel;
	}

	/**
	 * Fill values from JobContainer into gui
	 */
	public void resetValues() {
		returnCodeTestPanel.resetValues();
		fileTestPanel.resetValues();
		timeTestPanel.resetValues();
		thenDeps.resetValues();
		elseDeps.resetValues();
		super.resetValues();
	}

	public void updateValues(boolean vsiteChanged) {
		super.updateValues(vsiteChanged);
		fileTestPanel.updateValues(vsiteChanged);
		returnCodeTestPanel.updateValues(vsiteChanged);
		thenDeps.updateValues();
		elseDeps.updateValues();
		elseDeps.setBackground(dependencyBackground.darker());
		thenDeps.setBackground(dependencyBackground.darker());
		JPATree tree = ResourceManager.getCurrentInstance().getJPATree();
		if (tree.getCurrentGroupNode().getActionContainer() instanceof ElseContainer) {
			elseDeps.setBackground(dependencyBackground);
		}
		else if (tree.getCurrentGroupNode().getActionContainer() instanceof ThenContainer) {
			thenDeps.setBackground(dependencyBackground);
		}
	}
}