/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import com.pallas.unicore.container.FileImportContainer;

/**
 * JPAPanel to define a file import from Nspace or Xspace into Uspace This panel
 * can be plugged into the clients data area and can be connected to a jpa tree
 * node
 * 
 * @author Ralf Ratering
 * @version $Id: ImportJPAPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class ImportJPAPanel extends JPAPanel {

	// text field for task name
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.panels.ResourceStrings");
	// container to be modified
	private FileImportContainer container;
	// panel containing file imports
	private FileImportPanel importPanel;
	private JTextField nameTextField;

	public ImportJPAPanel(JFrame parentFrame, FileImportContainer container) {
		super(parentFrame, container);
		this.container = container;
		buildComponents();
	}

	/**
	 * Overwrite abstract method to apply the current GUI values to the
	 * container
	 */
	public void applyValues() {
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		importPanel.applyValues();
	}

	/**
	 * Build GUI
	 */
	private void buildComponents() {
		importPanel = new FileImportPanel(container);
		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ENTER_IDENTIFIER"));
		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel(res.getString("NAME")));
		namePanel.add(nameTextField);
		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(namePanel, BorderLayout.NORTH);
		subPanel.add(importPanel, BorderLayout.CENTER);
		setTitledBorder(res.getString("IMPORT_BORDER"));
		add(subPanel, BorderLayout.CENTER);
		revalidate();
	}

	/**
	 * Overwrite abstract method to reset current GUI entries to last container
	 * contents.
	 */
	public void resetValues() {
		nameTextField.setText(container.getName());
		importPanel.resetValues();
	}

	/**
	 * Overwrite abstract method that will be called whenever the panel becomes
	 * visible
	 * 
	 * @param vsiteChanged
	 *            true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		importPanel.updateValues();
	}
}