/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;

import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.dialogs.ShowLogDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.resourcemanager.DetailsEntry;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorSorter;

/**
 * Panel to display job output from job monitor
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @author Klaus-Dieter Oertel
 * @version $Id: OutcomePanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class OutcomePanel extends JPanel {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.panels");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.panels.ResourceStrings");
	private static int selectedIndex = 0;
	private Vector additionalOutcomePanels = new Vector();
	// container to get outcome from
	private ActionContainer container;
	private OutcomeDetailsTable detailsTable;
	private AjoViewPanel ajoView;
	private Font fixedFont;
	// parent frame
	private Client parentFrame;
	// text areas
	private JTextArea stdoutArea, stderrArea, detailsArea;
	// some buttons
	private JButton stdoutSaveButton, stderrSaveButton;
	private JTabbedPane tabbedPane;

	/**
	 * Constructor builds gui
	 * 
	 * @param container
	 *            to get outcome from
	 */
	public OutcomePanel(ActionContainer container) {
		this.container = container;
		buildComponents();
		tabbedPane.getModel().setSelectedIndex(selectedIndex);
	}

	/**
	 * Constructor builds gui
	 * 
	 * @param container
	 *            to get outcome from
	 * @param parent
	 *            container
	 */
	public OutcomePanel(Client parent, ActionContainer container) {
		this.container = container;
		this.parentFrame = parent;
		buildComponents();
		tabbedPane.getModel().setSelectedIndex(selectedIndex);
	}

	private void appendToTextArea(File file, JTextArea area) {
		// task did not produce stderr/stdout
		if (file == null) {
			return;
		}
		// file should be there but is not...
		if (!file.exists()) {
			logger.warning("Could not read outcome file: "
					+ file.getAbsolutePath());
			return;
		}
		try {
			DefaultEditorKit kit = new DefaultEditorKit();
			Document doc = area.getDocument();
			Reader reader = new FileReader(file);
			try {
				kit.read(reader, doc, doc.getLength());
				logger.finer("Reading outcome file: " + file.getAbsolutePath()
						+ " successful.");
			}
			catch (BadLocationException e) {
				logger.log(Level.SEVERE, "Error appending to text area", e);
			}
		}
		catch (IOException e) {
			logger.log(Level.WARNING, "Could not read outcome file", e);
		}
	}

	/**
	 * Build GUI
	 */
	private void buildComponents() {
		//
		// stdout panel
		//
		stdoutArea = new JTextArea();
		JScrollPane stdoutScrollPane = new JScrollPane(stdoutArea);
		stdoutSaveButton = new JButton(res.getString("SAVE"));
		stdoutSaveButton.addActionListener(new ButtonListener());
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(stdoutSaveButton);
		JPanel stdoutPanel = new JPanel(new BorderLayout());
		stdoutPanel.add(stdoutScrollPane, BorderLayout.CENTER);
		stdoutPanel.add(buttonPanel, BorderLayout.SOUTH);
		//
		// stderr panel
		//
		stderrArea = new JTextArea();
		JScrollPane stderrScrollPane = new JScrollPane(stderrArea);
		stderrSaveButton = new JButton(res.getString("SAVE"));
		stderrSaveButton.addActionListener(new ButtonListener());
		JPanel stderrButtonPanel = new JPanel();
		stderrButtonPanel.add(stderrSaveButton);
		JPanel stderrPanel = new JPanel(new BorderLayout());
		stderrPanel.add(stderrScrollPane, BorderLayout.CENTER);
		stderrPanel.add(stderrButtonPanel, BorderLayout.SOUTH);
		// details panel
		detailsArea = new JTextArea();
		JScrollPane detailsTextScrollPane = new JScrollPane(detailsArea);
		detailsTextScrollPane.setBorder(new TitledBorder("Summary"));
		String tooltip = "Double click on row will display textual overview for "
				+ "selected entry.";
		detailsTable = new OutcomeDetailsTable();
		detailsTable.setToolTipText(tooltip);
		detailsTable.addMouseListener(new DetailsTableListener());
		JScrollPane detailsTableScrollPane = new JScrollPane(detailsTable);
		detailsTableScrollPane.setBorder(new TitledBorder("History"));
		detailsTableScrollPane.setToolTipText(tooltip);
		JPanel detailsPanel = new JPanel(new BorderLayout());
		detailsPanel.add(detailsTextScrollPane, BorderLayout.NORTH);
		detailsPanel.add(detailsTableScrollPane, BorderLayout.CENTER);
		detailsPanel.setToolTipText(tooltip);
		// ajo view
		ajoView = new AjoViewPanel(container.getActionGroup());
		tabbedPane = new JTabbedPane();
		tabbedPane.add(stdoutPanel, res.getString("STDOUT"));
		tabbedPane.add(stderrPanel, res.getString("STDERR"));
		tabbedPane.add(detailsPanel, res.getString("DETAILS"));
		tabbedPane.add(ajoView, "AJO view");
		// add additional outcome panels from plugins
		if (container instanceof IPanelProvider) {
			IPanelProvider panelProvider = (IPanelProvider) container;
			additionalOutcomePanels.clear();
			for (int i = 0; i < panelProvider.getNrOfPanels(); i++) {
				tabbedPane.add(new JScrollPane(panelProvider.getPanel(i)),
						panelProvider.getPanelTitle(i));
				if (panelProvider.getPanel(i) instanceof Applyable) {
					additionalOutcomePanels.add(panelProvider.getPanel(i));
				}
			}
		}
		tabbedPane.getModel().addChangeListener(new TabbedPaneListener());
		setLayout(new BorderLayout());
		add(tabbedPane, BorderLayout.CENTER);
	}

	/**
	 * Gets the selected tabbed pane index
	 */
	public int getSelectedIndex() {
		return selectedIndex;
	}

	/**
	 * Fill outcome into text areas
	 */
	public void resetValues() {
		// if font has changed set it to back to a monospaced one
		if (fixedFont != stdoutArea.getFont()) {
			updateFont();
		}
		stdoutArea.setText("");
		stderrArea.setText("");
		Vector sortedEntries = OutcomeManager.getOutcomeEntries(container
				.getActionGroup());
		VectorSorter.sort(sortedEntries);
		for (int i = 0; i < sortedEntries.size(); i++) {
			OutcomeEntry entry = (OutcomeEntry) sortedEntries.elementAt(i);
			appendToTextArea(entry.getStdoutFile(), stdoutArea);
			appendToTextArea(entry.getStderrFile(), stderrArea);
		}
		Vector details = OutcomeManager.getDetails(container);
		VectorSorter.sort(details);
		detailsArea.setText(container.getName() + ": " + container.getStatus());
		if (container.getStatus() != null
				&& container.getStatus().isEquivalent(
						AbstractActionStatus.NOT_SUCCESSFUL)) {
			for (int i = 0; i < details.size(); i++) {
				DetailsEntry entry = (DetailsEntry) details.elementAt(i);
				if (entry.getStatus().isEquivalent(
						AbstractActionStatus.NOT_SUCCESSFUL)
						&& !entry.getReason().equals("")) {
					detailsArea.append("\nReason: " + entry.getReason());
				}
			}
		}
		detailsTable.clear();
		detailsTable.setDataVector(details);
		for (int i = 0; i < additionalOutcomePanels.size(); i++) {
			Applyable panel = (Applyable) additionalOutcomePanels.elementAt(i);
			panel.resetValues();
		}
		ajoView.resetValues();
	}

	/**
	 * Save stdout or stderr to file
	 * 
	 * @param text
	 *            text from stdout or stderr area
	 * @param title
	 *            title of file chooser dialog
	 */
	private void saveToFile(String text, String title) {
		SelectorDialog d = ResourceManager.getSelectorDialog(new File(
				ResourceManager.getUserDefaults().getLogDirectory()), null);
		d.setTitle(title);
		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			try {
				PrintWriter pw = new PrintWriter(new BufferedWriter(
						new FileWriter(file)));
				pw.println(text);
				pw.close();
			}
			catch (IOException ioe) {
				UserMessages.error("Saving file " + filename + " failed.", ioe
						.getMessage());
			}
		}
	}

	private void updateFont() {
		Font font = ResourceManager.getUserDefaults().getTheme()
				.getControlTextFont();
		int fontSize = font.getSize();
		fixedFont = new Font("monospaced", Font.PLAIN, fontSize);
		stdoutArea.setFont(fixedFont);
		stderrArea.setFont(fixedFont);
		detailsTable.setFont(new Font(font.getName(), font.getStyle(), 10));
	}

	public void updateValues() {
		if (tabbedPane.getTabCount() > selectedIndex) {
			tabbedPane.getModel().setSelectedIndex(selectedIndex);
		}
		else {
			tabbedPane.getModel()
					.setSelectedIndex(tabbedPane.getTabCount() - 1);
		}
	}

	/**
	 * Listener class listens to save button events
	 */
	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == stdoutSaveButton) {
				saveToFile(stdoutArea.getText(), res.getString("SAVE_OUT"));
			}
			else if (event.getSource() == stderrSaveButton) {
				saveToFile(stderrArea.getText(), res.getString("SAVE_ERR"));
			}
		}
	}

	/**
	 * Listen to details table clicks
	 */
	private class DetailsTableListener extends MouseAdapter {

		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() >= 2) {
				DetailsEntry entry = detailsTable.getSelectedEntry();
				ShowLogDialog logDialog = new ShowLogDialog(ResourceManager
						.getCurrentInstance(), detailsTable.getSelectedEntry()
						.toString(), "UNICORE: Details about "
						+ entry.getContainer());
				Font font = logDialog.getFont();
				logDialog.setTextFont(new Font("monospaced", Font.PLAIN, font
						.getSize()));
				logDialog.setSize(800, 250);
				logDialog.setEditable(false);
				logDialog.setLocationRelativeTo(ResourceManager
						.getCurrentInstance());
				logDialog.setVisible(true);
			}
		}
	}

	/**
	 * Listen to tabbed pane events
	 *  
	 */
	private class TabbedPaneListener implements ChangeListener {

		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == tabbedPane.getModel()) {
				selectedIndex = tabbedPane.getModel().getSelectedIndex();
				for (int i = 0; i < additionalOutcomePanels.size(); i++) {
					Applyable panel = (Applyable) additionalOutcomePanels
							.elementAt(i);
					panel.updateValues();
				}
			}
		}
	}
}