/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Vector;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;
import org.unicore.resources.CapacityResource;
import org.unicore.resources.InformationResource;
import org.unicore.resources.Memory;
import org.unicore.resources.NumericInfoResource;
import org.unicore.resources.PerformanceResource;
import org.unicore.resources.SoftwareResource;
import org.unicore.resources.TextInfoResource;
import com.pallas.unicore.client.controls.ResourceEntry;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.TableSorter;
import com.pallas.unicore.extensions.NamedResourceSet;

/**
 * Panel to display information about a ResourceSet of a vsite.
 * 
 * @author Ralf Ratering
 * @version $Id: ResourceInfoPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class ResourceInfoPanel extends JPanel {

	/**
	 * Table model for non editable default tables
	 *  
	 */
	protected class InfoTableModel extends DefaultTableModel {

		public InfoTableModel(String[] columnNames, int numRows) {
			super(columnNames, numRows);
		}

		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	/**
	 * The main program for the ResourceInfoPanel class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		NamedResourceSet siteResources = new NamedResourceSet("bla");
		siteResources.add(new Memory("Memory", 100.0, 50.0, 10.0));
		ResourceInfoPanel resourceInformationPanel = new ResourceInfoPanel(
				siteResources);
		JFrame app = new JFrame();
		app.getContentPane().add(resourceInformationPanel);
		app.pack();
		app.show();
	}
	/**
	 * resources to be displayed
	 */
	protected NamedResourceSet resourceSet;

	public ResourceInfoPanel(NamedResourceSet resourceSet) {
		this.resourceSet = resourceSet;
		buildComponents();
	}

	/**
	 * Build gui
	 */
	protected void buildComponents() {
		Vector informations = resourceSet.addInformationResources(new Vector());
		Vector capacities = resourceSet.addCapacityResources(new Vector());
		Vector storages = resourceSet.addStorageResources(new Vector());
		Vector performances = resourceSet.addPerformanceResources(new Vector());
		Vector applications = resourceSet.addApplicationResources(new Vector());
		Vector contexts = resourceSet.addContextResources(new Vector());
		Vector softwares = resourceSet.addSoftwareResources(new Vector());
		// remove storages from capacity list
		for (int i = 0; i < storages.size(); i++) {
			capacities.remove(storages.elementAt(i));
		}
		// remove applications and contexts from software list
		for (int i = 0; i < applications.size(); i++) {
			softwares.remove(applications.elementAt(i));
		}
		for (int i = 0; i < contexts.size(); i++) {
			softwares.remove(contexts.elementAt(i));
		}
		JPanel subPanel = new JPanel();
		subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
		if (!informations.isEmpty()) {
			JTextArea infoArea = getInfoArea(informations);
			JPanel infoPanel = new JPanel(new BorderLayout());
			infoPanel.setBorder(new TitledBorder("Information"));
			infoPanel.add(infoArea, BorderLayout.CENTER);
			subPanel.add(infoPanel);
		}
		if (!applications.isEmpty()) {
			AdjustTable applicationTable = getSoftwareTable(applications);
			applicationTable.adjustWidths();
			// set scroll pane viewport to table size
			applicationTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), applicationTable.getRowCount()
							* applicationTable.getRowHeight()));
			JScrollPane applicationPanel = new JScrollPane(applicationTable);
			applicationPanel.setBorder(new TitledBorder("Applications"));
			subPanel.add(applicationPanel);
		}
		if (!contexts.isEmpty()) {
			AdjustTable contextTable = getSoftwareTable(contexts);
			contextTable.adjustWidths();
			// set scroll pane viewport to table size
			contextTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), contextTable.getRowCount()
							* contextTable.getRowHeight()));
			JScrollPane contextPanel = new JScrollPane(contextTable);
			contextPanel.setBorder(new TitledBorder("Contexts"));
			subPanel.add(contextPanel);
		}
		if (!softwares.isEmpty()) {
			AdjustTable softwareTable = getSoftwareTable(softwares);
			softwareTable.adjustWidths();
			// set scroll pane viewport to table size
			softwareTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), softwareTable.getRowCount()
							* softwareTable.getRowHeight()));
			JScrollPane softwarePanel = new JScrollPane(softwareTable);
			softwarePanel.setBorder(new TitledBorder("Software"));
			subPanel.add(softwarePanel);
		}
		if (!capacities.isEmpty()) {
			AdjustTable capacityTable = getCapacityTable(capacities);
			capacityTable.adjustWidths();
			// set scroll pane viewport to table size
			capacityTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), capacityTable.getRowCount()
							* capacityTable.getRowHeight()));
			JScrollPane capacityPanel = new JScrollPane(capacityTable);
			capacityPanel.setBorder(new TitledBorder("Capacities"));
			subPanel.add(capacityPanel);
		}
		if (!storages.isEmpty()) {
			AdjustTable storageTable = getCapacityTable(storages);
			storageTable.adjustWidths();
			// set scroll pane viewport to table size
			storageTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), storageTable.getRowCount()
							* storageTable.getRowHeight()));
			JScrollPane storagePanel = new JScrollPane(storageTable);
			storagePanel.setBorder(new TitledBorder("Storages"));
			subPanel.add(storagePanel);
		}
		if (!performances.isEmpty()) {
			AdjustTable performanceTable = getPerformanceTable(performances);
			performanceTable.adjustWidths();
			// set scroll pane viewport to table size
			performanceTable.setPreferredScrollableViewportSize(new Dimension(
					getWidth(), performanceTable.getRowCount()
							* performanceTable.getRowHeight()));
			JScrollPane performancePanel = new JScrollPane(performanceTable);
			performancePanel.setBorder(new TitledBorder("Performance"));
			subPanel.add(performancePanel);
		}
		setLayout(new BorderLayout());
		add(subPanel, BorderLayout.CENTER);
	}

	/**
	 * Build a table displaying all CapacityResources
	 */
	protected AdjustTable getCapacityTable(Vector capacities) {
		String[] columnNames = {"Name", "Description", "Minimum", "Maximum",
				"Default", "Units"};
		InfoTableModel tableModel = new InfoTableModel(columnNames, 0);
		// Instead of making the table display the data as it would normally
		// with:
		// AdjustTable tableView = new AdjustTable(dataModel);
		// Add a sorter, by using the following three lines instead of the one
		// above.
		TableSorter sorter = new TableSorter(tableModel);
		AdjustTable table = new AdjustTable();
		table.setModel(sorter);
		sorter.addMouseListenerToHeaderInTable(table);
		for (int i = 0; i < capacities.size(); i++) {
			CapacityResource resource = (CapacityResource) capacities
					.elementAt(i);
			Object[] rowData = {new ResourceEntry(resource),
					resource.getDescription(),
					new Double(resource.getMinRequest()),
					new Double(resource.getMaxRequest()),
					new Double(resource.getDefaultRequest()),
					resource.getUnits()};
			tableModel.addRow(rowData);
		}
		return table;
	}

	/**
	 * Build a text area displaying all InformationResources
	 *  
	 */
	protected JTextArea getInfoArea(Vector information) {
		JTextArea textArea = new JTextArea();
		textArea.setEditable(false);
		for (int i = 0; i < information.size(); i++) {
			InformationResource resource = (InformationResource) information
					.elementAt(i);
			if (resource instanceof TextInfoResource) {
				TextInfoResource text = (TextInfoResource) resource;
				textArea.append(text.getDescription() + ": " + text.getValue()
						+ "\n");
			}
			else if (resource instanceof NumericInfoResource) {
				NumericInfoResource num = (NumericInfoResource) resource;
				textArea.append(num.getDescription() + ": " + num.getValue()
						+ "\n");
			}
		}
		return textArea;
	}

	/**
	 * Build a table displaying all CapacityResources
	 * 
	 *  
	 */
	protected AdjustTable getPerformanceTable(Vector performances) {
		String[] columnNames = {"Name", "Description", "Minimum", "Maximum",
				"Default", "Performance", "Units"};
		InfoTableModel tableModel = new InfoTableModel(columnNames, 0);
		// Instead of making the table display the data as it would normally
		// with:
		// AdjustTable tableView = new AdjustTable(dataModel);
		// Add a sorter, by using the following three lines instead of the one
		// above.
		TableSorter sorter = new TableSorter(tableModel);
		AdjustTable table = new AdjustTable();
		table.setModel(sorter);
		sorter.addMouseListenerToHeaderInTable(table);
		for (int i = 0; i < performances.size(); i++) {
			PerformanceResource resource = (PerformanceResource) performances
					.elementAt(i);
			Object[] rowData = {new ResourceEntry(resource),
					resource.getDescription(),
					new Double(resource.getMinRequest()),
					new Double(resource.getMaxRequest()),
					new Double(resource.getDefaultRequest()),
					new Double(resource.getPerformance()), resource.getUnits()};
			tableModel.addRow(rowData);
		}
		return table;
	}

	protected AdjustTable getSoftwareTable(Vector softwares) {
		String[] columnNames = {"Name", "Description", "Version"};
		InfoTableModel tableModel = new InfoTableModel(columnNames, 0);
		// Instead of making the table display the data as it would normally
		// with:
		// AdjustTable tableView = new AdjustTable(dataModel);
		// Add a sorter, by using the following three lines instead of the one
		// above.
		TableSorter sorter = new TableSorter(tableModel);
		AdjustTable table = new AdjustTable();
		table.setModel(sorter);
		sorter.addMouseListenerToHeaderInTable(table);
		for (int i = 0; i < softwares.size(); i++) {
			SoftwareResource resource = (SoftwareResource) softwares
					.elementAt(i);
			String desc = resource.getDescription();
			Object[] rowData = {resource.getName(), resource.getDescription(),
					resource.getVersion()};
			tableModel.addRow(rowData);
		}
		return table;
	}
}