/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.autoupdate;

import java.awt.Color;
import java.awt.Component;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import com.pallas.unicore.client.trees.PanelNode;

/**
 * Cell renderer for displaying the different icons in a table
 * 
 * @author Thomas Kentemich
 * @version $Id: WatchTableCellRenderer.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class WatchTableCellRenderer extends DefaultTableCellRenderer {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.plugins.autoupdate");

	/**
	 * Constructor for the WatchTableCellRenderer object
	 */
	public WatchTableCellRenderer() {
	}

	/**
	 * Sets the value of the current table cell to value.
	 * 
	 * @param table
	 *            is the JTable the receiver is being configured for
	 * @param object
	 *            is the current column
	 * @param selected
	 *            if true, the cell will be drawn as if selected.
	 * @param row
	 * @param column
	 * @param isFocused
	 *            Description of Parameter
	 * @return the Component that the renderer uses to draw the value
	 */
	public Component getTableCellRendererComponent(JTable table, Object object,
			boolean selected, boolean isFocused, int row, int column) {
		Component c = super.getTableCellRendererComponent(table, object,
				selected, isFocused, row, column);
		if (!(c instanceof JLabel)) {
			logger.warning("Table component is not a JLabel, what else is it?");
			return c;
		}
		JLabel label = (JLabel) c;
		// if we have a PanelNode get icon from container
		// else choose a default icon
		if (object instanceof PanelNode) {
			PanelNode node = (PanelNode) object;
			ImageIcon icon = node.getCurrentIcon();
			if (icon != null) {
				label.setIcon(icon);
			}
			if (node.isUpToDate()) {
				label.setForeground(Color.black);
			}
			else {
				label.setForeground(Color.gray);
			}
		}
		//	label.setText("");
		label.setHorizontalAlignment(JLabel.CENTER);
		return c;
	}
}