/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.command;

import java.io.File;
import com.pallas.unicore.client.util.Defaults;

/**
 * Define, load and save command plugin default values
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: CommandDefaults.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class CommandDefaults extends Defaults {

	/**
	 * String constants defining property keys
	 */
	public final static String COMMANDDIRECTORY_KEY = "plugin.command.commanddirectory";

	/**
	 * Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(COMMANDDIRECTORY_KEY, userHome);
	}

	/**
	 * Gets the CommandDirectory attribute of the CommandDefaults object
	 * 
	 * @return The CommandDirectory value
	 */
	public String getCommandDirectory() {
		return getProperty(COMMANDDIRECTORY_KEY);
	}

	/**
	 * Load command default values from file
	 * ${HOME}/.unicore/CommandDefaults.txt
	 * 
	 * @return true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for ${HOME}/.unicore/CommandDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "CommandDefaults.txt");
	}

	/**
	 * Sets the CommandDirectory attribute of the CommandDefaults object
	 * 
	 * @param s
	 *            The new CommandDirectory value
	 */
	public void setCommandDirectory(String s) {
		put(COMMANDDIRECTORY_KEY, s);
	}

	/**
	 * Try to write script default values to file
	 * ${HOME}/.unicore/CommandDefaults.txt
	 * 
	 * @return true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicore/CommandDefaults.txt
		return writeToFile(unicoreDir + File.separator + "CommandDefaults.txt");
	}
}