/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.compile;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import org.unicore.ajo.DoubleLength;
import org.unicore.ajo.FortranSourceForm;
import org.unicore.ajo.IntegerLength;
import org.unicore.ajo.ListingLevel;
import org.unicore.ajo.OptimisationLevel;
import org.unicore.ajo.Option;
import org.unicore.ajo.PreprocessLevel;
import org.unicore.ajo.RealLength;
import org.unicore.sets.OptionEnumeration;
import org.unicore.sets.OptionSet;

/**
 * Panel to define options for compiler
 * 
 * @author Thomas Kentemich
 * @version $Id: CompilerOptionPanel.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class CompilerOptionPanel extends JPanel {

	/**
	 * container to be modified
	 */
	protected FortranContainer compileContainer;
	// some text fields
	private JTextField defines, undefines;
	//some ComboBoxes
	private JComboBox optimisationLevel, listingLevel, preprocessLevel,
			integerLength, realLength, doubleLength, sourceFormat;
	// some check boxes
	private JCheckBox compileForDebug, compileForProfiling, boundsChecking,
			argChecking;

	/**
	 * Constructor builds GUI
	 * 
	 * @param container
	 *            to be modified
	 */
	public CompilerOptionPanel(FortranContainer container) {
		this.compileContainer = container;
		buildComponents();
	}

	/**
	 * Apply gui entries to container
	 */
	public void applyValues() {
		compileContainer.setCompileForDebug(compileForDebug.isSelected());
		compileContainer.setCompileForProfile(compileForProfiling.isSelected());
		compileContainer.setCheckArrayBounds(boundsChecking.isSelected());
		compileContainer.setCheckArguments(argChecking.isSelected());
		compileContainer
				.setOptimisationLevel((OptimisationLevel) optimisationLevel
						.getSelectedItem());
		compileContainer.setListingLevel((ListingLevel) listingLevel
				.getSelectedItem());
		compileContainer.setSourceForm((FortranSourceForm) sourceFormat
				.getSelectedItem());
		compileContainer.setPreprocessLevel((PreprocessLevel) preprocessLevel
				.getSelectedItem());
		compileContainer.setIntegerLength((IntegerLength) integerLength
				.getSelectedItem());
		compileContainer.setRealLength((RealLength) realLength
				.getSelectedItem());
		compileContainer.setDoubleLength((DoubleLength) doubleLength
				.getSelectedItem());
		compileContainer.setDefines(getOptionSetFromString(defines.getText()));
		compileContainer.setUndefines(getOptionSetFromString(undefines
				.getText()));
	}

	/**
	 * Fill gui elements with container values
	 */
	public void resetValues() {
		compileForDebug.setSelected(compileContainer.getCompileForDebug());
		compileForProfiling.setSelected(compileContainer
				.getCompileForProfiling());
		boundsChecking.setSelected(compileContainer.getCheckArrayBounds());
		argChecking.setSelected(compileContainer.getCheckArguments());
		if (compileContainer.getOptimisationLevel() != null) {
			optimisationLevel.setSelectedItem(compileContainer
					.getOptimisationLevel());
		}
		if (compileContainer.getListingLevel() != null) {
			listingLevel.setSelectedItem(compileContainer.getListingLevel());
		}
		if (compileContainer.getSourceForm() != null) {
			sourceFormat.setSelectedItem(compileContainer.getSourceForm());
		}
		if (compileContainer.getIntegerLength() != null) {
			integerLength.setSelectedItem(compileContainer.getIntegerLength());
		}
		if (compileContainer.getRealLength() != null) {
			realLength.setSelectedItem(compileContainer.getRealLength());
		}
		if (compileContainer.getDoubleLength() != null) {
			doubleLength.setSelectedItem(compileContainer.getDoubleLength());
		}
		defines.setText(optionSetToString(compileContainer.getDefines()));
		undefines.setText(optionSetToString(compileContainer.getUndefines()));
	}

	/**
	 * Parse a string in the following format and return an OptionSet: 1.) Items
	 * are separated by white space or tab 2.) Items contain a key/value pair
	 * separated by "="
	 *  
	 */
	private OptionSet getOptionSetFromString(String in) {
		OptionSet optionSet = new OptionSet();
		StringTokenizer st = new StringTokenizer(in, " \t");
		int count = st.countTokens();
		if (count < 1) {
			return null;
		}
		while (st.hasMoreElements()) {
			String token = st.nextToken();
			StringTokenizer st2 = new StringTokenizer(token, "=");
			int count2 = st2.countTokens();
			String key = st2.nextToken();
			String value = st2.nextToken();
			System.out.println("Found Option: " + key + " " + value);
			optionSet.add(new Option(key, value));
		}
		return optionSet;
	}

	/**
	 * Build gui elements
	 */
	private void buildComponents() {
		// edit area
		defines = new JTextField(30);
		defines.setToolTipText("Enter pre-processor defines (key=value)");
		undefines = new JTextField(30);
		undefines.setToolTipText("Enter pre-processor un-defines (key=value)");
		Vector preprocessItems = new Vector();
		Enumeration enum = PreprocessLevel.elements();
		while (enum.hasMoreElements()) {
			preprocessItems.add(enum.nextElement());
		}
		preprocessLevel = new JComboBox(preprocessItems);
		preprocessLevel.setSelectedItem(PreprocessLevel.NONE);
		Vector listingItems = new Vector();
		enum = ListingLevel.elements();
		while (enum.hasMoreElements()) {
			listingItems.add(enum.nextElement());
		}
		listingLevel = new JComboBox(listingItems);
		listingLevel.setSelectedItem(ListingLevel.NONE);
		Vector sourceformItems = new Vector();
		enum = FortranSourceForm.elements();
		while (enum.hasMoreElements()) {
			sourceformItems.add(enum.nextElement());
		}
		sourceFormat = new JComboBox(sourceformItems);
		sourceFormat.setSelectedItem(FortranSourceForm.FREE);
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(5, 5, 5, 5);
		c.anchor = GridBagConstraints.WEST;
		JPanel subPanel1 = new JPanel(gridbag);
		subPanel1.setBorder(new TitledBorder("Compile time options"));
		addToPanel(subPanel1, new JLabel("Preprocessor level: "),
				preprocessLevel, gridbag, c);
		addToPanel(subPanel1, new JLabel("Defines: "), defines, gridbag, c);
		addToPanel(subPanel1, new JLabel("Undefines: "), undefines, gridbag, c);
		addToPanel(subPanel1, new JLabel("Source format: "), sourceFormat,
				gridbag, c);
		addToPanel(subPanel1, new JLabel("Listing level: "), listingLevel,
				gridbag, c);
		compileForDebug = new JCheckBox("Generate debug output");
		compileForProfiling = new JCheckBox("Generate profiling output");
		boundsChecking = new JCheckBox("Enable bounds checking");
		argChecking = new JCheckBox("Enable argument checking");
		Vector optimItems = new Vector();
		enum = OptimisationLevel.elements();
		while (enum.hasMoreElements()) {
			optimItems.add(enum.nextElement());
		}
		optimisationLevel = new JComboBox(optimItems);
		optimisationLevel.setSelectedItem(OptimisationLevel.DEFAULT);
		JPanel gridPanel = new JPanel(new GridLayout(3, 2, 10, 5));
		gridPanel.add(new JLabel("Optimisation level:"));
		gridPanel.add(optimisationLevel);
		gridPanel.add(compileForDebug);
		gridPanel.add(compileForProfiling);
		gridPanel.add(boundsChecking);
		gridPanel.add(argChecking);
		JPanel subPanel2 = new JPanel();
		subPanel2.setBorder(new TitledBorder("Runtime Options"));
		subPanel2.add(gridPanel);
		Vector integerItems = new Vector();
		enum = IntegerLength.elements();
		while (enum.hasMoreElements()) {
			integerItems.add(enum.nextElement());
		}
		integerLength = new JComboBox(integerItems);
		integerLength.setSelectedItem(IntegerLength.DONTCARE);
		Vector realItems = new Vector();
		enum = RealLength.elements();
		while (enum.hasMoreElements()) {
			realItems.add(enum.nextElement());
		}
		realLength = new JComboBox(realItems);
		realLength.setSelectedItem(RealLength.DONTCARE);
		Vector doubleItems = new Vector();
		enum = DoubleLength.elements();
		while (enum.hasMoreElements()) {
			doubleItems.add(enum.nextElement());
		}
		doubleLength = new JComboBox(doubleItems);
		doubleLength.setSelectedItem(DoubleLength.DONTCARE);
		/*
		 * JPanel subPanel3 = new JPanel(gridbag); subPanel3.setBorder(new
		 * TitledBorder("Datatype size")); addToPanel(subPanel3, new
		 * JLabel("Integer length"), integerLength, gridbag, c);
		 * addToPanel(subPanel3, new JLabel("Real length"), realLength, gridbag,
		 * c); addToPanel(subPanel3, new JLabel("Double length"), doubleLength,
		 * gridbag, c);
		 */
		JPanel subPanel3 = new JPanel(new GridLayout(3, 2, 10, 5));
		subPanel3.setBorder(new TitledBorder("Datatype size"));
		subPanel3.add(new JLabel("Integer length"));
		subPanel3.add(integerLength);
		subPanel3.add(new JLabel("Real length"));
		subPanel3.add(realLength);
		subPanel3.add(new JLabel("Double length"));
		subPanel3.add(doubleLength);
		//	JPanel mainPanel = new JPanel(new GridLayout(3, 1, 5, 5));
		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(subPanel2, BorderLayout.NORTH);
		mainPanel.add(subPanel1, BorderLayout.CENTER);
		mainPanel.add(subPanel3, BorderLayout.SOUTH);
		/*
		 * mainPanel.add(subPanel2); mainPanel.add(subPanel1);
		 * mainPanel.add(subPanel3);
		 */
		JScrollPane scrollPane = new JScrollPane(mainPanel);
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
	}

	private String optionSetToString(OptionSet options) {
		String result = new String();
		if (options == null) {
			return result;
		}
		OptionEnumeration enum = options.elements();
		while (enum.hasMoreElements()) {
			Option next = enum.nextElement();
			result = result + next.getToken() + "=" + next.getValue() + " ";
		}
		return result;
	}

	private void addToPanel(JPanel panel, Component l, Component r,
			GridBagLayout g, GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}
}