/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.compile;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;
import org.unicore.ajo.Portfolio;
import org.unicore.sets.PortfolioEnumeration;
import org.unicore.sets.PortfolioList;
import com.pallas.unicore.container.CompileContainer;

/**
 * Panel to define objects
 * 
 * @author Thomas Kentemich
 * @version $Id: ObjectsPanel.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ObjectsPanel extends JPanel {

	/**
	 * container to be modified
	 */
	protected CompileContainer compileContainer;
	private JButton objectAddButton, objectRemoveButton, libAddButton,
			libRemoveButton;
	private JTable objectTable;
	private JTable libTable;
	private DefaultTableModel objectTableModel;
	private DefaultTableModel libTableModel;

	/**
	 * Constructor builds GUI
	 * 
	 * @param container
	 *            to be modified
	 * @param sourcePanel
	 */
	public ObjectsPanel(CompileContainer container, SourcesPanel sourcePanel) {
		this.compileContainer = container;
		buildComponents();
		// add a PropertyChangeListener to catch events generated by the tree
		PropertyChangeListener pListener = new TableChangeListener();
		sourcePanel.addPropertyChangeListener("sourceTableTransfer", pListener);
		sourcePanel.addPropertyChangeListener("sourceTableRemove", pListener);
	}

	/**
	 * Apply gui entries to container
	 */
	public void applyValues() {
		// stop editing
		if (objectTable.getCellEditor() != null) {
			objectTable.getCellEditor().stopCellEditing();
		}
		if (libTable.getCellEditor() != null) {
			libTable.getCellEditor().stopCellEditing();
		}
		// get vector with complete sources
		Vector complete = getCompleteRows(objectTable);
		PortfolioList pList = new PortfolioList();
		for (int i = 0; i < complete.size(); i++) {
			pList.add(new Portfolio((String) complete.elementAt(i)));
		}
		compileContainer.setObjects(pList);
		complete = getCompleteRows(libTable);
		PortfolioList list = new PortfolioList();
		for (int i = 0; i < complete.size(); i++) {
			list.add(new Portfolio((String) complete.elementAt(i)));
		}
		compileContainer.setLibraries(list);
	}

	/**
	 * Fill gui elements with container values
	 */
	public void resetValues() {
		fetchCompiledFiles();
		PortfolioEnumeration pEnum;
		PortfolioList pList = compileContainer.getObjects();
		if (pList != null) {
			pEnum = pList.elements();
			while (pEnum.hasMoreElements()) {
				Portfolio next = pEnum.nextElement();
				insertNewEntry(objectTable, next.getName());
			}
		}
		pList = compileContainer.getLibraries();
		if (pList != null) {
			pEnum = pList.elements();
			while (pEnum.hasMoreElements()) {
				Portfolio next = pEnum.nextElement();
				insertNewEntry(libTable, next.getName());
			}
		}
		libRemoveButton.setEnabled(libTable.getRowCount() > 0);
		objectRemoveButton.setEnabled(objectTable.getRowCount() > 0);
	}

	private Vector getCompleteRows(JTable table) {
		// loop over rows
		int nrOfRows = table.getRowCount();
		DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
		Vector complete = new Vector();
		for (int i = 0; i < nrOfRows; i++) {
			String sourceName = (String) tableModel.getValueAt(i, 0);
			if (sourceName != null) {
				complete.add(tableModel.getValueAt(i, 0));
			}
		}
		return complete;
	}

	/**
	 * Remove all table entries
	 */
	private void clearTable(JTable table) {
		int nrOfRows = table.getRowCount();
		DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
		for (int i = 0; i < nrOfRows; i++) {
			tableModel.removeRow(0);
		}
	}

	/**
	 * Build gui elements
	 */
	private void buildComponents() {
		ButtonListener buttonListener = new ButtonListener();
		// edit area
		String[] columnNames;
		columnNames = new String[1];
		columnNames[0] = "Object file";
		objectTableModel = new DefaultTableModel(columnNames, 0);
		objectTable = new JTable(objectTableModel) {

			/**
			 * Gets the CellEditable attribute of the ObjectsPanel object
			 * 
			 * @param row
			 *            Description of Parameter
			 * @param col
			 *            Description of Parameter
			 * @return The CellEditable value
			 */
			public boolean isCellEditable(int row, int col) {
				return true;
			}
		};
		objectTable.setRowHeight(20);
		JScrollPane objectScrollPane = new JScrollPane(objectTable);
		JPanel objectButtonPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		objectAddButton = new JButton("Add");
		objectRemoveButton = new JButton("Remove");
		objectRemoveButton.setEnabled(false);
		objectButtonPanel.add(objectAddButton);
		objectButtonPanel.add(objectRemoveButton);
		objectAddButton.addActionListener(buttonListener);
		objectRemoveButton.addActionListener(buttonListener);
		JPanel subPanel1 = new JPanel(new BorderLayout());
		subPanel1.setBorder(new TitledBorder(""));
		subPanel1.add(objectScrollPane, BorderLayout.CENTER);
		subPanel1.add(objectButtonPanel, BorderLayout.SOUTH);
		columnNames = new String[1];
		columnNames[0] = "Library path";
		libTableModel = new DefaultTableModel(columnNames, 0);
		libTable = new JTable(libTableModel) {

			/**
			 * Gets the CellEditable attribute of the ObjectsPanel object
			 * 
			 * @param row
			 *            Description of Parameter
			 * @param col
			 *            Description of Parameter
			 * @return The CellEditable value
			 */
			public boolean isCellEditable(int row, int col) {
				return true;
			}
		};
		libTable.setRowHeight(20);
		JScrollPane libScrollPane = new JScrollPane(libTable);
		JPanel libButtonPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		libAddButton = new JButton("Add");
		libRemoveButton = new JButton("Remove");
		libRemoveButton.setEnabled(false);
		libButtonPanel.add(libAddButton);
		libButtonPanel.add(libRemoveButton);
		libAddButton.addActionListener(buttonListener);
		libRemoveButton.addActionListener(buttonListener);
		JPanel subPanel2 = new JPanel(new BorderLayout());
		subPanel2.setBorder(new TitledBorder(""));
		subPanel2.add(libScrollPane, BorderLayout.CENTER);
		subPanel2.add(libButtonPanel, BorderLayout.SOUTH);
		JPanel mainPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		mainPanel.add(subPanel1);
		mainPanel.add(subPanel2);
		setLayout(new BorderLayout());
		add(mainPanel, BorderLayout.CENTER);
	}

	/**
	 * Helper method to handle GridBagLayout
	 */
	private void addToPanel(JPanel panel, Component l, Component r,
			GridBagLayout g, GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}

	/**
	 * add a row to the table
	 */
	private void addRow(JTable theTable, Object data) {
		Vector rowData = new Vector();
		rowData.add(data);
		((DefaultTableModel) theTable.getModel()).addRow(rowData);
		int selectedRow = theTable.getRowCount() - 1;
		theTable.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 * remove a row from the table
	 */
	private void removeRow(JTable theTable) {
		while (theTable.getSelectedRow() > -1) {
			((DefaultTableModel) theTable.getModel()).removeRow(theTable
					.getSelectedRow());
		}
		int selectedRow = theTable.getRowCount() - 1;
		if (selectedRow >= 0) {
			theTable.setRowSelectionInterval(selectedRow, selectedRow);
		}
	}

	private void insertNewEntry(JTable table, String name) {
		// loop over rows
		int nrOfRows = table.getRowCount();
		DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
		Vector complete = new Vector();
		for (int i = 0; i < nrOfRows; i++) {
			String sourceName = (String) tableModel.getValueAt(i, 0);
			if (name.equals(sourceName)) {
				// already contained
				return;
			}
		}
		addRow(table, name);
	}

	private void fetchCompiledFiles() {
		String[] files = compileContainer.getSources();
		if (files != null) {
			for (int i = 0; i < files.length; i++) {
				String name = files[i];
				String objectFile = name.substring(0, name.lastIndexOf('.'))
						+ ".o";
				insertNewEntry(objectTable, objectFile);
			}
		}
	}

	private class ButtonListener implements ActionListener {

		public void actionPerformed(java.awt.event.ActionEvent e) {
			Object src = e.getSource();
			if (src == objectAddButton) {
				addRow(objectTable, new String());
			}
			else if (src == objectRemoveButton) {
				// remove an entry from the list
				removeRow(objectTable);
			}
			else if (src == libAddButton) {
				addRow(libTable, new String());
			}
			else if (src == libRemoveButton) {
				removeRow(libTable);
			}
			libRemoveButton.setEnabled(libTable.getRowCount() > 0);
			objectRemoveButton.setEnabled(objectTable.getRowCount() > 0);
		}
	}

	private class TableChangeListener implements PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent evt) {
			String evtName = evt.getPropertyName();
			if (evtName.equals("sourceTableTransfer")) {
				// this updates the status of the watched job
				fetchCompiledFiles();
				libRemoveButton.setEnabled(libTable.getRowCount() > 0);
				objectRemoveButton.setEnabled(objectTable.getRowCount() > 0);
			}
			else if (evtName.equals("sourceTableRemove")) {
				clearTable(objectTable);
				fetchCompiledFiles();
				PortfolioEnumeration pEnum;
				PortfolioList pList = compileContainer.getObjects();
				if (pList != null) {
					pEnum = pList.elements();
					while (pEnum.hasMoreElements()) {
						Portfolio next = pEnum.nextElement();
						insertNewEntry(objectTable, next.getName());
					}
				}
				libRemoveButton.setEnabled(libTable.getRowCount() > 0);
				objectRemoveButton.setEnabled(objectTable.getRowCount() > 0);
			}
		}
	}
}