/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.povray;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 * This class manages the plugin
 * 
 * @author Ralf Ratering
 * @version $Id: PovrayPlugin.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class PovrayPlugin extends TaskPlugable {

	private static PovrayDefaults povrayDefaults;

	/**
	 * Return a new container for this plugin
	 * 
	 * @param parentContainer
	 *            parent GroupContainer
	 * @return new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {
		// build new plugin container
		PovrayContainer container = new PovrayContainer(parentContainer);
		container.setName("New_" + getName() + counter);
		container.setIcon(new ImageIcon(getIconImage()));
		// apply defaults
		container.setOutputDirectory(povrayDefaults.getOutputDirectory());
		container.setWidth(povrayDefaults.getWidth());
		container.setHeight(povrayDefaults.getHeight());
		container.setAntiAliasing(povrayDefaults.getAntiAliasing());
		// count instance
		counter++;
		return container;
	}

	public String getIconPath() {
		return "com/pallas/unicore/client/plugins/povray/povray.gif";
	}

	public String getName() {
		return "POV-Ray";
	}

	/**
	 * Build a new panel that manages the settings of a plugin container
	 * 
	 * @param tCont
	 *            plugin container to build panel for
	 * @return JPAPanel that represents container settings in GUI
	 */
	public JPAPanel getPanelInstance(ActionContainer tCont) {
		// build new plugin panel
		PovrayJPAPanel panel = new PovrayJPAPanel(getClient(),
				(PovrayContainer) tCont);
		panel.setDefaults(povrayDefaults);
		return panel;
	}

	/**
	 * Get a short info message for the plugin
	 * 
	 * @return String with info message
	 */
	public String getPluginInfo() {
		return "POV-Ray Plugin\n" + "Author: Ralf Ratering\n"
				+ "(C)2000-2004 Intel";
	}

	public JMenuItem getSettingsItem() {
		// handling setting menu events must be done by the plugin writers
		// as this is an optional feature
		JMenuItem settingsItem = new JMenuItem("POV-Ray Defaults", 'P');
		settingsItem.addActionListener(new SettingsListener());
		return settingsItem;
	}

	/**
	 * Show a dialog that configures plugin defaults
	 */
	private void showDefaultDialog() {
		PovrayDefaultsDialog povrayDefaultsDialog = new PovrayDefaultsDialog(
				getClient(), povrayDefaults);
		povrayDefaultsDialog.show();
		// apply changes (e.g. editor font) to currently
		// selected panel, if it is a script plugin panel
		getClient().updateDataArea();
	}

	/**
	 * Called to inititialize the plugin
	 */
	public void startPlugin() {
		povrayDefaults = new PovrayDefaults();
	}

	/**
	 * Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}

	private class SettingsListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			showDefaultDialog();
		}
	}
}