/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.script;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Date;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import org.unicore.Vsite;
import org.unicore.ajo.ScriptType;
import org.unicore.resources.SoftwareResource;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.editor.RemoteTextEditor;
import com.pallas.unicore.client.panels.FileExportPanel;
import com.pallas.unicore.client.panels.FileImportPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.panels.OptionPanel;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Panel to define, load or save a script and set its parameters This class
 * represents the script plugin container in the client GUI's data area
 * 
 * @author Ralf Ratering
 * @version $Id: ScriptJPAPanel.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class ScriptJPAPanel extends JPAPanel {

	private class ComboBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			if (e.getStateChange() == ItemEvent.SELECTED) {
				Object item = e.getItem();
				if (!shellIsAvailable(item.toString(), container.getVsite())
						&& !item.toString().equals(NOT_AVAILABLE)) {
					types.remove(item);
				}
				((JComboBox) e.getSource()).setForeground(Color.BLACK);
			}
		}
	}

	private class ComboBoxRenderer extends DefaultListCellRenderer {

		public ComboBoxRenderer() {
			super();
		}

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);
			list.setForeground(Color.BLACK);
			return this;
		}
	}
	public final static String NOT_AVAILABLE = "Not Available";
	// panel to define file imports
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.plugins.script.ResourceStrings");
	// the container this panel represents
	private ScriptContainer container;
	// panel to define file exports
	private FileExportPanel exportPanel;
	// panel to define file imports
	private FileImportPanel importPanel;
	// text fields for script filename and task name
	private JTextField nameTextField;
	// panel for setting script options
	private OptionPanel optionsPanel;
	// default script directory
	private ScriptDefaults scriptDefaults;
	// panel to edit the script
	private RemoteTextEditor textEditor = new RemoteTextEditor();
	private JComboBox typeComboBox;
	private ComboBoxModel typeComboBoxModel;
	// objects to define a JComboBox for different script types
	private Vector types;

	/**
	 * Constructor
	 * 
	 * @param parentFrame
	 *            reference to main JFrame for modal sub dialogs
	 * @param container
	 *            container that this panel represents
	 */
	public ScriptJPAPanel(JFrame parentFrame, ScriptContainer container) {
		super(parentFrame, container);
		this.container = container;
		// build gui
		buildComponents();
	}

	/**
	 * Apply values from GUI to container
	 */
	public void applyValues() {
		// replace spaces in script name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		updateModifiedTime();
		// container id will change here.....
		container.setName(nameTextField.getText());
		if (this.getScriptType() == null) {
			container.setScriptType(null, true);
		}
		else {
			container.setScriptType(getScriptType(), shellIsAvailable(
					getScriptType().toString(), container.getVsite()));
		}
		container.setScriptFile(textEditor.getFile());
		container.setScriptContents(textEditor.getText());
		importPanel.applyValues();
		exportPanel.applyValues();
		optionsPanel.applyValues();
	}

	/**
	 * Build GUI components
	 */
	private void buildComponents() {
		nameTextField = new JTextField(15);
		nameTextField.setToolTipText(res.getString("ENTER_ID"));
		types = new Vector();
		typeComboBox = new JComboBox(types);
		this.updateScriptTypes();
		typeComboBox.setToolTipText(res.getString("CHOOSE_TYPE"));
		if (this.types.contains(ScriptType.SH.getName())) {
			typeComboBox.setSelectedItem(ScriptType.SH.getName());
		}
		else {
			this.typeComboBox.setSelectedIndex(0);
		}
		this.typeComboBox.setRenderer(new ComboBoxRenderer());
		typeComboBox.addItemListener(new ComboBoxListener());
		JPanel namePanel = new JPanel();
		namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
		namePanel.add(new JLabel("Name: "));
		namePanel.add(nameTextField);
		JPanel typePanel = new JPanel();
		typePanel.setLayout(new BoxLayout(typePanel, BoxLayout.X_AXIS));
		typePanel.add(new JLabel("Type: "));
		typePanel.add(typeComboBox);
		JPanel northPanel = new JPanel(new GridLayout(1, 2, 5, 10));
		northPanel.add(namePanel);
		northPanel.add(typePanel);
		// build import export panels
		UserDefaults userDefaults = ResourceManager.getUserDefaults();
		importPanel = new FileImportPanel(container, userDefaults
				.getImportDirectory());
		exportPanel = new FileExportPanel(container, userDefaults
				.getExportDirectory());
		optionsPanel = new OptionPanel(container);
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(textEditor, res.getString("SCRIPT_TITLE"));
		tabbedPane.add(optionsPanel, res.getString("OPTIONS"));
		tabbedPane.add(importPanel, res.getString("IMPORTS"));
		tabbedPane.add(exportPanel, res.getString("EXPORTS"));
		add(northPanel, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);
		setBorder(BorderFactory.createEtchedBorder());
		// setBorder(new TitledBorder(res.getString("EXECUTE_BORDER")));
	}

	private String formatScriptType(String shell) {
		for (int i = 0; i < shell.length(); i++) {
			if (Character.isWhitespace(shell.charAt(i))
					|| shell.charAt(i) == '_') {
				shell = shell.substring(0, i)
						+ shell.substring(i + 1, shell.length());
			}
		}
		return shell;
	}

	/**
	 * Get script type from gui
	 * 
	 * @return ScriptType
	 */
	private ScriptType getScriptType() {
		ScriptType type = null;
		if (typeComboBox.getSelectedItem() != null) {
			String s = (String) typeComboBox.getSelectedItem();
			Enumeration e = ScriptType.elements();
			while (e.hasMoreElements()) {
				ScriptType t = (ScriptType) e.nextElement();
				if (s.equals(t.getName())) {
					type = t;
				}
			}
		}
		return type;
	}

	/**
	 * Fill values from container into gui
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}
		// reset sub panels
		importPanel.resetValues();
		exportPanel.resetValues();
		optionsPanel.resetValues();
		// set task name
		nameTextField.setText(container.getName());
		// set script type
		ScriptType s = container.getScriptType();
		if (s != null) {
			typeComboBox.setSelectedItem(s.getName());
		}
		else {
			if (this.types.contains(ScriptType.SH.getName())) {
				typeComboBox.setSelectedItem(ScriptType.SH.getName());
			}
			else {
				this.typeComboBox.setSelectedIndex(0);
			}
		}
		// get script contents
		textEditor.setText(container.getScriptContents());
		textEditor.setFile(container.getScriptFile());
	}

	public void setDefaults(ScriptDefaults scriptDefaults) {
		this.scriptDefaults = scriptDefaults;
		textEditor.setDefaultDir(new File(scriptDefaults.getScriptDirectory()));
	}

	private boolean shellIsAvailable(String shell, Vsite vsite) {
		if (vsite == null) {
			return false;
		}
		NamedResourceSet resourceSet = ResourceManager.getResourceSet(vsite);
		Vector vector = resourceSet.addContextResources(new Vector());
		for (int i = 0; i < vector.size(); i++) {
			SoftwareResource resource = (SoftwareResource) vector.elementAt(i);
			String name = resource.getName();
			if (this.formatScriptType(shell).equalsIgnoreCase(
					this.formatScriptType(name))) {
				return true;
			}
		}
		return false;
	}

	/**
	 * compare gui entries to container values
	 */
	private void updateModifiedTime() {
		// if an entry is missing update modified time and get out
		if (container.getName() == null || container.getScriptType() == null
				|| container.getScriptContents() == null) {
			container.setModifiedTime(new Date());
			return;
		}
		if (!container.getName().equals(nameTextField.getText())
				|| this.getScriptType() == null
				|| !container.getScriptType().equals(getScriptType())
				|| !container.getScriptContents().equals(textEditor.getText())) {
			container.setModifiedTime(new Date());
			return;
		}
		if (container.getScriptFile() != null
				&& !container.getScriptFile().equals(textEditor.getFile())) {
			container.setModifiedTime(new Date());
			return;
		}
	}

	private void updateScriptTypes() {
		types.clear();
		Vsite vsite = this.container.getVsite();
		if (vsite != null) {
			Enumeration e = ScriptType.elements();
			while (e.hasMoreElements()) {
				ScriptType t = (ScriptType) e.nextElement();
				String shell = t.getName();
				if (this.shellIsAvailable(shell, vsite)) {
					types.addElement(shell);
				}
			}
		}
		if (this.typeComboBox.getModel().getSize() == 0) {
			types.addElement(NOT_AVAILABLE);
		}
		this.typeComboBox.updateUI();
		Object selectedItem = this.typeComboBox.getSelectedItem();
		if (selectedItem != null) {
			if (selectedItem.toString().equals(NOT_AVAILABLE)
					&& this.typeComboBox.getModel().getSize() > 1) {
				if (types.contains(ScriptType.SH.getName())) {
					typeComboBox.setSelectedItem(ScriptType.SH.getName());
				}
				else {
					this.typeComboBox.setSelectedIndex(0);
				}
				typeComboBox.setForeground(Color.BLACK);
			}
			else {
				if (shellIsAvailable(selectedItem.toString(), container
						.getVsite())
						|| selectedItem.toString().equals(NOT_AVAILABLE)) {
					typeComboBox.setForeground(Color.BLACK);
				}
				else {
					typeComboBox.setForeground(Color.RED);
				}
			}
		}
	}

	/**
	 * Method will be called whenever this panel is activated
	 * 
	 * @param vsiteChanged
	 *            true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		// if we are online get current storage resources from server
		if (container.getVsite() != null) {
			if (vsiteChanged) {
				// pass vsite to remote editor
				textEditor.setVsite(container.getVsite());
				// update storage resources for sub panels
				importPanel.updateValues();
				exportPanel.updateValues();
				updateScriptTypes();
			}
		}
		// update editor if neccessary
		if (!textEditor.getFont().equals(scriptDefaults.getEditorFont())
				|| textEditor.getLineWrap() != scriptDefaults.getLineWrap()) {
			textEditor.setEditorFont(scriptDefaults.getEditorFont());
			textEditor.setLineWrap(scriptDefaults.getLineWrap());
			textEditor.repaint();
		}
	}
}