/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.io.StreamCorruptedException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * @author Thomas Kentemich
 * @version $Id: PluginObjectInputStream.java,v 1.2 2003/06/30 06:56:36 rrate
 *          Exp $
 */
public class PluginObjectInputStream extends ObjectInputStream {

	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.util.ResourceStrings");
	private PluginManager pManager = null;

	public PluginObjectInputStream(InputStream in, PluginManager pManager)
			throws IOException, StreamCorruptedException {
		super(in);
		this.pManager = pManager;
	}

	/**
	 * Overrides the standard loader and checks all plugin loaders
	 *  
	 */
	protected Class resolveClass(ObjectStreamClass clazz) throws IOException,
			ClassNotFoundException {
		// system classloader
		Class self = PluginObjectInputStream.class;
		ClassLoader loader = self.getClassLoader();
		Vector knownPlugins = pManager.getPlugins();
		if (!knownPlugins.isEmpty()) {
			/*
			 * iterate through the plugins and try to load classes with the
			 * associated classloader
			 */
			for (int i = 0; i < knownPlugins.size(); i++) {
				UnicorePlugin plugin = (UnicorePlugin) knownPlugins
						.elementAt(i);
				loader = plugin.getPluginClassloader();
				try {
					Class retClass = Class.forName(clazz.getName(), false,
							loader);
					return retClass;
				}
				catch (ClassNotFoundException cnfe) {
				}
			}
		}
		else {
			// system class loader
			return Class.forName(clazz.getName(), false, loader);
		}
		// finally class not found
		throw new ClassNotFoundException(res.getString("NOT_FOUND"));
	}
}