/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.io.Serializable;
import org.unicore.ajo.Dependency;

/**
 * Class representing a dependency between two ActionContainers.
 * 
 * @author Thomas Kentemich
 * @version $Id: DependencyContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class DependencyContainer implements Serializable {

	final static long serialVersionUID = -1437076470626701799L;
	private ActionContainer pred;
	private ActionContainer succ;

	public DependencyContainer() {
		pred = null;
		succ = null;
	}

	public DependencyContainer(ActionContainer pred, ActionContainer succ) {
		this.pred = pred;
		this.succ = succ;
	}

	public Dependency getDependency() {
		return new Dependency(pred.getActionGroup(), succ.getActionGroup());
	}

	public ActionContainer getPred() {
		return pred;
	}

	public ActionContainer getSucc() {
		return succ;
	}

	public void setPred(ActionContainer pred) {
		this.pred = pred;
	}

	public void setSucc(ActionContainer succ) {
		this.succ = succ;
	}

	public String toString() {
		// pred and succ have their own toString methods, so we can use them
		return new String(pred + " -> " + succ);
	}
}