package com.pallas.unicore.container;

import java.io.Serializable;
import org.unicore.sets.OptionSet;
import com.pallas.unicore.container.errorspec.ErrorSet;

/**
 * Implements a container class for executing commands. This class knows about
 * Context's environment variables, stdin/stdout/stderr Portfolios.
 * 
 * @author Thomas Kentemich
 * @version $Id: ExecuteContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public abstract class ExecuteContainer extends TaskContainer
		implements
			Serializable {

	final static long serialVersionUID = 4642089981942904957L;
	protected String commandLine;
	protected OptionSet environment;
	protected String stdoutPath, stderrPath, stdinPath;
	protected boolean verbose = false, version = false, redirectStdout,
			redirectStdin, redirectStderr;

	/**
	 * Constructor
	 * 
	 * @param parentContainer
	 *            parent container
	 */
	public ExecuteContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 * Self check the correctness of the values relevant for this container
	 * 
	 * @return Description of the Returned Value
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		// check contexts
		// check environment vars
		// check cmd line
		// check stdout
		// check verbose
		// check version
		setErrors(err);
		return err;
	}

	public String getCommandLine() {
		return commandLine;
	}

	public OptionSet getEnv() {
		return environment;
	}

	public boolean getRedirectStderr() {
		return redirectStderr;
	}

	public boolean getRedirectStdout() {
		return redirectStdout;
	}

	public String getStderr() {
		return stderrPath;
	}

	public String getStdout() {
		return stdoutPath;
	}

	public boolean isVerboseOn() {
		return verbose;
	}

	public boolean isVersionOn() {
		return version;
	}

	public void setCommandLine(String commandLine) {
		this.commandLine = commandLine;
	}

	/**
	 * Set the environment for this task
	 * 
	 *  
	 */
	public void setEnv(OptionSet environment) {
		this.environment = environment;
	}

	public void setRedirectStderr(boolean what) {
		this.redirectStderr = what;
	}

	public void setRedirectStdout(boolean what) {
		this.redirectStdout = what;
	}

	public void setStderr(String stderr) {
		this.stderrPath = stderr;
	}

	public void setStdout(String stdout) {
		this.stdoutPath = stdout;
	}

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	public void setVersion(boolean version) {
		this.version = version;
	}
}