/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.util.ResourceBundle;
import org.unicore.ajo.ActionGroup;
import org.unicore.resources.Resource;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.FileSystemOperationFactory;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Container for all tasks that copy, move or delete files, or make directories
 * 
 * @author Ralf Ratering
 * @version $Id: FileOperationContainer.java,v 1.4 2004/02/03 14:50:29 kdoertel
 *          Exp $
 */
public class FileOperationContainer extends FileContainer {

	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -7752008158209771600L;

	/**
	 * Constructor builds file container as child of job container
	 * 
	 * @param parentContainer
	 *            parent job group
	 */
	public FileOperationContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 * Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		FileSystemOperationFactory factory = new FileSystemOperationFactory(
				fileSystemOperations, actionGroup, this);
		factory.build();
		// copy identifier from action group to container to to have a unique id
		// for the njs.
		syncIdentifiers();
	}

	/**
	 * Add container specific errors to those of super classes
	 * 
	 * @return ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		if (fileSystemOperations == null || fileSystemOperations.length == 0) {
			err
					.add(new UError(getIdentifier(), res
							.getString("NO_OPERATIONS")));
		}
		else {
			NamedResourceSet resourceSet = ResourceManager.getResourceSet(this
					.getVsite());
			for (int i = 0; i < fileSystemOperations.length; i++) {
				FileSystemOperation fileSystemOperation = fileSystemOperations[i];
				if (!fileSystemOperation.isComplete()) {
					err.add(new UError(id, res
							.getString("INCOMPLETE_FILE_OPERATION")
							+ fileSystemOperation
							+ res.getString("AT_ROW")
							+ (i + 1)));
				}
				Resource storage = (Resource) fileSystemOperation
						.getStorageItem();
				if (storage == null || !resourceSet.contains(storage)) {
					err.add(new UError(id, res
							.getString("INCOMPLETE_FILE_OPERATION")
							+ fileSystemOperation
							+ res.getString("AT_ROW")
							+ (i + 1)
							+ ".\n\t"
							+ res.getString("STORAGE")
							+ storage
							+ res.getString("NOT_AVAILABLE")
							+ this.getVsite()));
				}
			}
		}
		setErrors(err);
		return err;
	}
}