/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.Vsite;
import org.unicore.resources.Storage;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Class representing a file transfer between two uspaces
 * 
 * @author Ralf Ratering
 * @version $Id: FileTransfer.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FileTransfer extends FileStorage {

	private static DataFlavor flavor;
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = 8649468376722199373L;

	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor = new DataFlavor(Class
						.forName("com.pallas.unicore.extensions.FileTransfer"),
						"FileTransfer");
			}
			catch (ClassNotFoundException e) {
				logger
						.log(
								Level.SEVERE,
								"Error getting DataFlavore for com.pallas.unicore.extensions.FileTransfer",
								e);
			}
		}
		return flavor;
	}
	/**
	 * JobContainer defining destination uspace
	 */
	protected JobContainer destinationJob;

	/**
	 * Constructor including destination job group
	 * 
	 * @param sourceName
	 *            name of source file
	 * @param destinationJob
	 *            destination job group
	 * @param destinationName
	 *            name of destination file
	 */
	public FileTransfer(Object storageItem, String sourceName,
			JobContainer destinationJob, String destinationName) {
		this(storageItem, sourceName, destinationJob, null, destinationName);
	}

	/**
	 * Constructor including destination job group
	 * 
	 * @param sourceStorageItem
	 *            defining storage location
	 * @param sourceName
	 *            name of source file
	 * @param destinationJob
	 *            destination job group
	 * @param destinationStorage
	 *            destination storage location
	 * @param destinationName
	 *            name of destination file
	 */
	public FileTransfer(Object sourceStorageItem, String sourceName,
			JobContainer destinationJob, Storage destinationStorage,
			String destinationName) {
		super(sourceStorageItem, sourceName, destinationStorage,
				destinationName);
		// sourcename == destination name currently
		//		this.destinationName = sourceName;
		this.destinationJob = destinationJob;
		if (this.otherStorage == null && this.destinationJob != null) {
			Vsite vsite = this.destinationJob.getVsite();
			if (vsite != null) {
				this.otherStorage = ResourceManager.getResourceSet(vsite)
						.getUspace();
			}
		}
	}

	/**
	 * Get separator of destination file system (always UNIX)
	 * 
	 * @return separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 * @return destination JobContainer
	 */
	public JobContainer getDestinationJob() {
		return destinationJob;
	}

	/**
	 * Get separator of source file system (always UNIX)
	 * 
	 * @return separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	public DataFlavor getSubClassFlavor() {
		return flavor;
	}

	public boolean isComplete() {
		boolean result = super.isComplete();
		result = result && destinationJob != null;
		result = result && otherStorage != null;
		return result;
	}

	public void setDestinationJob(JobContainer destinationJob) {
		this.destinationJob = destinationJob;
	}

	/**
	 * @return String describing operation
	 */
	public String toString() {
		return storageItem + ":" + sourceName + " -> " + otherStorage + ":"
				+ destinationName + " @ " + destinationJob;
	}
}