/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.logging.Level;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.GetActionStatus;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.GetActionStatus_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Get status of an AbstractJob
 * 
 * @author Ralf Ratering
 * @version $Id: GetJobStatus.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetJobStatus extends ObservableRequestThread {

	private AbstractJob job;
	private AbstractActionStatus status;

	/**
	 * Constructor
	 * 
	 * @param job
	 *            Get status for this AbstractJob
	 */
	public GetJobStatus(AbstractJob job) {
		this.job = job;
	}

	/**
	 * Get the AbstractJob
	 * 
	 * @return job
	 */
	public AbstractJob getJob() {
		return job;
	}

	/**
	 * Get the resulting AbstractActionStatus
	 * 
	 * @return The status value
	 */
	public AbstractActionStatus getStatus() {
		return status;
	}

	/**
	 * Start the Thread
	 */
	public void run() {
		Vsite vsite = job.getVsite();
		AbstractJob getStatusJob = new AbstractJob(ResourceManager
				.getServicePrefix()
				+ "JobStatus_" + ResourceManager.getNextObjectIdentifier(),
				null, null, vsite, ResourceManager.getUser(vsite));
		GetActionStatus getStatus = new GetActionStatus("GetActionStatus",
				null, job.getAJOId());
		getStatusJob.add(getStatus);
		try {
			Reply reply = nonPolling(getStatusJob, vsite, ResourceManager
					.getUser(vsite));
			if (reply == null || !(reply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Bad NJS reply.");
			}
			else {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] serializedOutcome = roReply.getOutcome();
				if (serializedOutcome == null) {
					throw new Exception("No job outcome available.");
				}
				AbstractJob_Outcome jobOutcome = (AbstractJob_Outcome) new ObjectInputStream(
						new ByteArrayInputStream(serializedOutcome))
						.readObject();
				jobOutcome = ConsignForm.convertFrom(jobOutcome);
				OutcomeEnumeration outcomes = jobOutcome.getOutcomes();
				if (outcomes.hasMoreElements()) {
					GetActionStatus_Outcome statusOutcome = (GetActionStatus_Outcome) outcomes
							.nextElement();
					status = statusOutcome.getActionStatus();
				}
			}
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Could not get status for job " + job, e);
		}
		notifyObservers(this, status);
	}
}