/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ExportTask;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.ActionGroup.InvalidDependencyException;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.resources.Storage;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Send a list of files to an XSpace storage
 * 
 * @author Ralf Ratering
 * @version $Id: SendFilesToXspace.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class SendFilesToXspace extends ObservableRequestThread {

	private String[] localFilenames;
	private String[] destinationFilenames;
	private Vsite vsite;
	private Storage storage;
	private Vector fileImports = new Vector();

	/**
	 * Set parameters of request in constructor
	 * 
	 * @param localFilenames
	 *            source filenames
	 * @param vsite
	 *            destination vsite
	 * @param storage
	 *            destination storage resource
	 * @param destinationFilenames
	 *            array of filenames to transfer
	 */
	public SendFilesToXspace(String[] localFilenames, Vsite vsite,
			Storage storage, String[] destinationFilenames) {
		this.localFilenames = localFilenames;
		this.vsite = vsite;
		this.storage = storage;
		this.destinationFilenames = destinationFilenames;
		buildFileImports();
	}

	private void buildFileImports() {
		fileImports.clear();
		for (int i = 0; i < localFilenames.length; i++) {
			FileImport fileImport = new FileImport(FileStorage.NSPACE_STRING,
					localFilenames[i], destinationFilenames[i]);
			fileImports.add(fileImport);
		}
	}

	/**
	 * Start the request
	 */
	public void run() {
		Storage storage = null;
		MakePortfolio makePortfolio = new MakePortfolio("MakePortfolio");
		makePortfolio.setFiles(destinationFilenames);
		ExportTask exportTask = new ExportTask("ExportTask");
		exportTask.addResource(storage);
		exportTask.setPortfolio(makePortfolio.getPortfolio());
		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job = new AbstractJob(servicePrefix
				+ "SendFilesToAlternativeUspace"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				user);
		job.add(exportTask);
		try {
			job.addDependency(makePortfolio, exportTask);
		}
		catch (InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add dependency.", e);
		}
		Reply reply = nonPolling(job, vsite, user, fileImports);
		AbstractJob_Outcome ajOut = null;
		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning("Virtual site not available: " + vsite
					+ "\nReason: " + reply.getLastEntry().getComment());
			return;
		}
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes)).readObject();
				ajOut = ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
			}
			catch (Exception e) {
				logger.log(Level.SEVERE,
						"Could send files to alternative uspace.", e);
			}
		}
		notifyObservers(this, ajOut);
	}
}