/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.resourcemanager;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.unicore.resources.Memory;
import org.unicore.resources.RunTime;
import org.unicore.sets.ResourceSet;
import com.pallas.unicore.extensions.NamedResourceSet;

/**
 * This class is a container for a set of related ResourceSets (for example all
 * sets belonging to a JobGroup. It provides functions to manipulate single and
 * collections of ResourceSets and to store and load the tray.
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: ResourceTray.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class ResourceTray extends Hashtable {

	final static long serialVersionUID = 2339131310553778886L;

	public static void main(String[] args) {
		ResourceTray tray = new ResourceTray("TEST");
		ResourceSet set1 = new ResourceSet();
		Memory cap1 = new Memory("tr1", 20, 10, 0);
		set1.add(cap1);
		String key1 = "key1";
		String key2 = "key2";
		tray.put(set1, key1);
		tray.put(set1, key2);
		cap1.setMaxRequest(50);
		RunTime cap2 = new RunTime("tr2", 50, 10, 0);
		cap1.setMaxRequest(20);
		tray.put(set1, key1);
	}
	// the name of the tray
	private String name;

	public ResourceTray() {
		super();
	}

	public ResourceTray(String name) {
		this.name = name;
	}

	public ResourceTray(String name, ResourceSet initial, Object initialKey) {
		this(name);
		put(initialKey, initial);
	}

	/**
	 * Overwrite get method to Return an empty site default resource set instead
	 * of null
	 *  
	 */
	public Object get(Object key) {
		Object value = super.get(key);
		if (value == null) {
			value = new NamedResourceSet();
		}
		return value;
	}

	public Vector getAllResourceSets() {
		Vector sets = new Vector();
		Enumeration enum = elements();
		while (enum.hasMoreElements()) {
			NamedResourceSet set = (NamedResourceSet) enum.nextElement();
			if (!sets.contains(set)) {
				sets.add(set);
			}
		}
		return sets;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}
}