/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.security;

import java.security.cert.X509Certificate;
import java.util.Enumeration;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.Ulogin;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.ServerRequest;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.util.ExtensionPlugable;

/**
 * @author Thomas Kentemich
 * @version $Id: JobConverter.java,v 1.1 2004/05/25 14:58:55 rmenday Exp $
 */
public class JobConverter {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.security");

	public final static ConsignJob convert(AbstractJob job, boolean polling,
			boolean initSpecial) throws Exception {
		if (initSpecial) {
			logger.log(Level.INFO, "Initializing SSO for site "
					+ job.getVsite());
			initSSO(job);
		}
		AbstractJob convertedJob = JobSigner.convertTo(job);
		ConsignJob cj = new ConsignJob();
		cj.setTarget(convertedJob.getAJOId());
		cj.setVsite(convertedJob.getVsite());
		cj.setEndorser(convertedJob.getEndorser());
		X509Certificate[] chain = {convertedJob.getEndorser().getCertificate()};
		cj.setConsignor(new Ulogin(chain));
		cj.setAJO(convertedJob.getConsignForm());
		cj.setSignature(convertedJob.getSignature());
		cj.setPolling(polling);
		return cj;
	}

	public final static ServerRequest convert(ServerRequest request,
			boolean initSpecial) throws Exception {
		if (initSpecial) {
			logger.log(Level.INFO, "Initializing UPL  SSO for site "
					+ request.getVsite());
			initSSO(request);
		}
		return request;
	}

	/**
	 * Used to initialize special features of a vsite via a plugin. The plugins
	 * are asked to provide special stuff for the vsite in question via the
	 * setupSpecialVsiteFeatures() method. This is use for exmple to add a
	 * Globus proxy certificate to an AJO.
	 * 
	 * @return
	 */
	private static void initSSO(AbstractJob job) {
		Vector plugins = Client.getPluginManager().getPluginClasses();
		Enumeration enum = job.getActions();
		// init for this job
		for (int i = 0; i < plugins.size(); i++) {
			if (plugins.elementAt(i) instanceof ExtensionPlugable) {
				try {
					ExtensionPlugable plugable = (ExtensionPlugable) plugins
							.elementAt(i);
					Object returnedObject = plugable.setupSpecialVsiteFeatures(
							job.getVsite(), job);
				}
				catch (Exception e) {
					logger.log(Level.WARNING,
							"Initializing vsite features for site "
									+ job.getVsite() + " failed.", e);
					return;
				}
			}
		}
		// search sub-jobs
		while (enum.hasMoreElements()) {
			AbstractAction action = (AbstractAction) enum.nextElement();
			if (action instanceof AbstractJob) {
				initSSO((AbstractJob) action);
			}
		}
		return;
	}

	private static void initSSO(ServerRequest request) {
		// init for this request
		Vector plugins = Client.getPluginManager().getPluginClasses();
		for (int i = 0; i < plugins.size(); i++) {
			if (plugins.elementAt(i) instanceof ExtensionPlugable) {
				try {
					ExtensionPlugable plugable = (ExtensionPlugable) plugins
							.elementAt(i);
					//					if (plugable instanceof ProxyCertificatePlugin) {
					Object sso = plugable.setupSpecialVsiteFeatures(request
							.getVsite(), null);
					if (sso instanceof byte[]) {
						logger.fine(new String((byte[]) sso));
						request.setSSO((byte[]) sso);
					}
					//					}
				}
				catch (Exception e) {
					logger.log(Level.WARNING,
							"Initializing vsite features  (ServerRequest) for site "
									+ request.getVsite() + " failed.", e);
					return;
				}
			}
		}
	}
}