package com.pallas.unicore.utility;

import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.AAIdentifier;
import org.unicore.AJOIdentifier;
import org.unicore.PortfolioIdentifier;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.AbstractService;
import org.unicore.ajo.AbstractTask;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.CompileTask;
import org.unicore.ajo.ConditionalAction;
import org.unicore.ajo.CopyPortfolioToOutcome;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.Dummy;
import org.unicore.ajo.ExecuteTask;
import org.unicore.ajo.ExportTask;
import org.unicore.ajo.FileAction;
import org.unicore.ajo.FortranTask;
import org.unicore.ajo.GetPortfolio;
import org.unicore.ajo.ImportTask;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.LinkMapLevel;
import org.unicore.ajo.LinkTask;
import org.unicore.ajo.ListingLevel;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.ajo.PreprocessLevel;
import org.unicore.ajo.PutPortfolio;
import org.unicore.ajo.UserTask;
import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.Resource;
import org.unicore.sets.PortfolioEnumeration;
import org.unicore.sets.ResourceEnumeration;

/**
 * @author Sven van den Berghe, Fujitsu Laboratories of Europe, Sept 2002
 * @author Ralf Ratering, Intel
 * @version $Id: PortfolioExaminer.java,v 1.2 2004/05/27 15:43:48 ratering Exp $
 */
public class PortfolioExaminer {

	private static Logger logger = Logger.getLogger("com.pallas.unicore");

	/**
	 * The main program for the PortfolioExaminer class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		// Get the name of the serialised AJO from the command line and raed it
		// into "test_ajo"
		if (args.length != 1) {
			System.err.println("Call with a single arg (the serialised AJO to examine)");
			System.exit(1);
		}
		AbstractJob test_ajo = null;
		try {
			test_ajo = (AbstractJob) (new ObjectInputStream(new FileInputStream(args[0])))
					.readObject();
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "Problems reading AJO from <" + args[0] + ">", ex);
			System.exit(1);
		}
		try {
			// Run through the AJO finding all Portfolios and recodring how they
			// are used, if revealed etc
			AbstractActionParser ap = AbstractActionParser.create(test_ajo, new AJOIdentifier(
					"NSPACE", 1));
			// Look any Portfolios that are not revealed
			Iterator i = ap.portfolios();
			while (i.hasNext()) {
				PortfolioDetails pd = (PortfolioDetails) i.next();
				if (!pd.isRevealed() && pd.parent_ajo != AbstractActionParser.SPOOL_ID) {
					// NB SPOOL_ID marks Portfolios spooled by PutPortfolio
					// (most other spool interactions are not recorded)
					String to_print = "Portfolio is not revealed in <" + args[0] + "> <"
							+ pd.pid.getName() + "/" + Integer.toHexString(pd.pid.getValue())
							+ "> in <" + pd.parent_ajo.getName() + "/"
							+ Integer.toHexString(pd.parent_ajo.getValue()) + ">";
					if (pd.created_by != null) {
						to_print += " was created by <" + pd.created_by.getName() + "/"
								+ Integer.toHexString(pd.created_by.getValue()) + ">";
					}
					else {
						to_print += " was NOT created !";
					}
					Iterator j = pd.used_by.iterator();
					while (j.hasNext()) {
						to_print += " ," + ((AAIdentifier) j.next()).getName();
					}
					logger.info(to_print);
					patch(test_ajo, pd.pid, pd.parent_ajo, pd.created_by);
				}
			}
			System.out.println("Testing ...");
			AbstractActionParser ap_test = AbstractActionParser.create(test_ajo, new AJOIdentifier(
					"NSPACE", 1));
			Iterator i_test = ap_test.portfolios();
			while (i_test.hasNext()) {
				PortfolioDetails pd_test = (PortfolioDetails) i_test.next();
				if (!pd_test.isRevealed() && pd_test.parent_ajo != AbstractActionParser.SPOOL_ID) {
					System.out.println("Patch went wrong ?#??!$%");
				}
			}
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "Problems parsing AJO", ex);
			System.exit(1);
		}
	}

	/**
	 * Add a DeclarePortfolio to the "ajo" to force the reveal the "pid"
	 * Portfolio
	 *  
	 */
	public static void patch(ActionGroup root_ajo, PortfolioIdentifier pid, AJOIdentifier ajo,
			AAIdentifier creator) {
		if (root_ajo.getId().equals(ajo)) {
			// This is the one, patch it
			DeclarePortfolio dp = new DeclarePortfolio("Patch unrevealed");
			dp.setPortfolio(new Portfolio(pid.getName(), pid.getValue()));
			// Where do we add the DdeclarePortfolio? If the creating AA is part
			// of a
			// child ActionGroup, then after it. Otherwise, at the start of
			// root_ajo.
			AbstractAction creating_parent = searchForTheCreator(root_ajo, creator);
			if (creating_parent != null) {
				root_ajo.add(dp);
				logger.info("Inserted after <" + creating_parent.getId().getName() + "/"
						+ Integer.toHexString(creating_parent.getId().getValue()) + ">");
				// creating_parent is part of root_ajo's DAG, insert DP after
				// this
				// with same dependencies (do not remove any)
				Enumeration e = root_ajo.getDependencies();
				while (e.hasMoreElements()) {
					Dependency d = (Dependency) e.nextElement();
					if (d.getPredecessor().equals(creating_parent)) {
						try {
							root_ajo.addDependency(new Dependency(dp, d.getSuccessor()));
						}
						catch (ActionGroup.InvalidDependencyException ex) {
							logger.log(Level.SEVERE, "Could not add dependency.", ex);
						}
					}
				}
				try {
					root_ajo.addDependency(new Dependency(creating_parent, dp));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
			else {
				// RRCHANGE: add first dp, then dependencies
				root_ajo.add(dp);
				// (add last so it does not appear in the ready() list
				// this would cause an exception!)
				logger.info("Inserted at beginning");
				// The creator not part of this AG's "tree", declare at start
				Enumeration e = root_ajo.ready();
				while (e.hasMoreElements()) {
					try {
						AbstractAction action = (AbstractAction) e.nextElement();
						// RRCHANGE: no dep onto dp itself
						if (action.equals(dp)) {
							continue;
						}
						logger.info("ADDING DEP: " + dp.getName() + " TO " + action.getName());
						root_ajo.addDependency(new Dependency(dp, action));
					}
					catch (ActionGroup.InvalidDependencyException ex) {
						logger.log(Level.SEVERE, "Could not add dependency.", ex);
					}
				}
			}
		}
		else {
			// Check all contained ActionGroups
			Enumeration actions = root_ajo.getActions();
			while (actions.hasMoreElements()) {
				AbstractAction action = (AbstractAction) actions.nextElement();
				if (action instanceof ActionGroup) {
					patch((ActionGroup) action, pid, ajo, creator);
				}
				// RRCHANGE: go into branches for ConditionalActions
				else if (action instanceof ConditionalAction) {
					ConditionalAction condAction = (ConditionalAction) action;
					if (condAction.getTrueBranch() instanceof ActionGroup) {
						patch((ActionGroup) condAction.getTrueBranch(), pid, ajo, creator);
					}
					if (condAction.getFalseBranch() instanceof ActionGroup) {
						patch((ActionGroup) condAction.getFalseBranch(), pid, ajo, creator);
					}
				}
			}
		}
	}

	/**
	 * Find the AbstractAction in _this_ AG that contains this "creator". If
	 * none return null. If it is here, return this AG
	 *  
	 */
	private static AbstractAction searchForTheCreator(ActionGroup action_group, AAIdentifier creator) {
		Enumeration e = action_group.getActions();
		while (e.hasMoreElements()) {
			AbstractAction a = (AbstractAction) e.nextElement();
			if (a.getId().equals(creator)) {
				return a;
			}
			else if (a instanceof ActionGroup) {
				if (searchForTheCreator((ActionGroup) a, creator) != null) {
					// Inside this AG => return AG not action
					return a;
				}
			}
			else if (a instanceof ConditionalAction) {
				ConditionalAction c = (ConditionalAction) a;
				if (c.getTrueBranch() != null) {
					if (c.getTrueBranch().getId().equals(creator)) {
						return a;
					}
					if (c.getTrueBranch() instanceof ActionGroup) {
						if (searchForTheCreator((ActionGroup) c.getTrueBranch(), creator) != null) {
							return a;
						}
					}
				}
				if (c.getFalseBranch() != null) {
					if (c.getFalseBranch().getId().equals(creator)) {
						return a;
					}
					if (c.getFalseBranch() instanceof ActionGroup) {
						if (searchForTheCreator((ActionGroup) c.getFalseBranch(), creator) != null) {
							return a;
						}
					}
				}
			}
		}
		// All the way through without a creator, so not in this one
		return (AbstractAction) null;
	}

	public static void test(ActionGroup test_ajo) {
		try {
			// Run through the AJO finding all Portfolios and recodring how they
			// are used, if revealed etc
			AbstractActionParser ap = AbstractActionParser.create(test_ajo, new AJOIdentifier(
					"NSPACE", 1));
			// Look any Portfolios that are not revealed
			Iterator i = ap.portfolios();
			while (i.hasNext()) {
				PortfolioDetails pd = (PortfolioDetails) i.next();
				if (!pd.isRevealed() && pd.parent_ajo != AbstractActionParser.SPOOL_ID) {
					// NB SPOOL_ID marks Portfolios spooled by PutPortfolio
					// (most other spool interactions are not recorded)
					String to_print = "Portfolio is not revealed in <" + test_ajo.getId().getName()
							+ "> <" + pd.pid.getName() + "/"
							+ Integer.toHexString(pd.pid.getValue()) + "> in <"
							+ pd.parent_ajo.getName() + "/"
							+ Integer.toHexString(pd.parent_ajo.getValue()) + ">";
					if (pd.created_by != null) {
						to_print += " was created by <" + pd.created_by.getName() + "/"
								+ Integer.toHexString(pd.created_by.getValue()) + ">";
					}
					else {
						to_print += " was NOT created !";
					}
					Iterator j = pd.used_by.iterator();
					while (j.hasNext()) {
						to_print += " ," + ((AAIdentifier) j.next()).getName();
					}
					logger.info(to_print);
					patch(test_ajo, pd.pid, pd.parent_ajo, pd.created_by);
				}
			}
			AbstractActionParser ap_test = AbstractActionParser.create(test_ajo, new AJOIdentifier(
					"NSPACE", 1));
			Iterator i_test = ap_test.portfolios();
			while (i_test.hasNext()) {
				PortfolioDetails pd_test = (PortfolioDetails) i_test.next();
				if (!pd_test.isRevealed() && pd_test.parent_ajo != AbstractActionParser.SPOOL_ID) {
					logger.log(Level.SEVERE, "Patch went wrong ?#??!$%");
				}
			}
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "Problems parsing AJO", ex);
		}
	}
}

class AbstractActionParser {

	public static AJOIdentifier SPOOL_ID = new AJOIdentifier("SPOOLED", 0);

	/**
	 * Return the correct parser for the AbstractAction typoe
	 * 
	 *  
	 */
	public static AbstractActionParser create(AbstractAction action, AJOIdentifier parent_ajo) {
		if (action instanceof AbstractService) {
			// catch the lot
			return new NoPortfolios(action, parent_ajo);
		}
		// catch the lot
		else if (action instanceof FileAction) {
			return new NoPortfolios(action, parent_ajo);
		}
		else if (action instanceof IncarnateFiles) {
			return new NoPortfolios(action, parent_ajo);
		}
		else if (action instanceof DeclarePortfolio) {
			return new DeclarePortfolioParser((DeclarePortfolio) action, parent_ajo);
		}
		else if (action instanceof DeletePortfolio) {
			return new DeletePortfolioParser((DeletePortfolio) action, parent_ajo);
		}
		else if (action instanceof GetPortfolio) {
			return new GetPortfolioParser((GetPortfolio) action, parent_ajo);
		}
		else if (action instanceof PutPortfolio) {
			return new PutPortfolioParser((PutPortfolio) action, parent_ajo);
		}
		else if (action instanceof Dummy) {
			return new NoPortfolios(action, parent_ajo);
		}
		else if (action instanceof MakePortfolio) {
			return new MakePortfolioParser((MakePortfolio) action, parent_ajo);
		}
		// Before ExecuteTask
		else if (action instanceof LinkTask) {
			return new LinkTaskParser((LinkTask) action, parent_ajo);
		}
		// Before CompileTask
		else if (action instanceof FortranTask) {
			return new FortranTaskParser((FortranTask) action, parent_ajo);
		}
		// Before ExecuteTask
		else if (action instanceof CompileTask) {
			return new CompileTaskParser((CompileTask) action, parent_ajo);
		}
		// Also does ExecuteScriptTask, note before ExecuteTask
		else if (action instanceof UserTask) {
			return new UserTaskParser((UserTask) action, parent_ajo);
		}
		else if (action instanceof ExecuteTask) {
			return new ExecuteTaskParser((ExecuteTask) action, parent_ajo);
		}
		else if (action instanceof ImportTask) {
			return new ImportTaskParser((ImportTask) action, parent_ajo);
		}
		else if (action instanceof ExportTask) {
			return new ExportTaskParser((ExportTask) action, parent_ajo);
		}
		// Do before ActionGroup (subtype)
		else if (action instanceof AbstractJob) {
			return new AbstractJobParser((AbstractJob) action, parent_ajo);
		}
		// Also does RepeatGroup and ForGroup
		else if (action instanceof ActionGroup) {
			return new ActionGroupParser((ActionGroup) action, parent_ajo);
		}
		// Does If and IfSuccess
		else if (action instanceof ConditionalAction) {
			return new ConditionalActionParser((ConditionalAction) action, parent_ajo);
		}
		else if (action instanceof CopyPortfolioToOutcome) {
			return new CopyPortfolioToOutcomeParser((CopyPortfolioToOutcome) action, parent_ajo);
		}
		//assert false: "Class not mapped to parser <"+action.getClass()+">";
		return null;
	}
	private Map portfolios;

	/**
	 * Constructor for the AbstractActionParser object
	 */
	protected AbstractActionParser() {
		portfolios = new HashMap();
	}

	/**
	 * Add a description of a Portfolio for an AbstractAction that creates it.
	 * This call is for the AAs that create a Portfolio hidden.
	 *  
	 */
	protected void addNewHidden(PortfolioIdentifier pid, AJOIdentifier parent_ajo,
			AAIdentifier created_by) {
		/*
		 * assert pid != null:"Null PortfolioIdentifier"; assert parent_ajo !=
		 * null:"Null parent AJO identifier"; assert created_by != null:"Null
		 * created identifier";
		 */
		PortfolioDetails pf = new PortfolioDetails(pid, parent_ajo);
		pf.created_by = created_by;
		addPD(pf);
	}

	/**
	 * Add a description of a Portfolio for an AbstractAction that creates it.
	 * This call is for teh AAs that create a Portfolio revealed.
	 *  
	 */
	protected void addNewRevealed(PortfolioIdentifier pid, AJOIdentifier parent_ajo,
			AAIdentifier created_by) {
		/*
		 * assert pid != null:"Null PortfolioIdentifier"; assert parent_ajo !=
		 * null:"Null parent AJO identifier"; assert created_by != null:"Null
		 * created identifier";
		 */
		PortfolioDetails pf = new PortfolioDetails(pid, parent_ajo);
		pf.created_by = created_by;
		pf.revealed = true;
		addPD(pf);
	}

	// since we need to get instances out
	/**
	 * Add a PortfolioDescription to out current list. If the Portfolio is
	 * already there merge the two lots of information, otherwise just addit.
	 *  
	 */
	protected void addPD(PortfolioDetails pd) {
		if (portfolios.containsKey(pd)) {
			// Already there, need to merge info (into existing)
			PortfolioDetails existing_pd = (PortfolioDetails) portfolios.get(pd);
			if (existing_pd.created_by == null) {
				// Different creating AAs in conditional branches
				existing_pd.created_by = pd.created_by;
			}
			Iterator i = pd.used_by.iterator();
			while (i.hasNext()) {
				existing_pd.used_by.add(i.next());
			}
			if (pd.revealed) {
				existing_pd.revealed = true;
			}
		}
		else {
			portfolios.put(pd, pd);
			// new one
		}
	}

	/**
	 * Add a decsription of a Portfolio for an AbstractAction that uses the
	 * Portfolio. Using a Portfolio will reveal it in teh Uspace.
	 * 
	 *  
	 */
	protected void addUsed(PortfolioIdentifier pid, AJOIdentifier parent_ajo, AAIdentifier used_by) {
		/*
		 * assert pid != null:"Null PortfolioIdentifier"; assert parent_ajo !=
		 * null:"Null parent AJO identifier"; assert used_by != null:"Null user
		 * identifier";
		 */
		PortfolioDetails pf = new PortfolioDetails(pid, parent_ajo);
		pf.used_by.add(used_by);
		addPD(pf);
	}

	protected Iterator portfolios() {
		return portfolios.keySet().iterator();
	}

	/**
	 * Look through the AbstractTask's resources for an AlternativeUspace
	 * instance. If found, then return the AJOIdentifier of the AUS, otherwise
	 * return the supplied default AJOIdentifier.
	 * 
	 *  
	 */
	protected AJOIdentifier scanForAUS(AbstractTask at, AJOIdentifier default_id) {
		AJOIdentifier return_value = default_id;
		if (at.getResources() != null) {
			ResourceEnumeration re = at.getResources().elements();
			while (re.hasMoreElements()) {
				Resource r = re.nextElement();
				if (r instanceof AlternativeUspace) {
					return_value = ((AlternativeUspace) r).getUspace();
				}
			}
		}
		return return_value;
	}

	/**
	 * Look through the AbstractTask's resources for an AlternativeUspace or
	 * Spool instance. If found, then return the AJOIdentifier of the AUS or the
	 * dummy Spool identifier, otherwise return the supplied default
	 * AJOIdentifier.
	 * 
	 *  
	 */
	protected AJOIdentifier scanForAUSorSpool(AbstractTask at, AJOIdentifier default_id) {
		AJOIdentifier return_value = default_id;
		if (at.getResources() != null) {
			ResourceEnumeration re = at.getResources().elements();
			while (re.hasMoreElements()) {
				Resource r = re.nextElement();
				if (r instanceof AlternativeUspace) {
					return_value = ((AlternativeUspace) r).getUspace();
				}
				if (r instanceof org.unicore.resources.Spool) {
					return_value = SPOOL_ID;
				}
			}
		}
		return return_value;
	}
}

class AbstractJobParser extends ActionGroupParser {

	public AbstractJobParser(AbstractJob sub_ajo, AJOIdentifier parent_ajo) {
		super(sub_ajo, sub_ajo.getAJOId());
		// parse with sub-AJO as the parent
		//assert parent_ajo != null;
		// Check for Portfolios streamed with this AJO and create
		// "push" instance of PortfolioDetails for them so that we can
		// match with the using AJO's instances (and later check for reveal)
		if (sub_ajo.getStreamed() != null) {
			PortfolioEnumeration i = sub_ajo.getStreamed().elements();
			while (i.hasMoreElements()) {
				Portfolio p = i.nextElement();
				addNewHidden(p.getId(), sub_ajo.getAJOId(), parent_ajo);
				// in the sub-AJO, created by parent HIDDEN
				addUsed(p.getId(), parent_ajo, sub_ajo.getId());
				// in the parent AJO, used by the sub-AJO
			}
		}
	}
}

class ActionGroupParser extends AbstractActionParser {

	public ActionGroupParser(ActionGroup ag, AJOIdentifier parent_ajo) {
		super();
		Enumeration actions = ag.getActions();
		while (actions.hasMoreElements()) {
			AbstractActionParser ap = AbstractActionParser.create((AbstractAction) actions
					.nextElement(), parent_ajo);
			// Merge the two, updating those for which we already have details
			Iterator i = ap.portfolios();
			while (i.hasNext()) {
				addPD((PortfolioDetails) i.next());
			}
		}
	}
}

class CompileTaskParser extends ExecuteTaskParser {

	public CompileTaskParser(CompileTask ct, AJOIdentifier parent_ajo) {
		super(ct, parent_ajo);
		// Parse for super class values
		addNewRevealed(ct.getObjects().getId(), parent_ajo, ct.getId());
		if (ct.getListingLevel() != ListingLevel.NONE) {
			addNewRevealed(ct.getListings().getId(), parent_ajo, ct.getId());
		}
		if (ct.getSources() != null) {
			PortfolioEnumeration pe = ct.getSources().elements();
			while (pe.hasMoreElements()) {
				addUsed(pe.nextElement().getId(), parent_ajo, ct.getId());
			}
		}
		if (ct.getIncludes() != null) {
			PortfolioEnumeration pe = ct.getIncludes().elements();
			while (pe.hasMoreElements()) {
				addUsed(pe.nextElement().getId(), parent_ajo, ct.getId());
			}
		}
	}
}

class ConditionalActionParser extends AbstractActionParser {

	public ConditionalActionParser(ConditionalAction ca, AJOIdentifier parent_ajo) {
		super();
		if (ca.getTrueBranch() != null) {
			Iterator i = AbstractActionParser.create(ca.getTrueBranch(), parent_ajo).portfolios();
			while (i.hasNext()) {
				addPD((PortfolioDetails) i.next());
			}
		}
		if (ca.getFalseBranch() != null) {
			Iterator i = AbstractActionParser.create(ca.getFalseBranch(), parent_ajo).portfolios();
			while (i.hasNext()) {
				addPD((PortfolioDetails) i.next());
			}
		}
	}
}

class CopyPortfolioToOutcomeParser extends AbstractActionParser {

	public CopyPortfolioToOutcomeParser(CopyPortfolioToOutcome cp, AJOIdentifier parent_ajo) {
		super();
		addUsed(cp.getTarget(), scanForAUSorSpool(cp, parent_ajo), cp.getId());
	}
}

class DeclarePortfolioParser extends AbstractActionParser {

	public DeclarePortfolioParser(DeclarePortfolio dp, AJOIdentifier parent_ajo) {
		super();
		addUsed(dp.getPortfolio().getId(), scanForAUS(dp, parent_ajo), dp.getId());
	}
}

class DeletePortfolioParser extends AbstractActionParser {

	public DeletePortfolioParser(DeletePortfolio dp, AJOIdentifier parent_ajo) {
		super();
		addUsed(dp.getPortfolio(), scanForAUSorSpool(dp, parent_ajo), dp.getId());
	}
}

class ExecuteTaskParser extends AbstractActionParser {

	public ExecuteTaskParser(ExecuteTask et, AJOIdentifier parent_ajo) {
		super();
		if (et.stdoutIsRedirected()) {
			addNewRevealed(et.getStdout().getId(), parent_ajo, et.getId());
		}
		if (et.stderrIsRedirected()) {
			addNewRevealed(et.getStderr().getId(), parent_ajo, et.getId());
		}
		if (et.getStdin() != null) {
			addUsed(et.getStdin().getId(), parent_ajo, et.getId());
		}
	}
}

class ExportTaskParser extends AbstractActionParser {

	public ExportTaskParser(ExportTask et, AJOIdentifier parent_ajo) {
		super();
		addUsed(et.getPortfolio().getId(), parent_ajo, et.getId());
	}
}

class FortranTaskParser extends CompileTaskParser {

	public FortranTaskParser(FortranTask ft, AJOIdentifier parent_ajo) {
		super(ft, parent_ajo);
		// Parse for super class values
		if (ft.getPreprocessLevel() != PreprocessLevel.NONE) {
			addNewRevealed(ft.getPreprocessedFiles().getId(), parent_ajo, ft.getId());
		}
	}
}

class GetPortfolioParser extends AbstractActionParser {

	public GetPortfolioParser(GetPortfolio gp, AJOIdentifier parent_ajo) {
		super();
		addNewRevealed(gp.getTarget().getId(), parent_ajo, gp.getId());
		addUsed(gp.getSource(), scanForAUS(gp, SPOOL_ID), gp.getId());
	}
}

class ImportTaskParser extends AbstractActionParser {

	public ImportTaskParser(ImportTask it, AJOIdentifier parent_ajo) {
		super();
		addNewRevealed(it.getPortfolio().getId(), parent_ajo, it.getId());
	}
}

class LinkTaskParser extends ExecuteTaskParser {

	public LinkTaskParser(LinkTask lt, AJOIdentifier parent_ajo) {
		super(lt, parent_ajo);
		// Parse for super class values
		addNewRevealed(lt.getExecutable().getId(), parent_ajo, lt.getId());
		if (lt.getMapLevel() != LinkMapLevel.NONE) {
			addNewRevealed(lt.getMap().getId(), parent_ajo, lt.getId());
		}
		if (lt.getLibraries() != null) {
			PortfolioEnumeration pe = lt.getLibraries().elements();
			while (pe.hasMoreElements()) {
				addUsed(pe.nextElement().getId(), parent_ajo, lt.getId());
			}
		}
		if (lt.getObjects() != null) {
			PortfolioEnumeration pe = lt.getObjects().elements();
			while (pe.hasMoreElements()) {
				addUsed(pe.nextElement().getId(), parent_ajo, lt.getId());
			}
		}
	}
}

class MakePortfolioParser extends AbstractActionParser {

	public MakePortfolioParser(MakePortfolio mp, AJOIdentifier parent_ajo) {
		super();
		addNewRevealed(mp.getPortfolio().getId(), scanForAUS(mp, parent_ajo), mp.getId());
	}
}

class NoPortfolios extends AbstractActionParser {

	public NoPortfolios(AbstractAction aa, AJOIdentifier parent_ajo) {
		super();
	}
}

class PortfolioDetails {

	/**
	 * The AbstractAction that created the Portfolio (could be another AJO
	 * through streaming, PutPortfolio or Transfer).
	 * <p>
	 * 
	 * Creation thus means making the files in the Uspace (revealed or hidden)
	 */
	public AAIdentifier created_by;
	/**
	 * The AbstractJob where this Portfolio is used
	 */
	public final AJOIdentifier parent_ajo;
	/**
	 * The Identifier of the Portfolio (note that this is unique within an AJO,
	 * not globally)
	 */
	public final PortfolioIdentifier pid;
	public boolean revealed = false;
	/**
	 * The AbstractActions within the AJO that explicitly use this Portfolio
	 * (and so force the reveal of any hidden Portfolios).
	 * <p>
	 * 
	 * Note that DeclarePortfolio is considered a user in this code.
	 */
	public final Set used_by;

	public PortfolioDetails(PortfolioIdentifier pid, AJOIdentifier parent_ajo) {
		/*
		 * assert pid != null; assert parent_ajo != null;
		 */
		this.pid = pid;
		this.parent_ajo = parent_ajo;
		used_by = new HashSet();
	}

	/**
	 * Make this globally unique so test for Portfolio Id and AJO Id.
	 *  
	 */
	public boolean equals(Object o) {
		if (o instanceof PortfolioDetails) {
			PortfolioDetails test = (PortfolioDetails) o;
			return (pid.equals(test.pid) && parent_ajo.equals(test.parent_ajo));
		}
		else {
			return false;
		}
	}

	public int hashCode() {
		return pid.hashCode();
	}

	public boolean isRevealed() {
		return revealed || used_by.size() > 0;
	}
}

class PutPortfolioParser extends AbstractActionParser {

	public PutPortfolioParser(PutPortfolio pp, AJOIdentifier parent_ajo) {
		super();
		addUsed(pp.getSource().getId(), parent_ajo, pp.getId());

		// look for alternative uspace
		AJOIdentifier dest_parent = scanForAUS(pp, SPOOL_ID);

		if (pp.getTarget() != null) {
			addNewHidden(pp.getTarget(), dest_parent, pp.getId());
		}
	}
}

class UserTaskParser extends ExecuteTaskParser {

	public UserTaskParser(UserTask ut, AJOIdentifier parent_ajo) {
		super(ut, parent_ajo);
		// Parse fro super class values
		if (ut.getExecutable() != null) {
			addUsed(ut.getExecutable().getId(), parent_ajo, ut.getId());
		}
	}
}