/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.utility;

/**
 * Keep the version info
 * 
 * @author Thomas Kentemich
 * @version $Id: Version.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class Version {

	private static String _build = "5";
	private static String _date = "June 24, 2004";
	private static String _version = "5.0";

	/**
	 * Get version without build info
	 * 
	 * @return version without build info
	 */
	public final static String dotVersion() {
		return _version;
	}

	/**
	 * Get version including build info
	 * 
	 * @return version including build info
	 */
	public final static String fullVersion() {
		return _version + " build " + _build + " application edition";
	}

	/**
	 * Get build date
	 * 
	 * @return build date
	 */
	public static String getBuildDate() {
		return _date;
	}

	/**
	 * get build number
	 * 
	 * @return build number
	 */
	public static String getBuildNumber() {
		return _build;
	}

	/**
	 * current version
	 * 
	 * @param complete
	 *            true if build included
	 * @return current version string
	 */
	public static String getCurrentVersion(boolean complete) {
		if (!complete) {
			return _version;
		}
		return _version + " b" + _build;
	}

	/**
	 * get complete version info
	 * 
	 * @return complete version info
	 */
	public final static String spcVersion() {
		return _version + " build " + _build;
	}
}