package org.unicore;

import java.security.cert.X509Certificate;

/**
 * User describes a user of the Unicore system.
 * <p>
 * A user is a person with a recognised X509 certificate
 * and a particular user name (Xlogin) for a Vsite.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9
 *
 * @version User.java 1.6 05/13/02 
 * 
 **/
public class User extends Ulogin {

    /**
     * Create an uninitialised User
     *
     **/
    public User() {
        super();
        this.email_address = null;
        this.xlogin = null;
    }

    /**
     * Create a new User.
     *
     * @param The X509 certificate identifying this person
     * @param email_address An address to send messages to
     *
     * @throws java.security.cert.CertificateException There was a problem encoding the certificate.
     *
     **/
    public User( X509Certificate certificate,
                 String email_address ) throws java.security.cert.CertificateException {
        super(certificate);
        this.email_address = email_address;
        this.xlogin = null;
    }

    private String email_address;
    /**
     * Return the email address is used to send information from
     * the NJS to the user. 
     *
     **/
    public String getEmailAddress() {return email_address;}

    /**
     * Set the user's email address.
     *
     **/
    public void setEmailAddress(String email_address) {this.email_address = email_address;}

    private String xlogin;
    /**
     * Return the user's choice of the Xlogin to use at this Vsite. 
     * <p>
     * An Xlogin is a user name at a Vsite.
     * <p>
     * If the user leaves this empty, then the NJS uses the default value
     * for the certificate.
     * <p>
     * If the user sets a value, then this is used if there is
     * an entry in the UUDB that allows the certificate to be mapped to that value.
     *
     **/
    public String getXlogin() {return xlogin;}

    /**
     * Set the user's choice of Xlogin to use at this Vsite. 
     *
     **/
    public void setXlogin(String alogin) {xlogin = alogin;}

    private String project;
    /**
     * Return the user's choice of the project to use at a Vsite. 
     * <p>
     * If the user leaves this empty, then it is set at the Vsite to the default
     * project for the selected Xlogin.
     * <p>
     * If the user sets a value, then this is used by the NJS if there is
     * an entry in the UUDB that allows the certificate and Xlogin to use this project.
     *
     **/
    public String getProject() {return project;}

    /**
     * Set the user's choice of project to use at a Vsite. 
     *
     **/
    public void setProject(String project) {this.project = project;}

    static final long serialVersionUID = -3813634978992005538L;

}
//
// Change history:
//    8 Mar 2001 SvdB Unfinalised, to allow extension by NJSs etc
//   30 Apr 2000 SvdB moved to User for V3.0,
//                    added project field  
//    4 Feb 2000 SvdB created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
