package org.unicore.ajo;

import org.unicore.sets.OptionSet;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.PortfolioList;
import org.unicore.sets.PortfolioSet;

/**
 * ExecuteScripTask executes a script.
 * <p>
 * @see org.unicore.outcome.ExecuteScriptTask_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2(ish)
 *
 * @version ExecuteScriptTask.java 1.5 09/09/02 
 * 
 **/
public final class ExecuteScriptTask extends UserTask {

    static final long serialVersionUID = -4272908536091797835L;

    /**
     * Create a new ExecuteScriptTask.
     * <p>
     * @param name The name of the ExecuteScriptTask
     * @param extra_information Client supplied extra information about the ExecuteScriptTask
     * @param resources Resources needed to execute the ExecuteScriptTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param executable File to execute
     * @param measure_time Report exection time?
     * @param run_for_debug Debug?
     * @param run_for_profile Profile?
     * @param script_type Script language used fot the script
     *
     **/
    public ExecuteScriptTask ( String name,
                     byte[] extra_information,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     Portfolio executable,
                     boolean measure_time,
                     boolean run_for_debug,
                     boolean run_for_profile,
                     ScriptType script_type
                   ) {

        super(name,extra_information,resource_limits,
              environment_variables,
              command_line,stdin, redirect_stdout, redirect_stderr,verbose,version,
              executable,measure_time, run_for_debug,run_for_profile);

        this.script_type = script_type;
    }

    /**
     * Create a new ExecuteScriptTask.
     * <p>
     * @param name The name of the ExecuteScriptTask
     * @param resources Resources needed to execute the ExecuteScriptTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param executable File to execute
     * @param measure_time Report exection time?
     * @param run_for_debug Debug?
     * @param run_for_profile Profile?
     * @param script_type Script language used fot the script
     *
     **/
    public ExecuteScriptTask ( String name,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     Portfolio executable,
                     boolean measure_time,
                     boolean run_for_debug,
                     boolean run_for_profile,
                     ScriptType script_type
                   ) {

        this(name,(byte[]) null,resource_limits,
              environment_variables,
              command_line,stdin, redirect_stdout, redirect_stderr,verbose,version,
              executable,measure_time, run_for_debug,run_for_profile, script_type);
    }

    /**
     * Create a new ExecuteScriptTask.
     * <p>
     * @param name The name of the ExecuteScriptTask
     *
     **/
    public ExecuteScriptTask (String name) {
        this(name,(byte[]) null,(ResourceSet) null,
              (OptionSet) null,
              (String) null, (Portfolio) null, false, false, false, false,
              (Portfolio) null,  false,  false,  false, ScriptType.SH);
    }

    public ExecuteScriptTask () {
        this("");
    }

    private ScriptType script_type;
    /**
     * Get the scripting language in which the script
     * is written.
     *
     **/
    public ScriptType getScriptType() { return script_type;}

    /**
     * Set the scripting language in which the script
     * is written.
     *
     **/
    public void setScriptType(ScriptType script_type) {this.script_type = script_type;}

}
//
// Change history:
//          11 Apr 2002 SvdB Removed deprecated constructor
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//          29 Mar 1999 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
