package org.unicore.ajo;

import org.unicore.Unicore;

/**
 * This object represents the restricted form of expression allowed in
 * If, RepeatGroup and FileCheck actions.
 * <p>
 * There is no evlaution of an expression, this will be done on
 * the server side and code implemented there will honour the contract
 * described here.
 * <p>
 * The value held by an Expression forms the right hand term of the
 * expression.
 *
 * @see If
 * @see RepeatGroup
 * @see FileCheck
 *
 * @author David F Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version Expression.java 1.5 06/11/02
 *
 **/
 
public final class Expression extends Unicore {

    static final long serialVersionUID = 0;

    public Expression () {
        this(null,null);
    }

    /**
     * Constructor takes a relation and the value to be used on the right hand
     * side of the expession evaluation.
     *
     **/
    public Expression (Relations relation, Object value) {
        if(relation != null) this.relation = relation;
        this.value    = value;
    }
    
    protected Object value;

    public Object getValue() {return value;}

    private Relations relation = Relations.EQ;

    public Relations getRelation() {return relation;}
    
}
//
// Change history:
//        18 Mar 2002 SvdB V4 no arg constructor and other mnodifications
//        31 Jan 2001  DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
