package org.unicore.ajo;

import java.util.Enumeration;

/**
 * The types of use to which a file imported into
 * a Uspace may be put. 
 * <p>
 * It may be possible
 * to perform some optmisations given information about the
 * usage of the file within Uspace (e.g read only access
 * does not need a copy).
 *
 * @deprecated AJO 4.0 Any transfer optimisations aill be coded into AJOs (e.g. {@link SymbolicLink})
 *
 * @see ImportTask
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version FileImportStatus.java 1.4 05/13/02 
 *
 **/
public final class FileImportStatus extends Enum {

    static final long serialVersionUID = -3408666550471275389L;

    /**
     * The user will write to the Uspace version of the files but 
     * does not wish to change the original files.
     * <p>
     * The incarnation of the Import 
     * must copy the file into Uspace and may not optimise it away.
     *
     **/
    public final static FileImportStatus 
            TAKECOPY = new FileImportStatus(0,"Copy file into Uspace");

    /**
     * The user will not write to the Uspace version of these files.
     * <p>
     * So, an incarnation may optimise by linking rather than copying.
     *
     **/
    public final static FileImportStatus 
            READONLY = new FileImportStatus(1,"Read only access");

    /**
     * The user intends to change the external version of the
     * file. 
     * <p>
     * This Import will be matched by an Export and
     * incarnations can optimise the Import and Export by a link.
     *
     **/
    public final static FileImportStatus 
            MODIFY = new FileImportStatus(2,"Modify access");

    /**
     * Constructor for this Class.
     *
     * @param aValue The value for this instance.
     * @param aName The name for this instance.
     *
     **/
    private FileImportStatus(int aValue, String aName) {
        super(aValue,aName);
    }

    /**
     * Return a enumeration over the instances of FileImportStatus
     *
     * @return The enumeration
     *
     **/
    public static Enumeration elements() {
        return Enum.elements( TAKECOPY.getClass() );
    }

}
//
// Change history:
//          22 Apr 2002 SvdB Deprecated
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           1 Oct 1998 SvdB Changed values to indexes (their point) 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
