package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Incarnate a Fifo. Following this task an application can
 * read or write to the named fifo. 
 * <p>
 * The Fifo is created in the Uspace named by the
 * instance of {@link org.unicore.resources.USpace} or
 * {@link org.unicore.resources.AlternativeUspace}
 *
 * @see Fifo
 * @see org.unicore.outcome.MakeFifo_Outcome
 *
 * @author S. van den Berghe (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 4.0
 *
 * @version MakeFifo.java 1.2 09/18/02 
 * 
 **/
public class MakeFifo extends FileTask {

    static final long serialVersionUID = 0;

    /**
     * Create a new MakeFifo.
     * <p>
     * The name is also given to the created Fifo
     * (this can be changed using {@link Fifo#setName}).
     *
     * @param name The name of the task and incarnated Fifo
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param overwrite Whether the fifo should overwrite an existing file
     *
     **/
    public MakeFifo( String name,
                     byte[] extra_information,
                     ResourceSet resources,
                     boolean overwrite
                              ) {
        super(name,extra_information,resources);
        this.overwrite = overwrite;
    }

    /**
     * Create a new MakeFifo with this name and
     * no files in the Fifo.
     * <p>
     * The name is also given to the created fifo
     * (this can be changed using {@link Fifo#setName}).
     *
     **/
    public MakeFifo(String name) {
        this(name, (byte[]) null, (ResourceSet) null, true);
    }

    public MakeFifo() {
        this("");
    }

    private Fifo fifo;
    /**
     * Return the Fifo created by this task.
     *
     **/
    public Fifo getFifo() {
        if(fifo == null) fifo = new Fifo(getName());
        return fifo;
    }

    boolean overwrite;
    /**
     * Overwrite existing destination files?
     * <p>
     * If no overwriting is allowed (this method returns false), then the task will
     * fail if the copy would overwrite an existing file.
     * <p>
     * Overwriting existing files is always
     * subject to permissions on the existing files and the task
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}


}
//
// Change history:
//          23 Apr 2002 SvdB Created from CreatePortfolioTask 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
