package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Change the name of a file or directory.
 * <p>
 * For details of where the from and to files are located see {@link FileAction}
 * <p>
 * The from and to files are always on the same Storage
 *
 * @see org.unicore.outcome.RenameFile_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version RenameFile.java 1.8 09/18/02
 * 
 **/
public final class RenameFile extends FileAction {

    static final long serialVersionUID = 5595384274422779368L;

    /**
     * Create a new RenameFile.
     * <p>
     * @param name The name of RenameFile 
     * @param extra_information Client supplied extra information about the RenameFile
     * @param resources Resources needed to execute the RenameFile
     * @param from The name of the file to rename
     * @param to The destination file name
     * @param overwrite Whether the rename should overwrite an existing file
     *
     **/
    public RenameFile( String name,
                       byte[] extra_information,
                       ResourceSet resources,
                       String from,
                       String to,
                       boolean overwrite
                     ) {

        super(name,extra_information,resources);

        this.from = from;
        this.to = to;
        this.overwrite = overwrite;
    }

    /**
     * Create a new RenameFile.
     * <p>
     * @param name The name of RenameFile 
     * @param resources Resources needed to execute the RenameFile
     * @param from The name of the file to rename
     * @param to The destination file name
     * @param overwrite Whether the rename should overwrite an existing file
     *
     **/
    public RenameFile( String name,
                       ResourceSet resources,
                       String from,
                       String to,
                       boolean overwrite
                     ) {

        this(name,(byte[]) null, resources, from, to, overwrite);
    }

    /**
     * Create a new RenameFile.
     * <p>
     * @param name The name of RenameFile 
     *
     **/
    public RenameFile( String name) {
        this(name,(byte[]) null,(ResourceSet) null, (String) null, (String) null, false);
    }

    public RenameFile() {
        this("");
    }

    private String from;
    /**
     *  Return original name of the file.
     *
     **/
    public String getFrom() {return from;}

    /**
     *  Set original name of the file.
     *
     **/
    public void setFrom(String from) {this.from = from;}

    private String to;
    /**
     *  Return final name of the file.
     *
     **/
    public String getTo() {return to;}

    /**
     *  Set final name of the file.
     *
     **/
    public void setTo(String to) {this.to = to;}

    boolean overwrite;
    /**
     * Overwrite a existing destination file?
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}


}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          12 Jun 2000 Changed name from Rename 
//           5 May 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
