package org.unicore.ajo;

import org.unicore.TicketIdentifier;

/**
 * A Ticket object serves as a reference for users, Unicore sites and Unicore
 * Resource brokers for resource related transactions amongst them, such as 
 * queries for available resources and booking of resources.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.6
 *
 * @version Ticket.java 1.4 06/13/02
 * 
 **/
public class Ticket extends org.unicore.Unicore {

    static final long serialVersionUID = -2210240845285391936L;

    private TicketIdentifier identifier;

    private java.util.Date expiry;

    public Ticket () {
        this("",null);
    }

    /**
     * Create a new (unique) Ticket.
     *
     * @param name The name of the Ticket (convenience only).
     * @param expiry The time at which this ticket will expire.
     *
     **/
    public Ticket (String name, java.util.Date expiry) {
        this.identifier = new TicketIdentifier(name);
        this.expiry = expiry;
    }

    /**
     * Return the identifier of this Ticket
     *
     **/
    public TicketIdentifier getId() {return identifier;}

    /**
     * Return the name of this Ticket
     *
     **/
    public String getName() {return identifier.getName();}

    /**
     * Return the expiry time of this ticket.
     *
     **/
    public java.util.Date getExpiryTime() {return expiry;}

    /**
     * Equality of Tickets is determined by equality of their Identifiers.
     *
     **/
    public boolean equals(Object in) {
        return( in.getClass() ==  this.getClass() &&
                getId().equals(((Ticket)in).getId()) 
              );
    }

    /**
     * Return a hash code based on the Identifier value.
     *
     **/
    public int hashCode() {return getId().hashCode();}

}
//
// Change history:
//          03 May 2002 DFS  Expiry time added.
//          18 Mar 2002 SvdB V4 no arg constrcutor
//          11 Dec 2001 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
