package org.unicore.ajo;

import org.unicore.sets.OptionSet;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.PortfolioList;
import org.unicore.sets.PortfolioSet;

/**
 * UserTask executes the supplied executable.
 * <p>
 * <b>NOTE:</b> This Class does not specify any input files
 *              for the executable. These are implicitly in the Uspace
 *              as the result of transfers, imports or other UserTasks.
 * <p>
 * @see org.unicore.outcome.UserTask_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version UserTask.java 1.6 09/09/02 
 * 
 **/
public class UserTask extends ExecuteTask {

    static final long serialVersionUID = 5659607852434413699L;

    /**
     * Create a new UserTask.
     * <p>
     * @param name The name of the UserTask
     * @param extra_information Client supplied extra information about the UserTask
     * @param resources Resources needed to execute the UserTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param executable File to execute
     * @param measure_time Report exection time?
     * @param run_for_debug Debug?
     * @param run_for_profile Profile?
     *
     **/
    public UserTask( String name,
                     byte[] extra_information,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     Portfolio executable,
                     boolean measure_time,
                     boolean run_for_debug,
                     boolean run_for_profile
                   ) {

        super(name,extra_information,resource_limits,
              environment_variables,
              command_line,stdin,redirect_stdout,redirect_stderr,verbose,version);

        this.executable = executable;
        this.measure_time = measure_time;
        this.run_for_debug = run_for_debug;
        this.run_for_profile = run_for_profile;
    }

    /**
     * Create a new UserTask.
     * <p>
     * @param name The name of the UserTask
     * @param resources Resources needed to execute the UserTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param executable File to execute
     * @param measure_time Report exection time?
     * @param run_for_debug Debug?
     * @param run_for_profile Profile?
     *
     **/
    public UserTask( String name,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     Portfolio executable,
                     boolean measure_time,
                     boolean run_for_debug,
                     boolean run_for_profile
                   ) {

        this(name,(byte[]) null,resource_limits,
              environment_variables,
              command_line,stdin,redirect_stdout,redirect_stderr,verbose,version,
              executable, measure_time, run_for_debug, run_for_profile);
    }

    /**
     * Create a new UserTask.
     * <p>
     * @param name The name of the UserTask
     *
     **/
    public UserTask(String name) {
        this(name,(byte[]) null,(ResourceSet) null,
              (OptionSet) null,
              (String) null, (Portfolio) null, false, false, false, false,
              (Portfolio) null,  false,  false,  false);
    }

    public UserTask() {
        this("");
    }

    private Portfolio executable;
    /**
     * Return the executable file to run for this task.
     * <p>
     * This Portfolio should incarnate to a single file.
     *
     **/
    public Portfolio getExecutable() {return executable;}

    /**
     * Set the executable file to run for this task.
     * 
     **/
    public void setExecutable(Portfolio executable) {this.executable = executable;}

    private boolean measure_time;
    /**
     * If true the Task will be incarnated so as to record the 
     * execution time and make it available to the user.
     *
     * @since AJO 3
     *
     **/
    public boolean isTimeMeasured() {return measure_time;}

    /**
     * Set time measuring policy.
     *
     **/
    public void setTimeMeasured(boolean measure_time) {this.measure_time = measure_time;}

    private boolean run_for_debug;
    /**
     * If this is true, then the executable should
     * be run to provide debugging information (including
     * analysing any core file) and send the results
     * to stdout.
     * <p>
     * The executable must have been
     * properly prepared for debugging. A run time error
     * occurs if this is not done.
     *
     * @since AJO 3
     *
     **/
    public boolean runForDebug() {return run_for_debug;}

    /**
     * Set debug policy.
     *
     **/
    public void setRunForDebug(boolean run_for_debug) {this.run_for_debug = run_for_debug;}

    private boolean run_for_profile;
    /**
     * If this is true, then the executable should
     * be run with profiling support, returning the
     * results on stdout.
     * <p>
     * The executable must have been
     * properly prepared for profiling. A run time error
     * occurs if this is not done.
     *
     * @since AJO 3
     *
     **/
    public boolean runForProfile() {return run_for_profile;}

    /**
     * Set profile policy.
     *
     **/
    public void setRunForProfile(boolean run_for_profile) {
        this.run_for_profile = run_for_profile;
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           4 May 2000 SvdB V3.0, dropped input 
//           4 Mar 1999   MD and initialised it! 
//          19 Jan 1999 SvdB Added getInput method for MD  
//           7 Dec 1998 SvdB Second release to Unicore 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
