package org.unicore.idiomatic;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.resources.*;

/**
 * Copy a file from one Xspace to another (on the same Vsite)
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version CopyXtoX.java 1.3 08/13/02 <br>
 * 
 **/
public class CopyXtoX extends ActionGroup implements Idiomatic {

    /**
     * Create a new CopyXtoX  to copy the file within a Vsite
     *
     * @param name A name for the CopyXtoX
     * @param extra_information Storage for data from the creator of this CopyXtoX
     * @param source_file The file to be copied
     * @param source_storage The {@link org.unicore.resources.Storage} where the file can be found
     * @param target_file The destination file name
     * @param target_storage The {@link org.unicore.resources.Storage} where the file is placed
     * @param overwrite Should the copy should overwrite an existing to file
     *
     **/
    public CopyXtoX( String name,
                     byte[] extra_information,
                     String source_file,
                     PathedStorage source_storage,
                     String target_file,
                     PathedStorage target_storage,
                     boolean overwrite
                   ) {
        super(name,extra_information);

        // Import the file
        import_it = new CopyFile(name+"-Import");
        import_it.setCopyFromUspace(false);

        String temp_file = "copyxtox.tempfile."+Integer.toHexString(import_it.getId().getValue());

        import_it.setTo(temp_file);
        import_it.setOverwrite(false);  // just in case

        setSourceFile(source_file);
        setSourceStorage(source_storage);

        // Export the file
        export_it = new CopyFile(name+"-Export");
        export_it.setCopyFromUspace(true);

        export_it.setFrom(temp_file);

        setTargetFile(target_file);
        setTargetStorage(target_storage);

        setOverwrite(overwrite);

        // -------- add to AJO
        add(import_it);
        add(export_it);
        add(new Dependency(import_it,export_it));

        // delete temp file
        delete_it = new DeleteFile(name+"-DeleteTempFile",(byte[])null,null,temp_file);
        add(delete_it);
        add(new Dependency(export_it,delete_it));

    }

    public CopyXtoX() {
        this("",(byte[]) null,(String) null,(PathedStorage) null,(String) null,(PathedStorage) null,false);
    }

    public CopyXtoX(String name) {
        this(name,(byte[]) null,(String) null,(PathedStorage) null,(String) null,(PathedStorage) null,false);
    }

    private CopyFile import_it;
    public CopyFile getImport() {return import_it;}

    private CopyFile export_it;
    public CopyFile getExport() {return export_it;}

    private DeleteFile delete_it;
    public DeleteFile getDelete() {return delete_it;}

    /**
     * @return The source file
     *
     **/
    public String getSourceFile() {return import_it.getFrom();}

    /**
     * @param source_file The new source file
     *
     **/
    public void setSourceFile(String source_file) {
        import_it.setFrom(source_file);
    }

    PathedStorage source_storage;
    /**
     * @return The source Storage
     *
     **/
    public PathedStorage getSourceStorage() {return source_storage;}

    /**
     * @param source_storage The new value for the source storage
     *
     **/
    public void setSourceStorage(PathedStorage source_storage) {
        import_it.removeResource(this.source_storage);
        import_it.addResource(source_storage);
        this.source_storage = source_storage;
    }

    PathedStorage target_storage;
    /**
     * @return The target Storage
     *
     **/
    public PathedStorage getTargetStorage() {return target_storage;}

    /**
     * @param target_storage The new value for the target storage
     *
     **/
    public void setTargetStorage(PathedStorage target_storage) {
        export_it.removeResource(this.target_storage);
        export_it.addResource(target_storage);
        this.target_storage = target_storage;
    }

    /**
     * @return The target file
     *
     **/
    public String getTargetFile() {return export_it.getTo();}

    /**
     * @param target_file The new target file
     *
     **/
    public void setTargetFile(String target_file) {
        export_it.setTo(target_file);
    }

    public boolean willOverwrite() {return export_it.willOverwrite();}

    public void setOverwrite(boolean overwrite) {export_it.setOverwrite(overwrite);}

    static final long serialVersionUID = 400;
}
//
//  Change history:
//          05 Jul 2002 SvdB Made temporary file unique
//          22 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
