package org.unicore.outcome;

import org.unicore.ajo.FileCheck;

/**
 * The result of executing a {@link org.unicore.ajo.FileCheck}.
 *
 * @author D. Snelling (Fujitsu Laboratories of Europe)
 * @author S. van den Berghe (Fujitsu Laboratories of Europe)
 * 
 * @since AJO 3
 *
 * @version FileCheck_Outcome.java 1.11 08/13/02 
 * 
 **/
public class FileCheck_Outcome extends AbstractTask_Outcome implements Decision {

    static final long serialVersionUID = 400;

    public static String SATISFIED     = "satisfied";
    public static String NOT_SATISFIED = "not satisfied";

    public FileCheck_Outcome() {
        this(null,null);
    }

    /**
     * Create a new FileCheck_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}.
     *
     * @param source The FileCheck for which this Outcome will hold the results.
     * @param initial The status of the FileCheck at the time that the Outcome is created.
     *
     **/
    public FileCheck_Outcome(FileCheck source, AbstractActionStatus initial_status) {
        super(source, initial_status);
    }

    private XFile file;

    public void setFile(XFile file) {this.file = file;}

    /**
     *
     * @return The actual values for the file
     *
     * @since AJO 4.0
     *
     **/
    public XFile getFile() {return file;}

    private String decision = null;
    /**
     * Return the decision produced by the FileCheck's execution.
     *
     * @return FileCheck_Outcome.SATISFIED or FileCheck_Outcome.NOT_SATISFIED or
     *         null (never execited, failure in incarnation)
     *
     * @since AJO 4.0
     *
     **/
    public String getDecision() {
        return decision;
    }

    /**
     * The conditions set on the file at the time of the FileCheck's 
     * execution were satisfied.
     *
     * @since AJO 4.0
     *
     **/
    public void satisfied() {
        decision = SATISFIED;
    }

    /**
     * The conditions set on the file at the time of the FileCheck's 
     * execution were not satisfied.
     *
     * @since AJO 4.0
     *
     **/
    public void notsatisfied() {
        decision = NOT_SATISFIED;
    }

}
//
// Change history:
//          08 Jul 2002 SvdB Decision
//          07 Jun 2002 SvdB Added File 
//          18 Mar 2002 SvdB V4 no arg constructor, now a TaskOutcome, no tests
//          10 Jan 2002 SvdB FileCheck moved from resources check, thus added OK methods
//          15 Aug 2000 SvdB Changed name of parent
//          15 May 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
