package org.unicore.outcome;

import org.unicore.ajo.GetActionStatus;

import org.unicore.AAIdentifier;

/**
 * The result of executing a {@link org.unicore.ajo.GetActionStatus}.
 * <p>
 * The results include the status of the target AbstractAction at
 * the time that the GetActionStatus was executed. If the target 
 * was an ActionGroup, then the status is the top-level status
 * and does not include the individual statuses of each child
 * of the ActionGroup.
 *
 * @see org.unicore.ajo.GetActionStatus
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version GetActionStatus_Outcome.java 1.6 05/13/02 
 * 
 **/
public class GetActionStatus_Outcome extends AbstractService_Outcome {

    static final long serialVersionUID = -1059796586992987423L;

    protected AbstractActionStatus status;

    protected AAIdentifier target_id;

    public GetActionStatus_Outcome() {
        this(null,null);
    }

    /**
     * Create a new GetActionStatus_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus the status of the target AbstractAction.
     *
     * @param source The GetActionStatus for which this Outcome will hold the results.
     * @param initial The status of the GetActionStatus at the time that the Outcome is created.
     *
     **/
    public GetActionStatus_Outcome(GetActionStatus      source,
                                   AbstractActionStatus initial_status) {
        super(source, initial_status);
    }

    /**
     * Return the status of the target AbstractAction.
     * <p>
     * Note that the status of executing the GetActionStatus is returned
     * in getStatus which will, amongst other things, be NOT_SUCCESSFUL
     * if the target AbstractAction does not exist.
     *
     **/
    public AbstractActionStatus getActionStatus() {return status;}

    /**
     * Set the status of the target AbstractAction.
     *
     **/
    public void setActionStatus(AbstractActionStatus o) {status = o;}

    /**
     * Return the Identifier of the target AbstractAction.
     *
     **/
    public AAIdentifier getActionIdentifier() {return target_id;}

    /**
     * Set the Identifier of the target AbstractAction.
     *
     **/
    public void setActionIdentifier(AAIdentifier i) {target_id = i;}

    private StatusHistory target_status_history;
    /**
     * Return the history of status changes of the target AbstractAction.
     *
     * @since AJO 4.0
     *
     **/
    public StatusHistory getTargetStatusHistory() {return target_status_history;}

    /**
     * Set the history of status changes of the target AbstractAction.
     *
     * @since AJO 4.0
     *
     **/
    public void setTargetStatusHistory(StatusHistory target_status_history) {this.target_status_history=target_status_history;}

}
//
// Change history:
//          12 Apr 2002 SvdB Addede get/setTargetStatusHistory
//          18 Mar 2002 SvdB V4 no arg constructor
//          15 Feb 2001 SvdB Added record of target Id
//           5 May 2000 SvdB V3.0 
//          12 Mar 1999 SvdB renamed methods not to override parent's 
//          10 Dec 1998 SvdB Constructor to public 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
