package org.unicore.outcome;

import org.unicore.Unicore;

import org.unicore.ajo.AbstractAction;
import org.unicore.AAIdentifier;

/**
 * @see org.unicore.ajo.If
 *
 * @author David F. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version If_Outcome.java 1.4 07/11/02
 * 
 **/
public class If_Outcome extends ConditionalAction_Outcome {

    static final long serialVersionUID = 0;
 
    public If_Outcome() {
        this((AbstractAction)null,null);
    }

    /**
     * Create a new If_Outcome.
     * <p>
     * @param source The If for which this Outcome will hold the results.
     * @param initial The status of the If at the time that the Outcome is created
     *
     **/
    public If_Outcome(AbstractAction source, AbstractActionStatus initial) {
      super (source, initial);
    }

    /**
     * Create a new If_Outcome.
     * <p>
     * @param source_id The AAIdentifier of the If for which this Outcome will hold the results.
     * @param initial The status of the If at the time that the Outcome is created
     *
     **/
    public If_Outcome(AAIdentifier source_id, AbstractActionStatus initial) {
      super (source_id, initial);
    }

    /**
     * Return true if the true branch of the If action was taken.
     * <p>
     * @return Boolean.TRUE if the true branch was taken, Boolean.FALSE if the false
     *         branch was taken and null otherwise (not yet executed or did not run)
     *
     **/
    public Boolean trueTaken () {
        if(getTakenOutcome() == null) return null;
        if(getTakenOutcome() == true_branch_outcome)  return Boolean.TRUE;
        if(getTakenOutcome() == false_branch_outcome) return Boolean.FALSE;
        return null;
    }

}
//
// Change history:
//           8 Juk 2002 SvdB Changed trueTaken to refer to results
//          18 Mar 2002 SvdB V4 no arg constructor
//          31 Jan 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
