package org.unicore.resources;

import org.unicore.AJOIdentifier;

/**
 * Resource to refer to the Uspaces of other AJOs (on the same Vsite
 * and owned by the same user)
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version AlternativeUspace.java 1.3 06/19/02 
 *
 **/
public class AlternativeUspace extends PathedStorage {

    static final long serialVersionUID = 0;

    public AlternativeUspace() {
        this("", 0.0, 0.0, 0.0, (String)null, (AJOIdentifier)null);
    }

    /**
     * Create a new AlternativeUspace.
     *
     * @param description A description of the AlternativeUspace
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param path The subdirectory within the AlternativeUspace (relative to the root of the AlternativeUspace)
     * @param uspace The (alternative) Uspace 
     *
     **/
    public AlternativeUspace(String description,
                             double maxrequest,
                             double defaultrequest,
                             double minrequest,
                             String path,
                             AJOIdentifier uspace) {
        super(description, maxrequest, defaultrequest, minrequest, path);
        setUspace(uspace);
    }

    private AJOIdentifier uspace;
    /**
     * Set the (alternative) Uspace
     *
     **/
    public void setUspace (AJOIdentifier uspace) { 
        this.uspace = uspace;
        setName("AlternativeUspace");
    }

    /**
     * @return The (alternative) Uspace (null means the current)
     *
     **/
    public AJOIdentifier getUspace() {return uspace;}

    /**
     * Return a hash code derived from the Alternative Uspace of the AlternativeUspace.
     *
     **/
    public int hashCode() {
        if(uspace == null) {
            return 2166282;
        }
        else {
            return uspace.hashCode();
        }
    }
}
//
// Change history:
//          21 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
