package org.unicore.resources;

/**
 * Describes information about the Vsite.
 * <p>
 * This is sent in a Resource Description to provide information
 * that a user might want to know about the Vsite but has no
 * relevance when supplied as a request.
 * <p>
 * If an InformationResource is included in a Resource Request it is ignored.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version InformationResource.java 1.11 05/13/02 
 *
 **/
public abstract class InformationResource extends Resource {

    static final long serialVersionUID = 1749788655302749772L;

    protected InformationResource() {
        this("","");
    }

    /**
     * Create a new InformationResource
     *
     * @param description A description of the InformationResource
     * @param tag Identifies the nature of the information
     *
     **/
    protected InformationResource (String description, String tag) {
        super (description);
        setTag(tag);
    }

    private String tag ="";
    /**
     * Return the information tag.
     * <p>
     * The Tag identifies the nature of the information. 
     * The unicore
     * clients use it for labeling the information in GUIs.
     *
     **/
    public  String getTag ()           { return tag; }

    /**
     * Change the information tag (to a non-null, non-empty string)j
     * 
     **/
    public  void   setTag (String tag) { 
        if(tag == null) {
            this.tag = "";
        }
        else {
            this.tag = tag;
        }
    }

    /**
     * InformationResources are equal if their tags
     * are equal.
     *
     **/
    public boolean equals(Object o) {
        if(o==null) return false;
        if(o.getClass() != getClass()) return false;
        return getTag().equals(((InformationResource)o).getTag()); // tag CANNOT be null
    }

    /**
     * Return a hash code for this InformationResource. 
     * <p>
     * Returns the hash code
     * of the tag (since the tag determines equality).
     *
     **/
    public int hashCode() {
        return getTag().hashCode(); // tag CANNOT be null
    }

}
//
// Change history:
//          15 Mar 2002 SvdB V4, no arg constructor
//          20022002    SvdB tage never null
//          13 Jul 2000 SvdB override equals
//          21 Jan 2000 DFS  First release with V3.0 resource model. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
