package org.unicore.resources;

/**
 * Resource for the Processors of a system. 
 * <p>
 * A messagge passing systems has a number of Nodes. Each Node
 * has a number of Processors.
 * <p>
 * The numbers in this class refer to the job or site totals.
 * These are assumed to be evenly distributed amongst the
 * Nodes so that the number of processors per node is found
 * by dividing this by the number of Nodes.
 * <p>
 * If no Processor resource is
 * present, then the site default value is used.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version Processor.java 1.11 11/29/02
 *
 **/
public final class Processor extends CapacityResource {

    static final long serialVersionUID = -8051930555786821807L;

    public Processor () {
        this("", 0.0, 0.0, 0.0);
    }

    /**
     * Create a new Processor.
     * <p>
     * Sets the units to "Processors per node"
     *
     * @param description A description of the Processor
     * @param maxrequest Maximum amount of resource allowed
     * @param defaultrequest Amount of resource granted if no specific request received
     * @param minrequest Minimum amount of resource allowed
     *
     **/
    public Processor (String description,
                      double maxrequest, 
                      double defaultrequest,
                      double minrequest) {
        super(description, maxrequest, defaultrequest, minrequest);
        this.units = "Processors per node";
    }

    public String toString() {
        return "Processor";
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor
//          14 Aug 2000 DFS Added minimum request.
//          21 Jan 2000  DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
