package org.unicore.resources;

import org.unicore.ajo.Ticket;

/**
 * Class to advertise quality of service capability. 
 * <p>
 * A  {@link org.unicore.outcome.CheckQoS_Outcome} provides 
 * the user with information about the cost and turn around time of 
 * her job. If this resource is available at a site, the
 * user may send a {@link org.unicore.ajo.CheckQoS} with the desired
 * resource set to the site and obtain a {@link org.unicore.outcome.CheckQoS_Outcome}.
 *
 * When sent to a site, this resource contains a {@link org.unicore.ajo.Ticket}.
 * The Ticket was provided by the site in response to a
 * {@link org.unicore.ajo.CheckQoS} and represents the Site's
 * commitment to run that task given that set of resources with to QoS quoted.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.6
 *
 * @version QoSCheck.java 1.5 05/13/02
 *
 **/
public final class QoSCheck extends ResourceCheck {

    static final long serialVersionUID = -1508985121972433054L;

    /**
     * Create a new QoSCheck
     *
     * @param description A description of the QoSResource
     * @param ticket An AJO's claim on a previously fetched QoS
     *
     **/
    public QoSCheck (String description, Ticket ticket, String units) {
        super (description, ticket);
        this.units = units;
    }

    /**
     * Create a new QoSCheck
     *
     **/
    public QoSCheck() {
        this("QoSCheck", (Ticket) null, "No units provided.");
    }

    public QoSCheck(Ticket ticket) {
        this("QoSCheck", ticket, "No units provided.");
    }

    public QoSCheck(String units) {
        this("QoSCheck", (Ticket) null, units);
    }

    private String units;
    /**
     * Return the units in which cost is measured at this Vsite.
     * Units conversion is possible, but has only this field
     * as a basis. "Exchange Rates" are not yet provided by the 
     * Unicore infrastructure. 
     *
     **/
    public  String getUnits () { return units; }

    /**
     * Set a new value for the cost units.
     * 
     **/
    public  void   setUnits (String units) { this.units = units; }

    /**
     * All QoSCheck are equal. 
     * <p>
     * <em> This may not be expected behaviour.</em>
     *
     **/
    public boolean equals(Object o) {
        try {
            return ( o.getClass() == getClass() );
        }
        catch(NullPointerException nex) {
            return false;
        }
    }

    /**
     * Return a hash code for this QoSCheck. Returns the same hash code
     * for all instances.
     *
     **/
    public int hashCode() {
        return 314159265;
    }

    public String toString() {
        return "QoSCheck ";
    }

}
//
// Change history:
//          13 Jul 2000 SvdB override equals
//          21 Jan 2000 DFS  First release with V3.0 resource model. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
