package org.unicore.resources;

/**
 * This class describes a software capability available at a site.
 * <p>
 * As part of a Resource Description this advertisies that the
 * Vsite supports the named software.
 * <p>
 * As part of a Resource Request this says that the Task uses
 * the named software.
 * <p>
 * Example SoftwareResources are MPI1, PVM, NAG, BLAS. The 
 * appropriate SoftwareResource should be placed in all Tasks
 * that could use the software, including the compile and link steps.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version SoftwareResource.java 1.13 05/13/02
 *
 **/
public class SoftwareResource extends CapabilityResource {

    static final long serialVersionUID = -7385708117424685734L;

    /**
     * Create a new SoftwareResource
     *
     * @param description A description of the SoftwareResource
     * @param name The name of the SoftwareResource
     * @param version The version of the SoftwareResource
     *
     **/
    public SoftwareResource(String description, String name, String version) {
        super(description);
        setName(name);
        setVersion(version);
    }

    /**
     * Create a new SoftwareResource
     *
     **/
    public SoftwareResource() {
        this("SoftwareResource", "", "");
    }

    private String name = "";
    /**
     * Return the name of the software
     *
     **/
    public  String getName () { return name; }

    /**
     * Set the name of the software
     *
     **/
    public  void   setName (String name) { 
        if(name == null) {
            this.name = "";
        }
        else {
            this.name = name; 
        }
    }

    private String version = "";
    /**
     * Get the version number of the software identified by the name.
     *
     **/
    public  String getVersion ()               { return version; }

    /**
     * Set the version number of the software.
     *
     **/
    public  void   setVersion (String version) { 
        if(version == null) {
            this.version = "";
        }
        else {
            this.version = version; 
        }
    }

    /**
     * SoftwareResources are equal if both the name and version information
     * are equal.
     *
     **/
    public boolean equals(Object o) {
        boolean rtn = false;
        if(o != null) {
            if(o.getClass() == getClass()) {
                SoftwareResource s = (SoftwareResource)o;
                // name and version CANNOT be null
                rtn = getName().equals(s.getName()) && getVersion().equals(s.getVersion());
            }
        }
        return rtn;
    }

    /**
     * Return a hash code for this SoftwareResource. 
     * <p>
     * Returns the hash code of the name.
     *
     **/
    public int hashCode() {
        return getName().hashCode();
    }

    public String toString() {
        return getName() + " " + getVersion();
    }

}
//
// Change history:
//          20022002    SvdB name, version never null, fixed equals
//           1 Feb 2002 SvdB Made non-final
//          13 Jul 2000 SvdB override equals
//          21 Jan 2000  DFS First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
