package org.unicore.resources;

/**
 * A storage area under the control of the NJS that can be used
 * to store Portfolios. A Spooled Portfolio is not deleted when the 
 * Uspace of its creating AJO is deleted. A Spooled Portfolio can be copied 
 * into the Uspace of other AJOs.
 * <p>
 * Portfolios can be spooled using {@link org.unicore.ajo.PutPortfolio}, 
 * {@link org.unicore.ajo.GetPortfolio} copies from a previously spooled Portfolio
 * and {@link org.unicore.ajo.DeletePortfolio} deletes a spooled Portfolio.
 * <p>
 * The incarnations of Spooled Portfolios will be created using the Xlogin and
 * project of the creating AJO and so AJOs copying or deleting them must
 * incarnate to the same Xlogin and project.
 * <p>
 * There is at most one Spool storage area in a Vsite
 *
 * @see org.unicore.ajo.Portfolio
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @version Spool.java 1.2 06/19/02
 *
 * @since AJO 4.0
 *
 **/
public class Spool extends Storage {

    static final long serialVersionUID = 0;

    public Spool() {
        this("", 0.0, 0.0, 0.0);
    }

    /**
     * Create a new Spool.
     *
     * @param description A description of the Spool
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     *
     **/
    public Spool (String description, 
                    double maxrequest, 
                    double defaultrequest, 
                    double minrequest ) {
        super (description, maxrequest, defaultrequest, minrequest);
        setName("Spool");
    }

}
//
// Change history:
//          20 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
