package org.unicore.resources;

/**
 * Provide textual information about a site.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version TextInfoResource.java 1.12 05/13/02
 *
 **/
public final class TextInfoResource extends InformationResource {

    static final long serialVersionUID = -4330231964356392731L;

    /**
     * Create a new TextInfoResource
     *
     * @param description A description of the TextInfoResource
     * @param tag Identifies the nature of the information
     * @param value The supplied information
     *
     **/
    public TextInfoResource(String description, String tag, String value) {
        super (description, tag);
        setValue(value);
    }

    /**
     * Create a new TextInfoResource
     *
     **/
    public TextInfoResource() {
        this("undescribed TextInfoResource", "untagged TextInfoResource", "unvalued TextInfoResource :-)");
    }

    private String value = "";
    /**
     * Get the information.
     *
     **/
    public  String getValue ()             { return value; }

    /**
     * Set the information.
     *
     **/
    public  void   setValue (String value) { 
        if(value == null) {
            this.value = "";
        }
        else {
            this.value = value; 
        }
    }

    /**
     * TextInfoResources are equal if both tag and value are equal
     *
     **/
    public boolean equals(Object o) {
        try {
            if(super.equals(o)) {
                return getValue().equals(((TextInfoResource)o).getValue());
            }
            else {
                return false;
            }
        }
        catch(NullPointerException nex) {
            return false;
        }
    }

    /**
     * Return a hash code for this TextInfoResource. 
     * <p>
     * Returns the hash code of the tag.
     *
     **/
    public int hashCode() {
        return getTag().hashCode();
    }

    public String toString() {
        return getTag() + ": " + getValue();
    }

}
//
// Change history:
//          20022002     SvdB value is never null
//          21 Jan 2000  DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
