package org.unicore.resources;

/**
 * Resource for access to the file space containing the parent AJO's Uspace.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version USpace.java 1.11 06/11/02 
 *
 **/
public final class USpace extends PathedStorage {

    static final long serialVersionUID = 7456373442298745630L;

    public USpace() {
        this("", 0.0, 0.0, 0.0, (String)null);
    }

    /**
     * Create a new USpace.
     *
     * @param description A description of the USpace
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param subspace The storage subspace
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    public USpace (String description, 
                   double maxrequest, 
                   double defaultrequest, 
                   double minrequest,
                   SubspaceValue subspace) {
        this(description, maxrequest, defaultrequest, minrequest, (String)null);
        setSubspace(subspace);
    }

    /**
     * Create a new USpace.
     *
     * @param description A description of the USpace
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param path Path to a subdirectory within the USpace (relative to the root of the Uspace)
     *
     * @since AJO 4.0
     *
     **/
    public USpace (String description,
                 double maxrequest,
                 double defaultrequest, 
                 double minrequest,
                 String path) {
        super (description, maxrequest, defaultrequest, minrequest, path);
        setName("USpace");
    }

}
//
// Change history:
//          19 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
