package org.unicore.upl;

import org.unicore.Vsite;

/**
 * A ListVsitesReply contains a list of the Vsites at a Usite.
 * <p>
 * A ListVsitesReply is returned in response to a 
 * ListVsites. It contains a list of the Vsites that the Gateway
 * provides access to and, optionally, some further information
 * about these Vsites. 
 *
 * A ListVsitesReply is generated by the Gateway and not passed
 * to a Vsite.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9 (AJO 1 in other versions)
 *
 * @version ListVsitesReply.java 1.4 05/13/02  
 * 
 **/

public final class ListVsitesReply extends Reply {

    /**
     * Create an empty ListVsitesReply.
     *
     **/
    public ListVsitesReply() {
    }

    /**
     * Create a ListVsitesReply with the list of Vsites.
     *
     **/
    public ListVsitesReply(Vsite[] vsites) {
        this.vsites = vsites;
    }

    private Vsite[] vsites;

    /**
     * Return the list of Vsites for this Usite.
     *
     **/
    public Vsite[] getList() {return vsites;}

    /**
     * Set the list of Vsites for this Usite.
     *
     **/
    public void setList(Vsite[] vsites) {this.vsites = vsites;}

    private byte[][] further_information;

    /**
     * Return the extra information about the Vsites (optional). 
     * <p>
     * This array should parallel the Vsite array.
     *
     **/
    public byte[][] getFurtherInformation() {return further_information;}

    /**
     * Set the extra information about the Vsites.
     *
     **/
    public void setFurtherInformation(byte[][] further_information) {
        this.further_information = further_information;
    }

    static final long serialVersionUID = 904441927754426760L;

}
//
// Change history:
//           4 Feb 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
