package org.unicore.upl;

import org.unicore.Vsite;
import org.unicore.Ulogin;
import org.unicore.AJOIdentifier;

/**
 * A ServerRequest is a {@link Request} that
 * must be passed on to a Unicore server for processing.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9
 *
 * @version ServerRequest.java 1.4 05/13/02  
 * 
 **/
public abstract class ServerRequest extends Request {

    /**
     * Create an uninitialised ServerRequest
     *
     **/
    public ServerRequest() {
        this(null, null, null);
    }

    /**
     * Create a ServerRequest
     *
     * @param target The AbstractJob that is the subject of this Request.
     * @param vsite  Where the AJO is executing
     *
     **/
    public ServerRequest(AJOIdentifier target,
                         Vsite vsite) {
        this(target, vsite, null);
    }

    /**
     * Create a ServerRequest
     *
     * @param target The AbstractJob that is the subject of this Request.
     * @param vsite  Where the AJO is executing
     * @param sso    The site security object
     *
     **/
    public ServerRequest(AJOIdentifier target,
                         Vsite vsite,
                         byte[] sso) {
        this.target = target;
        this.vsite  = vsite;
        this.sso    = sso;
    }

    private Vsite vsite;
    /**
     * Return the Vsite to which the request is to 
     * be passed.
     *
     **/
    public Vsite getVsite() {return vsite;}

    /**
     * Set the Vsite to which the request is to 
     * be passed.
     *
     **/
    public void setVsite(Vsite vsite) {this.vsite = vsite;}

    private AJOIdentifier target;
    /**
     * Return the Identifier of the AJO which this request
     * is about.
     *
     **/
    public AJOIdentifier getTarget() {return target;}

    /**
     * Set the Identifier of the AJO which this request
     * is about.
     *
     **/
    public void setTarget(AJOIdentifier target) {this.target = target;}

    private Ulogin consignor;
    /**
     * Return the client who consigned this request to the Vsite.
     * <p>
     * The Unicore user or NJS who consigned the Request
     * to a Gateway. This field is filled in by the Gateway
     * using information gained from the underlying transport
     * (for example the SSL connection).
     *
     **/
    public Ulogin getConsignor() {return consignor;}

    /**
     * Set the client who consigned this request to the Vsite.
     *
     **/
    public void setConsignor(Ulogin consignor) {this.consignor = consignor;}

    private byte[] sso;
    /**
     * Return the SSO.
     * <p>
     * The SSO is space for security conscious sites to place
     * information for additional authentication/authorisation
     *
     **/
    public byte[] getSSO() {return sso;}

    /**
     * Set the SSO.
     *
     **/
    public void setSSO(byte[] sso) {this.sso = sso;}

    // NOTE The Gateway fills this in but the creator calls the
    //      constructor on the instance (the Gateway
    //      deserialises it). So, this field is not
    //      set in the constructor.

    static final long serialVersionUID = 0;

}
//
// Change history:
//          15 Mar 2002, SvdB V4 Changes, moved SSO to here
//           4 May 2000, SvdB V3 mods 
//           4 Feb 2000, SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
