package org.unicore.utility;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.ConditionalAction;

import java.util.Enumeration;

/**
 * Give new identities to the AbstractAction and any child or branch actions by
 * setting new {@link org.unicore.Identifier} values.
 * <p>
 * This needs to be done if an AJO is used
 * by a JPA as a template from which different consigns are made.
 * A requirement is that every AbstractAction has a unique identifier
 * but its Identifier value is only set when its constructor is called. So
 * consigning a "template" multiple times could violate this.
 * <p>
 * This does not change the Identifiers of Portfolios (this is OK
 * as Portfolios are always addressed via their AJO)
 *
 * @see org.unicore.upl.ConsignJob
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version ResetIdentifiers.java 1.5 07/11/02 
 * 
 **/

public final class ResetIdentifiers {

    public static void doit(AbstractAction aa) {

        if(aa != null) {

            aa.getId().changeIdentity();

            if(aa instanceof ActionGroup) {
                Enumeration actions = ((ActionGroup) aa).getActions();
                while(actions.hasMoreElements()) {
                    doit((AbstractAction)actions.nextElement());
                }
            }
            else if(aa instanceof ConditionalAction) {
                doit(((ConditionalAction)aa).getTrueBranch());
                doit(((ConditionalAction)aa).getFalseBranch());
            }
        }

    }
}
//
// Change history:
//          10 Jul 2002 SvdB Do ConditionalAction branches too
//          28 Feb 2001 SvdB Do the AJO!, apply to ActionGroup
//           5 May 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
