package org.unicore.utility;

import java.io.IOException;

/**
 * A version of java.util.zip.ZIPOutputStream that
 * ensures that an underlying PacketisedOutputStream
 * properly writes a final packet when the ZipOutputStream
 * is finished. 
 * 
 * @see org.unicore.utility.PacketisedOutputStream
 *
 * @author Sven van den Berghe (fecit) 
 *
 * @since AJO 3
 *
 * @version ZipOutputStream.java 1.3 05/13/02 
 *
 **/
public class ZipOutputStream extends java.util.zip.ZipOutputStream {

    private PacketisedOutputStream output;

    public ZipOutputStream(PacketisedOutputStream output) {
        super(output);
        this.output = output;
    }

    /**
     * Calls super-class finish() and also ensures that
     * a final packet is written.
     *
     **/
    public void finish() throws IOException {
        super.finish();
        output.finish(); // write a final packet, does not close underlying
    }

}
//
// Change history:
//          26 Sep 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
